/**
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.domino.admin;

import java.awt.Component;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.Serializable;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.JCheckBox;
import javax.swing.SwingConstants;
import javax.swing.border.TitledBorder;

import org.apache.commons.lang.StringUtils;

import com.funambol.admin.AdminException;
import com.funambol.admin.ui.ConnectorManagementPanel;
import java.awt.Dimension;


/**
 * This class implements the connector configuration panel
 * for a Domino Connector component
 *
 *
 * @version $Id: DominoConnectorConfigPanel.java,v 1.1 2006/08/08 08:29:26 gibi Exp $
 */
public class DominoConnectorConfigPanel
               extends ConnectorManagementPanel
               implements Serializable {

    // --------------------------------------------------------------- Constants

    public static final String NAME_ALLOWED_CHARS =
            "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890-_.";

    // ------------------------------------------------------------ Private data

    private JPanel serverPanel   = new JPanel();
    private JPanel funambolPanel = new JPanel();

    private JLabel     hostLabel = new JLabel();
    private JTextField hostValue = new JTextField();
    private JLabel     sslLabel = new JLabel();
    private JCheckBox  sslValue = new JCheckBox();
    private JLabel     portLabel = new JLabel();
    private JTextField portValue = new JTextField();

    private JLabel     dataSourceLabel = new JLabel();
    private JTextField dataSourceValue = new JTextField();

    private JButton confirmButton = new JButton();

    // ------------------------------------------------------------ Constructors


    public DominoConnectorConfigPanel() {
      init();
    }


    // --------------------------------------------------------- Private methods

    /**
     * Create the panel
     * @throws Exception if error occures during creation of the panel
     */
    private void init() {

        int yy = 0;
        JLabel title = new JLabel();
        setLayout(null);

        yy += 5;
        title.setFont(titlePanelFont);
        title.setText("Funambol Domino Connector");
        title.setBounds(new Rectangle(14, yy, 316, 28));
        title.setAlignmentX(SwingConstants.CENTER);
        title.setBorder(new TitledBorder(""));

        serverPanel.setLayout(null);
        serverPanel.setBorder(new TitledBorder("Officer Configuration"));

        yy += 15;
        hostLabel.setText("BackEnd Server:");
        serverPanel.add(hostLabel);
        hostLabel.setBounds(10, yy, 116, 15);
        serverPanel.add(hostValue);
        hostValue.setBounds(150, yy, 220, 19);
        hostValue.setFont(defaultFont);

        yy += 30;
        portLabel.setText("Port:");
        serverPanel.add(portLabel);
        portLabel.setBounds(10, yy, 131, 15);
        serverPanel.add(portValue);
        portValue.setBounds(150, yy, 220, 19);
        portValue.setFont(defaultFont);

        yy += 30;
        sslLabel.setText("SSL:");
        serverPanel.add(sslLabel);
        sslLabel.setBounds(new Rectangle(10, yy, 150, 19));
        sslValue.setSelected(false);
        serverPanel.add(sslValue);
        sslValue.setBounds(150, yy, 220, 19);

        add(serverPanel);
        serverPanel.setBounds(10, 50, 380, 110);

        funambolPanel.setLayout(null);
        funambolPanel.setBorder(new TitledBorder("Funambol Configuration"));

        dataSourceLabel.setText("Funambol DataSource:");
        funambolPanel.add(dataSourceLabel);
        dataSourceLabel.setBounds(10, 20, 131, 15);
        funambolPanel.add(dataSourceValue);
        dataSourceValue.setBounds(150, 20, 220, 19);
        dataSourceValue.setFont(defaultFont);

        add(funambolPanel);
        funambolPanel.setBounds(10, 170, 380, 50);


        confirmButton.setFont(defaultFont);
        confirmButton.setText("Save");
        confirmButton.setBounds(160, 330, 70, 25);

        confirmButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent event ) {
                try {
                    validateValues();
                    getValues();
                    DominoConnectorConfigPanel.this.actionPerformed(
                             new ActionEvent(DominoConnectorConfigPanel.this,
                                             ACTION_EVENT_UPDATE,
                                             event.getActionCommand()));
                } catch (Exception e) {
                    notifyError(new AdminException(e.getMessage()));
                }
            }
        });

        add(confirmButton, null);

        Component[] components = getComponents();
        for (int i=0; (components != null) && (i<components.length); ++i) {
            components[i].setFont(defaultFont);
        }
        add(title);
  }

  /**
   * Set preferredSize of the panel
   * @return preferredSize of the panel
   */
  public Dimension getPreferredSize() {
      return new Dimension(525, 380);
  }

  /**
   *
   */
  public void updateForm() {

      DominoConnectorConfig dominoConf = (DominoConnectorConfig)getConfiguration();

      hostValue.setText(dominoConf.getHost());

      if (dominoConf.getSsl()) {
          sslValue.setSelected (true) ;
      } else {
          sslValue.setSelected (false) ;
      }

      portValue.setText(""+dominoConf.getPort());

      dataSourceValue.setText(dominoConf.getDataSource());

  }

  // ----------------------------------------------------------- Private methods

  /**
   * Checks if the values provided by the user are all valid. In caso of errors,
   * a IllegalArgumentException is thrown.
   *
   * @throws IllegalArgumentException if:
   *         <ul>
   *         <li>anyvalue is not in the form <host>:<port>
   *         </ul>
   */
  private void validateValues() throws IllegalArgumentException {
      String value = null;

      // in host
      value = hostValue.getText().trim();
      if (StringUtils.isEmpty(value)) {
          throw new IllegalArgumentException("Field 'BackEnd Server' cannot be empty");
      }

      // port
      value = portValue.getText().trim();
      if (StringUtils.isEmpty(value)) {
          throw new IllegalArgumentException("Field 'Port' cannot be empty");
      }
      try {
          Integer.parseInt(value);
      } catch (NumberFormatException e) {
          throw new IllegalArgumentException("Field 'Port' must be numeric");
      }


      // datasource
      value = dataSourceValue.getText().trim();
      if (StringUtils.isEmpty(value)) {
          throw new IllegalArgumentException("Field 'Funambol DataSource' cannot be empty");
      }


  }



  /**
   * Set properties with the values provided by the user.
   */
  private void getValues() {

      DominoConnectorConfig dominoConf = (DominoConnectorConfig)getConfiguration();

      dominoConf.setHost(hostValue.getText().trim());
      if (sslValue.isSelected()) {
         dominoConf.setSsl(true) ;
      } else {
         dominoConf.setSsl(false) ;
      }

      dominoConf.setPort(Integer.parseInt(portValue.getText().trim()));

      dominoConf.setDataSource(dataSourceValue.getText().trim());

  }

}
