/**
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.domino.admin ;

import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.Serializable;
import java.util.StringTokenizer;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import javax.swing.border.TitledBorder;

import org.apache.commons.lang.StringUtils;

import com.funambol.framework.engine.source.ContentType;
import com.funambol.framework.engine.source.SyncSourceInfo;

import com.funambol.admin.AdminException;
import com.funambol.admin.mo.SyncSourceManagementObject;
import com.funambol.admin.ui.SourceManagementPanel;

import com.funambol.domino.engine.source.DominoCalendarSyncSource;
import com.funambol.domino.engine.source.DominoContactSyncSource;
import com.funambol.domino.engine.source.DominoSyncSource;
import com.funambol.domino.engine.source.DominoTaskSyncSource;
import com.funambol.domino.util.Def;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import javax.swing.JCheckBox;


/**
 * This class implements the configuration panel for an DominoSyncSource
 *
 * @author Gilberto Migliavacca
 * @version $Id: DominoSyncSourceConfigPanel.java,v 1.1 2006/08/08 08:29:26 gibi Exp $
 */
public class DominoSyncSourceConfigPanel
                              extends SourceManagementPanel
                              implements Serializable {

  // ----------------------------------------------------------------- Constants

  public static final String NAME_ALLOWED_CHARS =
    "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890-_."     ;
  public static final String PORT_ALLOWED_CHARS =
    "0123456789"                                                            ;


  // -------------------------------------------------------------- Private data
  /** label for the panel's name */
  private JLabel             panelName = new JLabel();

  /** border to evidence the title of the panel */
  private TitledBorder       titledBorder1;
  private JLabel             nameLabel            = new JLabel        () ;
  private JTextField         nameValue            = new JTextField    () ;
  private JLabel             sourceUriLabel       = new JLabel        () ;
  private JTextField         sourceUriValue       = new JTextField    () ;
  private JLabel             hostLabel            = new JLabel        () ;
  private JTextField         hostValue            = new JTextField    () ;
  private JLabel             portLabel            = new JLabel        () ;
  private JTextField         portValue            = new JTextField    () ;
  private JLabel             conversionTypeLabel  = new JLabel        () ;
  private JComboBox          conversionValue      = new JComboBox     () ;
  private JLabel             dtEncryptLabel       = new JLabel        () ;
  private JCheckBox          dtEncryptValue       = new JCheckBox     () ;
  private JLabel             dtEncodeLabel        = new JLabel        () ;
  private JCheckBox          dtEncodeValue        = new JCheckBox     () ;    
  private JButton            confirmButton        = new JButton       () ;


  // ----------------------------------------------------------- Constructors

  public DominoSyncSourceConfigPanel() {
      init();
  }

  // ----------------------------------------------------------- Private methods
  /**
   * Create the panel
   * @throws Exception if error occures during creation of the panel
   */

  private void init(){
    // set layout
    this.setLayout(null);

    // set properties of label, position and border
    //  referred to the title of the panel
    titledBorder1 = new TitledBorder("");

    panelName.setFont(titlePanelFont);
    panelName.setText("Edit Domino SyncSource");
    panelName.setBounds(new Rectangle(14, 5, 316, 28));
    panelName.setAlignmentX(SwingConstants.CENTER);
    panelName.setBorder(titledBorder1);

    sourceUriLabel.setText("Source URI: ");
    sourceUriLabel.setFont(defaultFont);
    sourceUriLabel.setBounds(new Rectangle(14, 60, 150, 18)        );
    sourceUriValue.setFont(defaultFont);
    sourceUriValue.setBounds(new Rectangle(170, 60, 350, 18)       );

    nameLabel.setText("Name: ");
    nameLabel.setFont(defaultFont);
    nameLabel.setBounds(new Rectangle(14, 90, 150, 18)             );
    nameValue.setFont(defaultFont);
    nameValue.setBounds(new Rectangle(170, 90, 350, 18)            );

    hostLabel.setText("Domino Server: ");
    hostLabel.setFont(defaultFont);
    hostLabel.setBounds(new Rectangle(14, 120, 150, 18)            );
    hostValue.setFont(defaultFont);
    hostValue.setBounds(new Rectangle(170, 120, 350, 18)           );

    portLabel.setText("Domino Port: ");
    portLabel.setFont(defaultFont);
    portLabel.setBounds(new Rectangle(14, 150, 150, 18)            );
    portValue.setFont(defaultFont);
    portValue.setBounds(new Rectangle(170, 150, 350, 18)           );

    
    conversionTypeLabel.setText("Type: ");
    conversionTypeLabel.setFont(defaultFont);
    conversionTypeLabel.setBounds(new Rectangle(14, 180, 150, 18)  );
    conversionValue.setFont(defaultFont);
    conversionValue.setBounds(new Rectangle(170, 180, 80, 18)   );

    conversionValue.addItemListener(new ItemListener() {
        public void itemStateChanged(ItemEvent ie) {
            if (conversionValue.getSelectedIndex() == 0){
               dtEncryptValue.setSelected(false);
               dtEncodeValue.setSelected(true);
            } else {
               dtEncryptValue.setSelected(false);
               dtEncodeValue.setSelected(false);
            }   
        }
    });

    
    dtEncryptLabel.setText("encryption: ");
    dtEncryptLabel.setFont(defaultFont);
    dtEncryptLabel.setBounds(new Rectangle(170, 210, 70, 18));
    dtEncryptValue.setSelected(false);
    dtEncryptValue.setBounds(new Rectangle(250, 210, 25, 19));
  
    dtEncodeLabel.setText("encoding: ");
    dtEncodeLabel.setFont(defaultFont);
    dtEncodeLabel.setBounds(new Rectangle(280, 210, 70, 18));
    dtEncodeValue.setSelected(false);
    dtEncodeValue.setBounds(new Rectangle(360, 210, 25, 19));
    dtEncodeValue.setEnabled(false);
    
    dtEncryptValue.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
            if (e.getSource().equals(dtEncryptValue)) {
                if (dtEncryptValue.isSelected()){
                   dtEncodeValue.setSelected(true);                   
                } else {
                    if (conversionValue.getSelectedIndex() == 1){
                       dtEncodeValue.setSelected(false);                   
                    } else {
                       dtEncodeValue.setSelected(true);                   
                    }
                }   
            }
        }
    });
            
    confirmButton.setFont(defaultFont);
    confirmButton.setText("Add");
    confirmButton.setBounds(170, 270, 70, 25);

    confirmButton.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent event ) {
            try {
                validateValues();
                getValues();
                if (getState() == STATE_INSERT) {
                    DominoSyncSourceConfigPanel.this.actionPerformed
                            (new ActionEvent(DominoSyncSourceConfigPanel.this, 
                                             ACTION_EVENT_INSERT, 
                                             event.getActionCommand()));
                } else {
                    DominoSyncSourceConfigPanel.this.actionPerformed(
                            new ActionEvent(DominoSyncSourceConfigPanel.this, 
                                            ACTION_EVENT_UPDATE, 
                                            event.getActionCommand()));
                }
            } catch (Exception e) {
                notifyError(new AdminException(e.getMessage()));
            }
        }
    });





    // add all components to the panel
    this.add(panelName            , null ) ;
    this.add(sourceUriLabel       , null ) ;
    this.add(sourceUriValue       , null ) ;
    this.add(nameLabel            , null ) ;
    this.add(nameValue            , null ) ;
    this.add(hostLabel            , null ) ;
    this.add(hostValue            , null ) ;
    this.add(portLabel            , null ) ;
    this.add(portValue            , null ) ;
    this.add(conversionTypeLabel  , null ) ;
    this.add(conversionValue      , null ) ;
    this.add(dtEncryptLabel       , null ) ;
    this.add(dtEncryptValue       , null ) ;
    this.add(dtEncodeLabel        , null ) ;
    this.add(dtEncodeValue        , null ) ;
    this.add(confirmButton        , null ) ;

  }

  /**
   *
   *
   */
  public void updateForm() {

      if (!(getSyncSource() instanceof DominoSyncSource)) {
          error("This is not a DominoSyncSource!"  +
          "Unable to process SyncSource values."     );
          return;
      }

      if (getState() == STATE_INSERT) {
          confirmButton.setText("Add");
      } else if (getState() == STATE_UPDATE) {
          confirmButton.setText("Save");
      }

      DominoSyncSource syncSource = (DominoSyncSource) getSyncSource();

      sourceUriValue.setText(syncSource.getSourceURI());
      nameValue.setText(syncSource.getName());
      hostValue.setText(syncSource.getHost());
      portValue.setText(String.valueOf(syncSource.getPort()));

      conversionValue.removeAllItems();

      if (syncSource instanceof DominoCalendarSyncSource)  {
        conversionValue.addItem ("SIF-E" );
        conversionValue.addItem ("iCal"  );
      } else if (syncSource instanceof DominoContactSyncSource) {
        conversionValue.addItem ("SIF-C" );
        conversionValue.addItem ("vCard" );
      } else if (syncSource instanceof DominoTaskSyncSource) {
        conversionValue.addItem ("SIF-T" );
      }

      if ((syncSource instanceof DominoCalendarSyncSource) ||
          (syncSource instanceof DominoContactSyncSource)) {

          String cType = ((DominoSyncSource) syncSource).getType();

          if (Def.VCARD_TYPE.equals(cType) || Def.ICAL_TYPE.equals(cType)) {
              conversionValue.setSelectedIndex(1);
          } else {
              conversionValue.setSelectedIndex(0);
          }
      }

      SyncSourceManagementObject mo = (SyncSourceManagementObject)getManagementObject();
      String transformationsRequired = mo.getTransformationsRequired();
      //notifyError(new AdminException("TransformationsRequired: " + transformationsRequired));
      if (transformationsRequired == null){
          dtEncryptValue.setSelected(false);
          dtEncodeValue.setSelected(false);
      } else {
          if(transformationsRequired.equals("des;b64")){
              dtEncryptValue.setSelected(true);
              dtEncodeValue.setSelected(true);
          } else if (transformationsRequired.equals("b64")){
              dtEncryptValue.setSelected(false);
              dtEncodeValue.setSelected(true);
          } else {
              dtEncryptValue.setSelected(false);
              dtEncodeValue.setSelected(false);
          }
      }
      if (conversionValue.getSelectedIndex() == 0){
         dtEncodeValue.setSelected(true);
      }       
      
      
      
      if (syncSource.getSourceURI() != null) {
          sourceUriValue.setEditable(false);
      }
  }


  // ----------------------------------------------------------- Private methods

  /**
   * Checks if the values provided by the user are all valid. In caso of errors,
   * a IllegalArgumentException is thrown.
   *
   * @throws IllegalArgumentException if:
   *         <ul>
   *         <li>name, uri, type or directory are empty (null or zero-length)
   *         <li>the types list length does not match the versions list length
   *         </ul>
   */
  private void validateValues() throws IllegalArgumentException {
      String value = null;

      value = sourceUriValue.getText();
      if (StringUtils.isEmpty(value)) {
          throw new
            IllegalArgumentException(
                "Field 'Source URI' cannot be empty. "  +
                "Please provide a SyncSource URI."      );
      }

      value = nameValue.getText();
      if (StringUtils.isEmpty(value)) {
          throw new
            IllegalArgumentException(
                "Field 'Name' cannot be empty. "    +
                "Please provide a SyncSource name." );
      }

      if (!StringUtils.containsOnly(value, NAME_ALLOWED_CHARS.toCharArray())) {
          throw new
            IllegalArgumentException(
                "Only the following characters are "   +
                "allowed for field 'Name': \n"         +
                NAME_ALLOWED_CHARS                     );
      }

      value = hostValue.getText();
      if (StringUtils.isEmpty(value)) {
          throw new
          IllegalArgumentException(
              "Field 'Domino Server' cannot be empty. "     +
              "Please provide a SyncSource host."  );
      }

      value = portValue.getText();
      if (StringUtils.isEmpty(value)) {
          throw new
            IllegalArgumentException(
                "Field 'Domino Port' cannot be empty. "     +
                "Please provide a SyncSource port."  );
      }

      if (!StringUtils.containsOnly(value, PORT_ALLOWED_CHARS.toCharArray())) {
          throw new
            IllegalArgumentException(
                "Only the following characters are " +
                "allowed for field 'Domino Port': \n"       +
                PORT_ALLOWED_CHARS                   );
      }
  }

  /**
   * Set syncSource properties with the values provided by the user.
   */
  private void getValues() {

        StringTokenizer types    = null ;
        StringTokenizer versions = null ;

        DominoSyncSource syncSource = (DominoSyncSource)getSyncSource();

        if (syncSource instanceof DominoCalendarSyncSource) {
            ((DominoSyncSource) syncSource).setViewName(Def.CALENDAR_VIEW);
        } else if (syncSource instanceof DominoContactSyncSource) {
            ((DominoSyncSource) syncSource).setViewName(Def.CONTACT_VIEW);
        } else if (syncSource instanceof DominoTaskSyncSource) {
            ((DominoSyncSource) syncSource).setViewName(Def.TODO_VIEW);
        }

        syncSource.setSourceURI(sourceUriValue.getText().trim() ) ;
        syncSource.setName(nameValue.getText().trim() ) ;
        syncSource.setHost(hostValue.getText().trim() ) ;
        syncSource.setPort(Integer.parseInt(portValue.getText ().trim())) ;

        if (conversionValue.getSelectedIndex() == 0) {

            if (syncSource instanceof DominoCalendarSyncSource) {
                syncSource.setType(Def.SIFE_TYPE);
                types = new StringTokenizer(Def.SIFE_TYPES       , "," ) ;
                versions = new StringTokenizer(Def.SIFE_VERSIONS , "," ) ;
            } else if (syncSource instanceof DominoContactSyncSource ) {
                syncSource.setType(Def.SIFC_TYPE);
                types = new StringTokenizer(Def.SIFC_TYPES       , "," ) ;
                versions = new StringTokenizer(Def.SIFC_VERSIONS , "," ) ;
            } else if (syncSource instanceof DominoTaskSyncSource ) {
                syncSource.setType(Def.SIFT_TYPE);
                types = new StringTokenizer(Def.SIFT_TYPES       , "," ) ;
                versions = new StringTokenizer(Def.SIFT_VERSIONS , "," ) ;
            }


        } else if (syncSource instanceof DominoCalendarSyncSource ||
                   syncSource instanceof DominoTaskSyncSource) {

            syncSource.setType(Def.ICAL_TYPE);
            types = new StringTokenizer(Def.ICAL_TYPES       , "," ) ;
            versions = new StringTokenizer(Def.ICAL_VERSIONS , "," ) ;
        } else if (syncSource instanceof DominoContactSyncSource ) {

            syncSource.setType(Def.VCARD_TYPE);
            types = new StringTokenizer(Def.VCARD_TYPES       , "," ) ;
            versions = new StringTokenizer(Def.VCARD_VERSIONS , "," ) ;
        } 

        
        boolean dtencr = dtEncryptValue.isSelected();
        boolean dtenco = dtEncodeValue.isSelected();        
        String transformationsRequired = null;    
        if(dtencr){
           transformationsRequired = "des;b64"; 
        } else if (!dtencr && dtenco){
           transformationsRequired = "b64"; 
        } else {
           transformationsRequired = null; 
        }
        SyncSourceManagementObject mo = (SyncSourceManagementObject)getManagementObject();
        mo.setTransformationsRequired(transformationsRequired);                                

        
        ContentType[] contentTypes= new ContentType[types.countTokens()];

        for(int i = 0, l = contentTypes.length; i < l; ++i) {
          contentTypes[i] = new ContentType(types.nextToken().trim()    ,
                                            versions.nextToken().trim() );
        }

        syncSource.setInfo(new SyncSourceInfo(contentTypes, 0));
  }

  private void error(String msg) {
      JOptionPane.showMessageDialog(null, msg, "Error", JOptionPane.ERROR_MESSAGE);
  }

}
