/**
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */

package com.funambol.domino.document;

import com.funambol.domino.exception.DominoDocumentException;
import com.funambol.domino.util.Def;
import com.funambol.framework.logging.Sync4jLogger;

import java.text.SimpleDateFormat;
import java.util.*;
import java.util.logging.Logger;

import lotus.domino.Database;
import lotus.domino.DateTime;
import lotus.domino.Document;
import lotus.domino.NotesException;
import lotus.domino.Session;

/**
 *
 * @version $Id: DominoDocument.java,v 1.2 2006/08/17 08:33:26 adeac Exp $
 */
public abstract class DominoDocument {

    // ------------------------------------------------------------ Private data

    protected Logger log = Sync4jLogger.getLogger(Def.LOGGER_NAME);

    //-------------------------------------------------------------_ Public data
    //Recurring document properties names
    public static final String CS_FLAGS = "$CSFlags";
    public static final String TXT_NUM = "txtNum";
    public static final String REPEATS = "Repeats";
    public static final String ORG_REPEAT = "OrgRepeat";
    public static final String REPEAT_UNIT = "RepeatUnit";
    public static final String REPEAT_INTERVAL = "RepeatInterval";
    public static final String REPEAT_ADJUST = "RepeatAdjust";
    public static final String REPEAT_HOW = "RepeatHow";
    public static final String REPEAT_FOR = "RepeatFor";
    public static final String REPEAT_FOR_UNIT = "RepeatForUnit";
    public static final String REPEAT_WEEKENDS = "RepeatWeekends";
    public static final String REPEAT_INSTANCE_DATES = "RepeatInstanceDates";

    public static final String REPEAT_CUSTOM = "RepeatCustom";
    public static final String REPEAT_UNTIL = "RepeatUntil";
    public static final String REPEAT_START_DATE = "RepeatStartDate";
    public static final String REPEAT_DATES = "RepeatDates";
    public static final String REPEAT_END_DATES = "RepeatEndDates";
    public static final String END_DATE_TIME = "EndDateTime";
    public static final String START_DATE_TIME = "STARTDATETIME";
    public static final String CALENDAR_DATE_TIME = "CalendarDateTime";
    public static final String START_DATE = "StartDate";
    public static final String START_TIME = "StartTime";
    public static final String END_DATE = "EndDate";
    public static final String END_TIME = "EndTime";
    public static final String NO_PURGE = "$NoPurge";
    public static final String REF = "$REF";
    public static final String REF_OPTIONS = "$RefOptions";

    public static final String ATTACHMENT_NAMES = "h_AttachmentNamesAlt";
    public static final String ATTACHMENT_LENGTH = "h_AttachmentLengthsAlt";
    public static final String PUBLIC_ACCESS = "$PublicAccess";
    public static final String BUSY_PRIORITY = "$BusyPriority";
    public static final String PREVENT_REPLIES = "$PreventReplies";
    public static final String EXCLUDE_FROM_VIEW = "ExcludeFromView";
    public static final String SEQUENCE_NUM = "SequenceNum";
    public static final String UPDATE_SEQ = "UpdateSeq";
    public static final String CSWISL = "$CSWISL";
    public static final String CS_VERSION = "$CSVersion";
    public static final String ORG_TABLE = "OrgTable";
    public static final String VIEW_ICON = "_ViewIcon";
    public static final String APPT_UNID = "ApptUNID";
    public static final String APPOINTMENT_TYPE = "AppointmentType";
    public static final String CHAIR = "Chair";
    public static final String ALT_CHAIR = "AltChair";
    public static final String ALT_PRINCIPAL = "$AltPrincipal";

    protected Document document = null;
    private Database database = null;
    private Session session = null;
    protected TimeZone timeZone;

    /**
     * Creates a new instance of Recurrence
     */
    public DominoDocument(Document document, TimeZone timeZone) throws DominoDocumentException {

        if (document == null) {
            throw new DominoDocumentException("Document parameter can not be null!");
        }
        try {

            this.session = document.getParentDatabase().getParent();
            //this.setDatabase(session.getDbDirectory(session.getServerName()).openMailDatabase());
            this.database = document.getParentDatabase();
        } catch (NotesException ex) {
            throw new DominoDocumentException(ex);
        }

        this.document = document;
        this.timeZone = timeZone;
    }

    public Document getDocument() {
        return document;
    }

    public TimeZone getTimeZone() {
        return timeZone;
    }

    public void setTimeZone(TimeZone timeZone) {
        this.timeZone = timeZone;
    }

    public String getCsFlags() throws NotesException {
        return document.getItemValueString(CS_FLAGS);
    }

    public void setCsFlags(String csFlags) throws NotesException {
        if (getCsFlags() != null)
            document.replaceItemValue(CS_FLAGS, csFlags);
        else
            document.appendItemValue(CS_FLAGS, csFlags);
    }

    public String getChair() throws NotesException {
        return document.getItemValueString(CHAIR);
    }

    public void setChair(String chair) throws NotesException {
        if (getChair() != null)
            document.replaceItemValue(CHAIR, chair);
        else
            document.appendItemValue(CHAIR, chair);
    }

    public String getAltChair() throws NotesException {
        return document.getItemValueString(ALT_CHAIR);
    }

    public void setAltChair(String altChair) throws NotesException {
        if (getAltChair() != null)
            document.replaceItemValue(ALT_CHAIR, altChair);
        else
            document.appendItemValue(ALT_CHAIR, altChair);
    }

    public String getAltPrincipal() throws NotesException {
        return document.getItemValueString(ALT_PRINCIPAL);
    }

    public void setAltPrincipal(String altPrincipal) throws NotesException {
        if (getAltPrincipal() != null)
            document.replaceItemValue(ALT_PRINCIPAL, altPrincipal);
        else
            document.appendItemValue(ALT_PRINCIPAL, altPrincipal);
    }

    public String getRepeatCustom() throws NotesException {
        return document.getItemValueString(REPEAT_CUSTOM);
    }

    public void setRepeatCustom(String custom) throws NotesException {
        if (getRepeatCustom() != null)
            document.replaceItemValue(REPEAT_CUSTOM, custom);
        else
            document.appendItemValue(REPEAT_CUSTOM, custom);
    }

    public String getTxtNum() throws NotesException {
        return document.getItemValueString(TXT_NUM);
    }

    public void setTxtNum(String txtNum) throws NotesException {
        if (getTxtNum() != null)
            document.replaceItemValue(TXT_NUM, txtNum);
        else
            document.appendItemValue(TXT_NUM, txtNum);
    }

    public String getApptUNID() throws NotesException {
        return document.getItemValueString(APPT_UNID);
    }

    public void setApptUNID(String uid) throws NotesException {
        if (getApptUNID() != null)
            document.replaceItemValue(APPT_UNID, uid);
        else
            document.appendItemValue(APPT_UNID, uid);
    }

    public String getAppointmentType() throws NotesException {
        return document.getItemValueString(APPOINTMENT_TYPE);
    }

    public void setAppointmentType(String type) throws NotesException {
        if (getAppointmentType() != null)
            document.replaceItemValue(APPOINTMENT_TYPE, type);
        else
            document.appendItemValue(APPOINTMENT_TYPE, type);
    }

    public String getRepeats() throws NotesException {
        return document.getItemValueString(REPEATS);
    }

    public void setRepeats(String repeats) throws NotesException {
        if (getRepeats() != null)
            document.replaceItemValue(REPEATS, repeats);
        else
            document.appendItemValue(REPEATS, repeats);
    }

    public String getOrgRepeat() throws NotesException {
        return document.getItemValueString(ORG_REPEAT);
    }

    public void setOrgRepeat(String orgRepeat) throws NotesException {
        if (getOrgRepeat() != null)
            document.replaceItemValue(ORG_REPEAT, orgRepeat);
        else
            document.appendItemValue(ORG_REPEAT, orgRepeat);
    }

    public String getRef() throws NotesException {
        return document.getItemValueString(REF);
    }

    public void setRef(String ref) throws NotesException {
        if (getRef() != null)
            document.replaceItemValue(REF, ref);
        else
            document.appendItemValue(REF, ref);
    }

    public String getAttachmentNames() throws NotesException {
        return document.getItemValueString(ATTACHMENT_NAMES);
    }

    public void setAttachmentNames(String names) throws NotesException {
        if (getAttachmentNames() != null)
            document.replaceItemValue(ATTACHMENT_NAMES, names);
        else
            document.appendItemValue(ATTACHMENT_NAMES, names);
    }

    public String getAttachmentLength() throws NotesException {
        return document.getItemValueString(ATTACHMENT_LENGTH);
    }

    public void setAttachmentLength(String lenght) throws NotesException {
        if (getAttachmentLength() != null)
            document.replaceItemValue(ATTACHMENT_LENGTH, lenght);
        else
            document.appendItemValue(ATTACHMENT_LENGTH, lenght);
    }

    public String getPublicAccess() throws NotesException {
        return document.getItemValueString(PUBLIC_ACCESS);
    }

    public void setPublicAccess(String access) throws NotesException {
        if (getPublicAccess() != null)
            document.replaceItemValue(PUBLIC_ACCESS, access);
        else
            document.appendItemValue(PUBLIC_ACCESS, access);
    }

    public String getBusyPriority() throws NotesException {
        return document.getItemValueString(BUSY_PRIORITY);
    }

    public void setBusyPriority(String busy) throws NotesException {
        if (getBusyPriority() != null)
            document.replaceItemValue(BUSY_PRIORITY, busy);
        else
            document.appendItemValue(BUSY_PRIORITY, busy);
    }

    public String getPreventReplies() throws NotesException {
        return document.getItemValueString(PREVENT_REPLIES);
    }

    public void setPreventReplies(String prevent) throws NotesException {
        if (getPreventReplies() != null)
            document.replaceItemValue(PREVENT_REPLIES, prevent);
        else
            document.appendItemValue(PREVENT_REPLIES, prevent);
    }

    public Vector getExcludeFromView() throws NotesException {
        return document.getItemValue(EXCLUDE_FROM_VIEW);
    }

    public void setExcludeFromView(Vector view) throws NotesException {
        if (getExcludeFromView() != null)
            document.replaceItemValue(EXCLUDE_FROM_VIEW, view);
        else
            document.appendItemValue(EXCLUDE_FROM_VIEW, view);
    }

    public int getSequenceNum() throws NotesException {
        return document.getItemValueInteger(SEQUENCE_NUM);
    }

    public void setSequenceNum(int num) throws NotesException {
        if (getSequenceNum() != 0)
            document.replaceItemValue(SEQUENCE_NUM, String.valueOf(num));
        else
            document.appendItemValue(SEQUENCE_NUM, num);
    }

    public int getUpdateSeq() throws NotesException {
        return document.getItemValueInteger(UPDATE_SEQ);
    }

    public void setUpdateSeq(int seq) throws NotesException {
        if (getUpdateSeq() != 0)
            document.replaceItemValue(UPDATE_SEQ, String.valueOf(seq));
        else
            document.appendItemValue(UPDATE_SEQ, seq);
    }

    public Vector getCSWISL() throws NotesException {
        return document.getItemValue(CSWISL);
    }

    public void setCSWISL(Vector cswisl) throws NotesException {
        if (getCSWISL() != null)
            document.replaceItemValue(CSWISL, cswisl);
        else
            document.appendItemValue(CSWISL, cswisl);
    }

    public String getCsVersion() throws NotesException {
        return document.getItemValueString(CS_VERSION);
    }

    public void setCsVersion(String version) throws NotesException {
        if (getCsVersion() != null)
            document.replaceItemValue(CS_VERSION, version);
        else
            document.appendItemValue(CS_VERSION, version);
    }

    public String getOrgTable() throws NotesException {
        return document.getItemValueString(ORG_TABLE);
    }

    public void setOrgTable(String orgTable) throws NotesException {
        if (getOrgTable() != null)
            document.replaceItemValue(ORG_TABLE, orgTable);
        else
            document.appendItemValue(ORG_TABLE, orgTable);
    }

    public int getViewIcon() throws NotesException {
        return document.getItemValueInteger(VIEW_ICON);
    }

    public void setViewIcon(int viewIcon) throws NotesException {
        if (getViewIcon() != 0)
            document.replaceItemValue(VIEW_ICON, String.valueOf(viewIcon));
        else
            document.appendItemValue(VIEW_ICON, viewIcon);
    }

    public String getRefOptions() throws NotesException {
        return document.getItemValueString(REF_OPTIONS);
    }


    public void setRefOptions(String refOptions) throws NotesException {
        if (getRefOptions() != null)
            document.replaceItemValue(REF_OPTIONS, refOptions);
        else
            document.appendItemValue(REF_OPTIONS, refOptions);
    }

    public Vector getRepeatStartDate() throws NotesException {
        return document.getItemValue(REPEAT_START_DATE);
    }

    public void setRepeatStartDate(Object repeatStartDate) throws NotesException {
        if (getRepeatStartDate() != null)
            document.replaceItemValue(REPEAT_START_DATE, repeatStartDate);
        else
            document.appendItemValue(REPEAT_START_DATE, repeatStartDate);
    }

    public Vector getRepeatUntil() throws NotesException {
        return document.getItemValue(REPEAT_UNTIL);
    }

    public void setRepeatUntil(Object repeatUntil) throws NotesException {
        if (getRepeatUntil() != null)
            document.replaceItemValue(REPEAT_UNTIL, repeatUntil);
        else
            document.appendItemValue(REPEAT_UNTIL, repeatUntil);
    }

    public Vector getStartDate() throws NotesException {
        return document.getItemValue(START_DATE);
    }

    public void setStartDate(Object startDate) throws NotesException {
        if (getStartDate() != null)
            document.replaceItemValue(START_DATE, startDate);
        else
            document.appendItemValue(START_DATE, startDate);
    }

    public Vector getStartTime() throws NotesException {
        return document.getItemValue(START_TIME);
    }

    public void setStartTime(Object startTime) throws NotesException {
        if (document.getItemValue(START_TIME) != null) {
            document.replaceItemValue(START_TIME, startTime);
        } else {
            document.appendItemValue(START_TIME, startTime);
        }
    }

    public Vector getStartDateTime() throws NotesException {
        return document.getItemValue(START_DATE_TIME);
    }

    public void setStartDateTime(Object startDateTime) throws NotesException {
        if (getStartDateTime() != null)
            document.replaceItemValue(START_DATE_TIME, startDateTime);
        else
            document.appendItemValue(START_DATE_TIME, startDateTime);
    }

    public Vector getCalendarDateTime() throws NotesException {
        return document.getItemValue(CALENDAR_DATE_TIME);
    }

    public void setCalendarDateTime(Object calendarDateTime) throws NotesException {
        if (getCalendarDateTime() != null)
            document.replaceItemValue(CALENDAR_DATE_TIME, calendarDateTime);
        else
            document.appendItemValue(CALENDAR_DATE_TIME, calendarDateTime);
    }

    public Vector getEndDateTime() throws NotesException {
        return document.getItemValue(END_DATE_TIME);
    }

    public void setEndDateTime(Object endDateTime) throws NotesException {
        if (getEndDateTime() != null)
            document.replaceItemValue(END_DATE_TIME, endDateTime);
        else
            document.appendItemValue(END_DATE_TIME, endDateTime);
    }

    public Vector getEndDate() throws NotesException {
        return document.getItemValue(END_DATE);
    }

    public void setEndDate(Object endDate) throws NotesException {
        if (getEndDate() != null)
            document.replaceItemValue(END_DATE, endDate);
        else
            document.appendItemValue(END_DATE, endDate);
    }

    public Vector getEndTime() throws NotesException {
        return document.getItemValue(END_TIME);
    }

    public void setEndTime(Object endTime) throws NotesException {
        if (getEndTime() != null)
            document.replaceItemValue(END_TIME, endTime);
        else
            document.appendItemValue(END_TIME, endTime);
    }

    public Vector getNoPurge() throws NotesException {
        return document.getItemValue(NO_PURGE);
    }

    public void setNoPurge(Object noPurge) throws NotesException {
        if (getNoPurge() != null)
            document.replaceItemValue(NO_PURGE, noPurge);
        else
            document.appendItemValue(NO_PURGE, noPurge);
    }

    public Vector getRepeatDates() throws NotesException {
        return document.getItemValue(REPEAT_DATES);
    }

    public void setRepeatDates(Object repeatDates) throws NotesException {
        if (getRepeatDates() != null)
            document.replaceItemValue(REPEAT_DATES, repeatDates);
        else
            document.appendItemValue(REPEAT_DATES, repeatDates);
    }

    public Vector getRepeatEndDates() throws NotesException {
        return document.getItemValue(REPEAT_END_DATES);
    }

    public void setRepeatEndDates(Object repeatEndDates) throws NotesException {
        if (getRepeatEndDates() != null)
            document.replaceItemValue(REPEAT_END_DATES, repeatEndDates);
        else
            document.appendItemValue(REPEAT_END_DATES, repeatEndDates);
    }

    public Vector getRepeatInstanceDates() throws NotesException {
        return document.getItemValue(REPEAT_INSTANCE_DATES);
    }

    public void setRepeatInstanceDates(Object repeatInstanceDates)
            throws NotesException {
        if (getRepeatInstanceDates() != null)
            document.replaceItemValue(REPEAT_INSTANCE_DATES, repeatInstanceDates);
        else
            document.appendItemValue(REPEAT_INSTANCE_DATES, repeatInstanceDates);
    }

    public Database getDatabase() {
        return database;
    }

    public Session getSession() {
        return session;
    }

    public DateTime foundationDateToDominoDateTime(String clientDate) {

        DateTime dominoDate = null;
        Date date;
        GregorianCalendar cal;


        if (clientDate != null) {

            SimpleDateFormat formatter = new SimpleDateFormat();

            switch (clientDate.length()) {
                case 16: {
                    formatter.applyPattern("yyyyMMdd'T'HHmmss'Z'");
                    break;
                }
                case 10: {
                    formatter.applyPattern("yyyy-MM-dd");
                    break;
                }
                default: {
                    return null;
                }
            }

            try {
                formatter.setTimeZone(TimeZone.getTimeZone("UTC"));
                date = formatter.parse(clientDate);
                cal = new GregorianCalendar();
                if (getTimeZone()!=null) {
                    cal.setTimeZone(getTimeZone());
                }
                cal.setTime(date);
                dominoDate = createDateTime(cal);

            } catch (Exception ex) {

                return null;
            }

        }

        return dominoDate;
    }

    public DateTime createDateTime(DateTime dateTime, TimeZone tz) {
        GregorianCalendar cal;
        try {
            cal = new GregorianCalendar();
            if (tz!=null) {
                cal.setTimeZone(tz);
            }
            cal.setTime(dateTime.toJavaDate());
            return session.createDateTime(cal);
        } catch (Exception ex) {

            return null;
        }
    }

    public DateTime createDateTime(Calendar calendar) {

        try {

            return getDatabase().getParent().createDateTime(calendar);
        } catch (Exception ex) {

            return null;
        }

    }

    public String getUserName() {

        try {

            return session.getUserName();
        } catch (Exception ex) {

            return null;
        }

    }


}
