/**
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */

package com.funambol.domino.document;

import com.funambol.domino.exception.DominoDocumentException;
import com.funambol.domino.util.*;
import com.funambol.foundation.pdi.event.RecurrencePattern;

import java.util.*;
import java.util.logging.Level;

import lotus.domino.DateTime;
import lotus.domino.Document;
import lotus.domino.NotesException;

/**
 *
 * @version $Id: RecurrentDocument.java,v 1.8 2006/09/07 16:55:10 diogo Exp $
 */
public class RecurrentDocument extends DominoDocument {

    //RepeatUnit values
    public static final String REPEAT_UNIT_DAILY = "D";
    public static final String REPEAT_UNIT_WEEKLY = "W";
    public static final String REPEAT_UNIT_MONTHLY_DAY = "MP";
    public static final String REPEAT_UNIT_MONTHLY_DATES = "MD";
    public static final String REPEAT_UNIT_YEARLY   = "YD";
    public static final String REPEAT_UNIT_YEARLY_2 = "Y";

    //RepeatHow values
    public static final String REPEAT_HOW_FOR = "F";
    public static final String REPEAT_HOW_UNTIL = "U";

    //RepeatForUnit values
    public static final String REPEAT_FOR_DAYS = "D";
    public static final String REPEAT_FOR_WEEKS = "W";
    public static final String REPEAT_FOR_MONTHS = "M";
    public static final String REPEAT_FOR_YEARS = "Y";

    //RepeatWeekends values
    public static final String REPEAT_WEEKENDS_DELETE = "X";
    public static final String REPEAT_WEEKENDS_DO_NOT_MOVE = "D";

    //Foundation Mon + Tue + Wen + Thu + Fri mask
    public static final short WEEK_DAYS_MASK = 62;
    
    //Because in domino there is no "No end date option"
    public static final short FOR_EVER_VALUE = 712;

    private SimpleDocument parent = null;

    Vector startDateVector = null;
    Vector endDateVector = null;


    public String getRepeatInterval() throws NotesException {
        return parent.document.getItemValueString(REPEAT_INTERVAL);
    }

    public void setRepeatInterval(String repeatInterval) throws NotesException {
        if (parent.document.getItemValueString(REPEAT_INTERVAL) != null) {
            parent.document.replaceItemValue(REPEAT_INTERVAL, repeatInterval);
        } else {
            parent.document.appendItemValue(REPEAT_INTERVAL, repeatInterval);
        }
        if (document.getItemValueString(REPEAT_INTERVAL) != null) {
            document.replaceItemValue(REPEAT_INTERVAL, repeatInterval);
        } else {
            document.appendItemValue(REPEAT_INTERVAL, repeatInterval);
        }
    }

    public Vector getRepeatAdjust() throws NotesException {
        return parent.document.getItemValue(REPEAT_ADJUST);
    }

    public void setRepeatAdjust(Object repeatAdjust) throws NotesException {
        if (getRepeatAdjust() != null)
            parent.document.replaceItemValue(REPEAT_ADJUST, repeatAdjust);
        else
            parent.document.appendItemValue(REPEAT_ADJUST, repeatAdjust);
    }

    public String getRepeatHow() throws NotesException {
        return parent.document.getItemValueString(REPEAT_HOW);
    }

    public void setRepeatHow(String repeatHow) throws NotesException {
        if (parent.document.getItemValueString(REPEAT_HOW) != null) {
            parent.document.replaceItemValue(REPEAT_HOW, repeatHow);
        } else {
            parent.document.appendItemValue(REPEAT_HOW, repeatHow);
        }
        if (document.getItemValueString(REPEAT_HOW) != null) {
            document.replaceItemValue(REPEAT_HOW, repeatHow);
        } else {
            document.appendItemValue(REPEAT_HOW, repeatHow);
        }
    }

    public int getRepeatFor() throws NotesException {
        return parent.document.getItemValueInteger(REPEAT_FOR);
    }

    public void setRepeatFor(int repeatFor) throws NotesException {
        parent.document.replaceItemValue(REPEAT_FOR, new Double((double) repeatFor));
        document.replaceItemValue(REPEAT_FOR, new Double((double) repeatFor));
    }

    public String getRepeatForUnit() throws NotesException {
        return parent.document.getItemValueString(REPEAT_FOR_UNIT);
    }

    public void setRepeatForUnit(String repeatForUnit) throws NotesException {
        if (parent.document.getItemValueString(REPEAT_FOR_UNIT) != null) {
            parent.document.replaceItemValue(REPEAT_FOR_UNIT, repeatForUnit);
        } else {
            parent.document.appendItemValue(REPEAT_FOR_UNIT, repeatForUnit);
        }
        if (document.getItemValueString(REPEAT_FOR_UNIT) != null) {
            document.replaceItemValue(REPEAT_FOR_UNIT, repeatForUnit);
        } else {
            document.appendItemValue(REPEAT_FOR_UNIT, repeatForUnit);
        }
    }

    public String getRepeatWeekends() throws NotesException {
        return parent.document.getItemValueString(REPEAT_WEEKENDS);
    }

    public void setRepeatWeekends(String repeatWeekends) throws NotesException {
        if (parent.document.getItemValueString(REPEAT_WEEKENDS) != null) {
            parent.document.replaceItemValue(REPEAT_WEEKENDS, repeatWeekends);
        } else {
            parent.document.appendItemValue(REPEAT_WEEKENDS, repeatWeekends);
        }
        if (document.getItemValueString(REPEAT_WEEKENDS) != null) {
            document.replaceItemValue(REPEAT_WEEKENDS, repeatWeekends);
        } else {
            document.appendItemValue(REPEAT_WEEKENDS, repeatWeekends);
        }
    }

    public String getRepeatUnit() throws NotesException {
        return parent.document.getItemValueString(REPEAT_UNIT);
    }

    public void setRepeatUnit(String repeatUnit) throws NotesException {
        if (parent.document.getItemValueString(REPEAT_UNIT) != null) {
            parent.document.replaceItemValue(REPEAT_UNIT, repeatUnit);
        } else {
            parent.document.appendItemValue(REPEAT_UNIT, repeatUnit);
        }
        if (document.getItemValueString(REPEAT_UNIT) != null) {
            document.replaceItemValue(REPEAT_UNIT, repeatUnit);
        } else {
            document.appendItemValue(REPEAT_UNIT, repeatUnit);
        }
    }

    /**
     * Creates a new instance of Recurrence
     */
    public RecurrentDocument(Document document, TimeZone timeZone) throws DominoDocumentException {

        super(document, timeZone);

        try {

            String unid = document.getParentDocumentUNID();
            Document parent = getDatabase().getDocumentByUNID(unid);
            this.parent = new SimpleDocument(parent, getTimeZone());

            //All day event (type="2") don't need this kind of recurrence
            if (getAppointmentType().equals("2"))
                this.parent = null;

            //Single instances don't need Recurrence
            if (getRepeatInstanceDates().size() <= 1)
                this.parent = null;

        } catch (NotesException ex) {
            this.parent = null;
        }

//        try {
//
//            if(getParent() != null && getRepeatUnit() == null) {
//
//                setRepeatAdjust(this.parent.getRepeatAdjust());
//                setRepeatFor(this.parent.getRepeatFor());
//                setRepeatForUnit(this.parent.getRepeatForUnit());
//                setRepeatHow(this.parent.getRepeatHow());
//                setRepeatInterval(this.parent.getRepeatInterval());
//                setRepeatUnit(this.parent.getRepeatUnit());
//                setRepeatWeekends(this.parent.getRepeatWeekends());
//
//            }
//
//        } catch (NotesException ex) {
//            this.parent = null;
//        }

    }

    public SimpleDocument getParent() {

        return this.parent;
    }


    public boolean isRecurrent() {
        try {
            String csFlags = "" + getCsFlags();
            return csFlags.equals("i");
        } catch (NotesException ex) {
            return false;
        }
    }

    /**
     * comment
     *
     * @return RecurrencePattern
     * @throws DominoDocumentException
     */
    public RecurrencePattern getRecurrencePattern() throws DominoDocumentException {

        RecurrencePattern rp;
        try {

            String repeatUnit = getRepeatUnit();
            int interval = getInterval();
            short instance = getInstance();
            short monthOfYear = getMonthOfYear();
            short dayOfMonth = getDayOfMonth();
            short dayOfWeekMask = getDayOfWeekMask();
            String startDatePattern = getStartDatePattern();
            String endDatePattern = getEndDatePattern();
            boolean noEndDate = getNoEndDate();
            int occurrences = getOccurrences();

            rp = null;


            if (repeatUnit != null) {

                if (repeatUnit.equals(REPEAT_UNIT_DAILY)) {

                    if (!noEndDate) {
                        rp = RecurrencePattern.getDailyRecurrencePattern(interval,
                                startDatePattern,
                                endDatePattern,
                                noEndDate,
                                dayOfWeekMask);
                    } else {
                        rp = RecurrencePattern.getDailyRecurrencePattern(interval,
                                startDatePattern,
                                noEndDate,
                                occurrences,
                                dayOfWeekMask);
                    }

                } else if (repeatUnit.equals(REPEAT_UNIT_WEEKLY)) {

                    if (!noEndDate) {
                        rp = RecurrencePattern.getWeeklyRecurrencePattern(interval,
                                dayOfWeekMask,
                                startDatePattern,
                                endDatePattern,
                                noEndDate);
                    } else {
                        rp = RecurrencePattern.getWeeklyRecurrencePattern(interval,
                                dayOfWeekMask,
                                startDatePattern,
                                endDatePattern,
                                noEndDate,
                                occurrences);
                    }

                } else if (repeatUnit.equals(REPEAT_UNIT_MONTHLY_DATES)) {

                    if (!noEndDate) {
                        rp = RecurrencePattern.getMonthlyRecurrencePattern(interval,
                                dayOfMonth,
                                startDatePattern,
                                endDatePattern,
                                noEndDate);
                    } else {
                        rp = RecurrencePattern.getMonthlyRecurrencePattern(interval,
                                dayOfMonth,
                                startDatePattern,
                                endDatePattern,
                                noEndDate,
                                occurrences);
                    }

                } else if (repeatUnit.equals(REPEAT_UNIT_MONTHLY_DAY)) {

                    rp = RecurrencePattern.getMonthNthRecurrencePattern(interval,
                            dayOfWeekMask,
                            instance,
                            startDatePattern,
                            endDatePattern,
                            false,
                            occurrences);

                } else if (repeatUnit.equals(REPEAT_UNIT_YEARLY) 
                           || repeatUnit.equals(REPEAT_UNIT_YEARLY_2)) {
                    
                    if (!noEndDate) {
                        if (dayOfWeekMask==0 || instance==0) {
                        rp = RecurrencePattern.getYearlyRecurrencePattern(interval,
                                dayOfMonth,
                                monthOfYear,
                                startDatePattern,
                                endDatePattern,
                                noEndDate);
                        } else {
                            rp = RecurrencePattern.getYearNthRecurrencePattern(interval,
                                    dayOfWeekMask,
                                    monthOfYear,
                                    instance,
                                    startDatePattern,
                                    noEndDate);
                        }
                    } else {
                        if (dayOfWeekMask==0 || instance==0) {
                        rp = RecurrencePattern.getYearlyRecurrencePattern(interval,
                                dayOfMonth,
                                monthOfYear,
                                startDatePattern,
                                endDatePattern,
                                noEndDate,
                                occurrences);
                        } else {
                            rp = RecurrencePattern.getYearNthRecurrencePattern(interval,
                                    dayOfWeekMask,
                                    monthOfYear,
                                    instance,
                                    startDatePattern,
                                    endDatePattern,
                                    noEndDate,
                                    occurrences);
                        }
                    }
                }

            }
        } catch (Exception ex) {

            throw new DominoDocumentException(ex);
        }

        return rp;

    }

    public void setRecurrencePattern(RecurrencePattern rp, String dtStart, String dtEnd) throws DominoDocumentException {
        try {

            if (rp == null) {

                clearRecurrence();

            } else {

                if (parent == null)
                    createParent();

                int type = rp.getTypeId();
                switch (type) {
                    case RecurrencePattern.TYPE_DAYLY:
                        createDailyRecurrence(rp, dtStart, dtEnd);
                        break;
                    case RecurrencePattern.TYPE_WEEKLY:
                        createWeeklyRecurrence(rp, dtStart, dtEnd);
                        break;
                    case RecurrencePattern.TYPE_MONTHLY:
                        createMonthlyDatesRecurrence(rp, dtStart, dtEnd);
                        break;
                    case RecurrencePattern.TYPE_MONTH_NTH:
                        createMonthlyByDayRecurrence(rp, dtStart, dtEnd);
                        break;
                    case RecurrencePattern.TYPE_YEARLY:
                        createYearlyRecurrence(rp, dtStart, dtEnd);
                        break;
                    case RecurrencePattern.TYPE_YEAR_NTH:
                        createYearlyByDayRecurrence(rp, dtStart, dtEnd);
                        break;
                }

            }
        } catch (NotesException ex) {
            throw new DominoDocumentException(ex);
        }

    }

    public void clearRecurrence() throws NotesException {

        if (parent != null) {
            parent.getDocument().remove(true);
            document.makeResponse(null);
            setApptUNID(document.getUniversalID());
            setCsFlags(null);
            setOrgRepeat(null);
            setRepeatAdjust(null);
            setRepeatFor(0);
            setRepeatForUnit(null);
            setRepeatHow(null);
            setRepeatInstanceDates(null);
            setRepeatInterval(null);
            setRepeatUnit(null);
            setRepeatWeekends(null);
            setRepeats(null);
            setTxtNum(null);

            if (log.isLoggable(Level.FINEST)) {
                log.finest("Deleted recurrent properties from Appointment.");
            }

        }

    }

    public void setStartDate(Object startDate) throws NotesException {
        super.setStartDate(startDate);
        if (parent.document.getItemValue(START_DATE)!=null) {
            parent.document.replaceItemValue(START_DATE, startDate);
        } else {
         parent.document.appendItemValue(START_DATE, startDate);
        }
    }

    public void setStartTime(Object startTime) throws NotesException {
        super.setStartTime(startTime);
        if (parent.document.getItemValue(START_TIME)!=null) {
            parent.document.replaceItemValue(START_TIME, startTime);
        } else {
         parent.document.appendItemValue(START_TIME, startTime);
        }
    }

    public void setEndDate(Object endDate) throws NotesException {
        super.setEndDate(endDate);
         if (parent.document.getItemValue(END_DATE)!=null) {
            parent.document.replaceItemValue(END_DATE, endDate);
        } else {
         parent.document.appendItemValue(END_DATE, endDate);
        }
    }

    public void setEndTime(Object endTime) throws NotesException {
        super.setEndTime(endTime);
        if (parent.document.getItemValue(END_TIME)!=null) {
            parent.document.replaceItemValue(END_TIME, endTime);
        } else {
         parent.document.appendItemValue(END_TIME, endTime);
        }
    }

    private int getInterval() throws NotesException {

        String repeatInterval = getRepeatInterval();
        if (repeatInterval != null)
            return Integer.parseInt(repeatInterval);

        return 0;
    }

    private void setRepeatInterval(int interval) throws NotesException {
        setRepeatInterval(String.valueOf(interval));
    }

    private short getDayOfWeekMask() throws NotesException {

        //Codes for day of week mask
        //Sunday    = 1;
        //Monday    = 2;
        //Tuesday   = 4;
        //Wednesday = 8;
        //Thursday  = 16;
        //Friday    = 32;
        //Saturday  = 64;

        //codes for RepeatAdjust wen RepeatUnit is "W"
        //code	value
        //0	Sunday
        //1	Monday
        //2	Tuesday
        //3	Wednesday
        //4	Thursday
        //5	Friday
        //6	Saturday

        Vector repeatAdjust = getRepeatAdjust();
        String repeatUnit = getRepeatUnit();
        String repeatWeekends = getRepeatWeekends();

        if (repeatAdjust == null || repeatAdjust.isEmpty()) {
            return 0;
        }

        short dayOfWeekMask = 0;

        if (!(REPEAT_UNIT_MONTHLY_DAY.equals(repeatUnit) ||
            REPEAT_UNIT_YEARLY.equals(repeatUnit))) {
            short dayOfWeek;
            for (int i = 0; i < repeatAdjust.size(); i++) {
                dayOfWeek = Short.parseShort((String) repeatAdjust.get(i));
                dayOfWeekMask += (short) Math.pow(2, dayOfWeek);
            }

        } else if (repeatUnit.equals(REPEAT_UNIT_DAILY) &&
                repeatWeekends.equals(REPEAT_WEEKENDS_DELETE)) {
            return WEEK_DAYS_MASK;
        } else {

            String weekDayCode = (String) repeatAdjust.firstElement();
            // char'0' decimal value is 48
            int weekDay = weekDayCode.charAt(2) - 48 + 1;
            switch (weekDay) {

                case Calendar.SUNDAY   :
                    return 1;
                case Calendar.MONDAY   :
                    return 2;
                case Calendar.TUESDAY  :
                    return 4;
                case Calendar.WEDNESDAY:
                    return 8;
                case Calendar.THURSDAY :
                    return 16;
                case Calendar.FRIDAY   :
                    return 32;
                case Calendar.SATURDAY :
                    return 64;
            }
        }
        return dayOfWeekMask;
    }

    private short getInstance() throws NotesException {
        Vector repeatAdjust = getRepeatAdjust();
        String repeatUnit = getRepeatUnit();
        if (REPEAT_UNIT_MONTHLY_DAY.equals(repeatUnit) ||
             REPEAT_UNIT_YEARLY.equals(repeatUnit)
                && repeatAdjust != null && !repeatAdjust.isEmpty()) {
            String weekDayCode = (String) repeatAdjust.firstElement();
            // char'0' decimal value is 48
            return (short) (weekDayCode.charAt(0) - 48);
        }
        return 0;
    }

    private String getStartDatePattern() throws NotesException {

        return Utility.setClientDateTime(
                (DateTime) getParent().getRepeatInstanceDates().firstElement());

    }

    private String getEndDatePattern() throws NotesException {

        return Utility.setClientDateTime(
                (DateTime) getParent().getRepeatEndDates().lastElement());
    }

    private int getOccurrences() throws NotesException {

        if (getRepeatInstanceDates() != null)
            return getRepeatInstanceDates().size();

        return -1;

//        String repeatForUnit = getRepeatForUnit();
//        int interval = getInterval();
//        int repeatFor = getRepeatFor();
//        String repeatUnit = getRepeatUnit();
//
//        if(repeatUnit != null) {
//
//            if(repeatUnit.equals(REPEAT_UNIT_DAILY)) {
//
//                if(repeatForUnit.equals(REPEAT_FOR_DAYS))
//                    return  repeatFor / interval + 1;
//                else if(repeatForUnit.equals(REPEAT_FOR_WEEKS))
//                    return  repeatFor * 7 / interval + 1;
//                else if(repeatForUnit.equals(REPEAT_FOR_MONTHS))
//                    return  repeatFor * 31 / interval + 2;
//                else if(repeatForUnit.equals(REPEAT_FOR_YEARS))
//                    return  repeatFor * 365 / interval + 2;
//
//            } else if(repeatUnit.equals(REPEAT_UNIT_WEEKLY)) {
//
//                int weekDays = getWeekDaysCount();
//
//                if(repeatForUnit.equals(REPEAT_FOR_DAYS))
//                    return  repeatFor * weekDays / 7 / interval;
//                else if(repeatForUnit.equals(REPEAT_FOR_WEEKS))
//                    return  repeatFor * weekDays / interval ;
//                else if(repeatForUnit.equals(REPEAT_FOR_MONTHS))
//                    return  repeatFor * weekDays * 31 / 7 / interval;
//                else if(repeatForUnit.equals(REPEAT_FOR_YEARS))
//                    return  repeatFor * weekDays * 365 / 7 / interval;
//
//            } else if(repeatUnit.equals(REPEAT_UNIT_MONTHLY_DATES)
//            ||
//                    repeatUnit.equals(REPEAT_UNIT_MONTHLY_DAY)) {
//
//                int monthDays = getMonthDaysCount();
//
//                if(repeatForUnit.equals(REPEAT_FOR_DAYS))
//                    return  repeatFor * monthDays / 31 / interval;
//                else if(repeatForUnit.equals(REPEAT_FOR_WEEKS))
//                    return  repeatFor * monthDays / 7 / interval ;
//                else if(repeatForUnit.equals(REPEAT_FOR_MONTHS))
//                    return  repeatFor * monthDays / interval;
//                else if(repeatForUnit.equals(REPEAT_FOR_YEARS))
//                    return  repeatFor * monthDays * 12 / interval;
//
//            } else if(repeatUnit.equals(REPEAT_UNIT_YEARLY)) {
//
//                //because there is no option for interval in foundation event
//                //we don't make the calcs with the interval var
//                if(repeatForUnit.equals(REPEAT_FOR_DAYS))
//                    return  repeatFor / 365;
//                else if(repeatForUnit.equals(REPEAT_FOR_WEEKS))
//                    return  repeatFor * 7 / 365;
//                else if(repeatForUnit.equals(REPEAT_FOR_MONTHS))
//                    return  repeatFor * 31 / 365;
//                else if(repeatForUnit.equals(REPEAT_FOR_YEARS))
//                    return  repeatFor;
//            }
//
//        }
//
//
//        return -1;
    }

    private boolean getNoEndDate() throws NotesException {

        String repeatHow = getRepeatHow();

        if (repeatHow != null)
            return repeatHow.equals(REPEAT_HOW_FOR);

        return false;
    }

    private short getDayOfMonth() throws NotesException {

        Vector dates = getRepeatInstanceDates();
        if (dates != null) {
            DateTime startDate = (DateTime) dates.firstElement();
            GregorianCalendar cal = new GregorianCalendar();
            if (getTimeZone() != null) {
                cal.setTimeZone(getTimeZone());
            }
            cal.setTime(startDate.toJavaDate());
            return (short) cal.get(Calendar.DAY_OF_MONTH);
        }
        return 0;

    }

    private short getMonthOfYear() throws NotesException {

        Vector dates = getRepeatInstanceDates();
        if (dates != null) {
            DateTime startDate = (DateTime) dates.firstElement();
            GregorianCalendar cal = new GregorianCalendar();
            if (getTimeZone() != null) {
                cal.setTimeZone(getTimeZone());
            }
            cal.setTime(startDate.toJavaDate());
            return (short) (cal.get(Calendar.MONTH) + 1);
        }
        return 0;

    }

    private void createYearlyByDayRecurrence(RecurrencePattern rp, String dtStart, String dtEnd) {
        int repeatFor;
        try {
            setRepeatUnit(REPEAT_UNIT_YEARLY);
            int interval = rp.getInterval();
            if (interval >= 1 && interval <= 10) {
                setRepeatInterval(interval);
            } else if (interval > 10) {
                setRepeatInterval(10);
                interval = 10;
            } else if (interval < 1) {
                setRepeatInterval(1);
                interval = 1;
            }

            startDateVector = new Vector();
            endDateVector = new Vector();

            // estraggo la data e ora di inizio e di fine della prima ricorrenza
            DateTime startDateTime = foundationDateToDominoDateTime(dtStart);
            DateTime endDateTime = foundationDateToDominoDateTime(dtEnd);

            setStartDate(startDateTime);
            setStartTime(startDateTime);
            setEndDate(endDateTime);
            setEndTime(endDateTime);

            GregorianCalendar startCalendar = new GregorianCalendar();
            if (getTimeZone() != null) {
                startCalendar.setTimeZone(getTimeZone());
            }
            startCalendar.setTime(startDateTime.toJavaDate());

            Vector dayOfWeekInMonth = new Vector();
            int weekOfMonth = startCalendar.get(GregorianCalendar.DAY_OF_WEEK_IN_MONTH);
            int dayOfWeek = startCalendar.get(GregorianCalendar.DAY_OF_WEEK);

            dayOfWeekInMonth.addElement(weekOfMonth + "." + (dayOfWeek - 1));

            setRepeatAdjust(dayOfWeekInMonth);
            repeatFor = 0;
            if (rp.isNoEndDate()) {
                // metto il la scadenza ad esempio dopo 10 ricorrenze (10 anni)
                for (int i = 0; i < FOR_EVER_VALUE; i++) {
                    GregorianCalendar startTemp = new GregorianCalendar();
                    if (getTimeZone() != null) {
                        startTemp.setTimeZone(getTimeZone());
                    }
                    startTemp.setTime(startDateTime.toJavaDate());
                    startTemp.set(GregorianCalendar.DAY_OF_WEEK, dayOfWeek);
                    startTemp.set(GregorianCalendar.DAY_OF_WEEK_IN_MONTH, weekOfMonth);
                    GregorianCalendar endTemp = new GregorianCalendar();
                    if (getTimeZone() != null) {
                        endTemp.setTimeZone(getTimeZone());
                    }
                    endTemp.setTime(endDateTime.toJavaDate());
                    endTemp.set(GregorianCalendar.DAY_OF_WEEK, dayOfWeek);
                    endTemp.set(GregorianCalendar.DAY_OF_WEEK_IN_MONTH, weekOfMonth);
                    startDateVector.addElement(createDateTime(startTemp));
                    endDateVector.addElement(createDateTime(endTemp));
                    repeatFor++;
                    startDateTime.adjustYear(interval, true);
                    endDateTime.adjustYear(interval, true);
                }
            } else {
                DateTime untilDateTime = foundationDateToDominoDateTime(rp.getEndDatePattern());
                untilDateTime.adjustHour(24, true);
                while (endDateTime.timeDifference(untilDateTime) < 0) {
                    GregorianCalendar startTemp = new GregorianCalendar();
                    if (getTimeZone() != null) {
                        startTemp.setTimeZone(getTimeZone());
                    }
                    startTemp.setTime(startDateTime.toJavaDate());
                    startTemp.set(GregorianCalendar.DAY_OF_WEEK, dayOfWeek);
                    startTemp.set(GregorianCalendar.DAY_OF_WEEK_IN_MONTH, weekOfMonth);
                    GregorianCalendar endTemp = new GregorianCalendar();
                    if (getTimeZone() != null) {
                        endTemp.setTimeZone(getTimeZone());
                    }
                    endTemp.setTime(endDateTime.toJavaDate());
                    endTemp.set(GregorianCalendar.DAY_OF_WEEK, dayOfWeek);
                    endTemp.set(GregorianCalendar.DAY_OF_WEEK_IN_MONTH, weekOfMonth);
                    startDateVector.addElement(createDateTime(startTemp));
                    endDateVector.addElement(createDateTime(endTemp));
                    repeatFor++;
                    startDateTime.adjustYear(interval, true);
                    endDateTime.adjustYear(interval, true);
                    
                    endTemp.setTime(endDateTime.toJavaDate());
                    endTemp.set(GregorianCalendar.DAY_OF_WEEK, dayOfWeek);
                    endTemp.set(GregorianCalendar.DAY_OF_WEEK_IN_MONTH, weekOfMonth);
                    endDateTime.setLocalTime(endTemp);
                }
            }
            setRepeatWeekends(REPEAT_WEEKENDS_DO_NOT_MOVE);
            setRepeatFor(repeatFor);
            setRepeatForUnit(REPEAT_FOR_YEARS);
            setRecurrence();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void createYearlyRecurrence(RecurrencePattern rp, String dtStart, String dtEnd) {
        int repeatFor;
        try {

            setRepeatUnit(REPEAT_UNIT_YEARLY);
            int interval = rp.getInterval();
            if (interval >= 1 && interval <= 10) {
                setRepeatInterval(interval);
            } else if (interval > 10) {
                setRepeatInterval(10);
                interval = 10;
            } else if (interval < 1) {
                setRepeatInterval(1);
                interval = 1;
            }

            startDateVector = new Vector();
            endDateVector = new Vector();

            // get time of the first instance
            DateTime startDateTime = foundationDateToDominoDateTime(dtStart);
            DateTime endDateTime = foundationDateToDominoDateTime(dtEnd);

            setStartDate(startDateTime);
            setStartTime(startDateTime);
            setEndDate(endDateTime);
            setEndTime(endDateTime);

            repeatFor = 0;
            if (rp.isNoEndDate()) {
                // set the end time
                for (int i = 0; i < FOR_EVER_VALUE; i++) {
                    DateTime startTemp = createDateTime(startDateTime, getTimeZone());
                    DateTime endTemp = createDateTime(endDateTime, getTimeZone());
                    startDateVector.addElement(startTemp);
                    endDateVector.addElement(endTemp);
                    startDateTime.adjustYear(interval, true);
                    endDateTime.adjustYear(interval, true);
                    repeatFor++;
                }
            } else {
                DateTime untilDateTime = foundationDateToDominoDateTime(rp.getEndDatePattern());
                untilDateTime.adjustHour(24, true);
                while (startDateTime.timeDifference(untilDateTime) < 0) {
                    DateTime startTemp = createDateTime(startDateTime, getTimeZone());
                    DateTime endTemp = createDateTime(endDateTime, getTimeZone());
                    startDateVector.addElement(startTemp);
                    endDateVector.addElement(endTemp);
                    startDateTime.adjustYear(interval, true);
                    endDateTime.adjustYear(interval, true);
                    repeatFor++;
                }
            }
            setRepeatWeekends(REPEAT_WEEKENDS_DO_NOT_MOVE);
            setRepeatFor(repeatFor);
            setRepeatForUnit(REPEAT_FOR_YEARS);
            setRecurrence();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void createMonthlyDatesRecurrence(RecurrencePattern rp,
                                              String dtStart, String dtEnd) {
        int repeatFor;
        try {

            setRepeatUnit(REPEAT_UNIT_MONTHLY_DATES);
            int interval = rp.getInterval();
            if (interval >= 1 && interval <= 10) {
                setRepeatInterval(interval);
            } else if (interval > 10) {
                setRepeatInterval(10);
                interval = 10;
            } else if (interval < 1) {
                setRepeatInterval(1);
                interval = 1;
            }

            startDateVector = new Vector();
            endDateVector = new Vector();

            DateTime startDateTime = foundationDateToDominoDateTime(dtStart);
            DateTime endDateTime = foundationDateToDominoDateTime(dtEnd);

            setStartDate(startDateTime);
            setStartTime(startDateTime);
            setEndDate(endDateTime);
            setEndTime(endDateTime);

            GregorianCalendar startCalendar = new GregorianCalendar();
            if (getTimeZone() != null) {
                startCalendar.setTimeZone(getTimeZone());
            }
            startCalendar.setTime(startDateTime.toJavaDate());

            Vector daysOfMonth = new Vector();
            int dayOfMonth = startCalendar.get(GregorianCalendar.DAY_OF_MONTH);
            daysOfMonth.addElement(String.valueOf(dayOfMonth));
            setRepeatAdjust(daysOfMonth);

            repeatFor = 0;
            if (rp.isNoEndDate()) {
                for (int i = 0; i < FOR_EVER_VALUE; i++) {
                    DateTime startTemp = createDateTime(startDateTime, getTimeZone());
                    DateTime endTemp = createDateTime(endDateTime, getTimeZone());
                    startDateVector.addElement(startTemp);
                    endDateVector.addElement(endTemp);
                    startDateTime.adjustMonth(interval, true);
                    endDateTime.adjustMonth(interval, true);
                    repeatFor++;
                }
            } else {
                DateTime untilDateTime = foundationDateToDominoDateTime(rp.getEndDatePattern());
                untilDateTime.adjustHour(24, true);
                while (startDateTime.timeDifference(untilDateTime) < 0) {
                    DateTime startTemp = createDateTime(startDateTime, getTimeZone());
                    DateTime endTemp = createDateTime(endDateTime, getTimeZone());
                    startDateVector.addElement(startTemp);
                    endDateVector.addElement(endTemp);
                    startDateTime.adjustMonth(interval, true);
                    endDateTime.adjustMonth(interval, true);
                    repeatFor++;
                }
            }
            setRepeatFor(repeatFor);
            setRepeatForUnit(REPEAT_FOR_MONTHS);
            setRecurrence();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void createMonthlyByDayRecurrence(RecurrencePattern rp, String dtStart,
                                              String dtEnd) {
        int repeatFor;
        try {

            setRepeatUnit(REPEAT_UNIT_MONTHLY_DAY);
            int interval = rp.getInterval();
            if (interval >= 1 && interval <= 10) {
                setRepeatInterval(interval);
            } else if (interval > 10) {
                setRepeatInterval(10);
                interval = 10;
            } else if (interval < 1) {
                setRepeatInterval(1);
                interval = 1;
            }

            startDateVector = new Vector();
            endDateVector = new Vector();

            DateTime startDateTime = foundationDateToDominoDateTime(dtStart);
            DateTime endDateTime = foundationDateToDominoDateTime(dtEnd);

            setStartDate(startDateTime);
            setStartTime(startDateTime);
            setEndDate(endDateTime);
            setEndTime(endDateTime);

            GregorianCalendar startCalendar = new GregorianCalendar();
            if (getTimeZone() != null) {
                startCalendar.setTimeZone(timeZone);
            }
            startCalendar.setTime(startDateTime.toJavaDate());

            Vector dayOfWeekInMonth = new Vector();
            int weekOfMonth = startCalendar.get(GregorianCalendar.DAY_OF_WEEK_IN_MONTH);
            int dayOfWeek = startCalendar.get(GregorianCalendar.DAY_OF_WEEK);

            dayOfWeekInMonth.addElement(weekOfMonth + "." + (dayOfWeek - 1));

            setRepeatAdjust(dayOfWeekInMonth);
            repeatFor = 0;
            if (rp.isNoEndDate()) {
                for (int i = 0; i < FOR_EVER_VALUE; i++) {
                    GregorianCalendar startTemp = new GregorianCalendar();
                    if (getTimeZone() != null) {
                        startTemp.setTimeZone(getTimeZone());
                    }
                    startTemp.setTime(startDateTime.toJavaDate());
                    startTemp.set(GregorianCalendar.DAY_OF_WEEK, dayOfWeek);
                    startTemp.set(GregorianCalendar.DAY_OF_WEEK_IN_MONTH, weekOfMonth);
                    GregorianCalendar endTemp = new GregorianCalendar();
                    if (getTimeZone() != null) {
                        endTemp.setTimeZone(getTimeZone());
                    }
                    endTemp.setTime(endDateTime.toJavaDate());
                    endTemp.set(GregorianCalendar.DAY_OF_WEEK, dayOfWeek);
                    endTemp.set(GregorianCalendar.DAY_OF_WEEK_IN_MONTH, weekOfMonth);
                    startDateVector.addElement(createDateTime(startTemp));
                    endDateVector.addElement(createDateTime(endTemp));
                    startDateTime.adjustMonth(interval, true);
                    endDateTime.adjustMonth(interval, true);
                    repeatFor ++;
                }
            } else {
                DateTime untilDateTime = foundationDateToDominoDateTime(rp.getEndDatePattern());
                untilDateTime.adjustHour(24, true);
                while (endDateTime.timeDifference(untilDateTime) < 0) {
                    GregorianCalendar startTemp = new GregorianCalendar();
                    if (getTimeZone() != null) {
                        startTemp.setTimeZone(getTimeZone());
                    }
                    startTemp.setTime(startDateTime.toJavaDate());
                    startTemp.set(GregorianCalendar.DAY_OF_WEEK, dayOfWeek);
                    startTemp.set(GregorianCalendar.DAY_OF_WEEK_IN_MONTH, weekOfMonth);
                    GregorianCalendar endTemp = new GregorianCalendar();
                    if (getTimeZone() != null) {
                        endTemp.setTimeZone(getTimeZone());
                    }
                    endTemp.setTime(endDateTime.toJavaDate());
                    endTemp.set(GregorianCalendar.DAY_OF_WEEK, dayOfWeek);
                    endTemp.set(GregorianCalendar.DAY_OF_WEEK_IN_MONTH, weekOfMonth);
                    startDateVector.addElement(createDateTime(startTemp));
                    endDateVector.addElement(createDateTime(endTemp));
                    startDateTime.adjustMonth(interval, true);
                    endDateTime.adjustMonth(interval, true);
                    repeatFor++;
                    
                    endTemp.setTime(endDateTime.toJavaDate());
                    endTemp.set(GregorianCalendar.DAY_OF_WEEK, dayOfWeek);
                    endTemp.set(GregorianCalendar.DAY_OF_WEEK_IN_MONTH, weekOfMonth);
                    endDateTime.setLocalTime(endTemp);
                }
            }
            setRepeatFor(repeatFor);
            setRepeatForUnit(REPEAT_FOR_MONTHS);
            setRecurrence();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     *
     */
    private void createWeeklyRecurrence(RecurrencePattern rp, String dtStart, String dtEnd) {
        try {
            setRepeatUnit(REPEAT_UNIT_WEEKLY);
            int repeatFor;
            int interval = rp.getInterval();
            // RepeatInterval se RepeatUnit = "W" values have the range 1 to 8
            // every week, every 2 weeks, ...
            if (interval >= 1 && interval <= 8) {
                setRepeatInterval(interval);
            } else if (interval > 8) {
                setRepeatInterval(8);
                interval = 8;
            } else if (interval < 1) {
                setRepeatInterval(1);
                interval = 1;
            }

            startDateVector = new Vector();
            endDateVector = new Vector();

            DateTime startDateTime = foundationDateToDominoDateTime(dtStart);
            DateTime endDateTime = foundationDateToDominoDateTime(dtEnd);

            setStartDate(startDateTime);
            setStartTime(startDateTime);
            setEndDate(endDateTime);
            setEndTime(endDateTime);

            // get the day of the week
            GregorianCalendar startCalendar = new GregorianCalendar();
            GregorianCalendar endCalendar = new GregorianCalendar();
            if (getTimeZone() != null) {
                startCalendar.setTimeZone(getTimeZone());
                endCalendar.setTimeZone(getTimeZone());
            }
            startCalendar.setTime(startDateTime.toJavaDate());
            endCalendar.setTime(endDateTime.toJavaDate());

            // decode the DayOfWeekMask.
            int weekMask = rp.getDayOfWeekMask();
            Vector daysOfWeek = decodeWeekMask(weekMask);

            // domino doesn't accept rec. appt. without no end date
            if (rp.isNoEndDate()) {

                int recurrence = 0;
                repeatFor = 0;
                while (recurrence < 52) {
                    int dayOfWeek = startCalendar.get(GregorianCalendar.DAY_OF_WEEK);
                    if (daysOfWeek.contains("" + (dayOfWeek - 1))) {
                        DateTime startTemp = createDateTime(startCalendar);
                        startDateVector.addElement(startTemp);
                        DateTime endTemp = createDateTime(endCalendar);
                        endDateVector.addElement(endTemp);
                        repeatFor++;
                    }

                    startCalendar.add(GregorianCalendar.DATE, 1);
                    endCalendar.add(GregorianCalendar.DATE, 1);
                    dayOfWeek = startCalendar.get(GregorianCalendar.DAY_OF_WEEK);

                    while (dayOfWeek != GregorianCalendar.SUNDAY) {
                        if (daysOfWeek.contains("" + (dayOfWeek - 1))) {
                            DateTime startTemp = createDateTime(startCalendar);
                            startDateVector.addElement(startTemp);
                            DateTime endTemp = createDateTime(endCalendar);
                            endDateVector.addElement(endTemp);
                            repeatFor++;
                        }
                        startCalendar.add(GregorianCalendar.DATE, 1);
                        endCalendar.add(GregorianCalendar.DATE, 1);
                        dayOfWeek = startCalendar.get(GregorianCalendar.DAY_OF_WEEK);
                    }
                    // "interval" value can change the week
                    startCalendar.add(GregorianCalendar.DATE, 7 * (interval - 1));
                    endCalendar.add(GregorianCalendar.DATE, 7 * (interval - 1));
                    recurrence++;
                }
            }
            // if NoEndDate...
            else {
                repeatFor = 0;

                if (rp.getInstance() == 0) {

                    DateTime untilDateTime = foundationDateToDominoDateTime(
                            rp.getEndDatePattern());
                    untilDateTime.adjustHour(24, true);
                    System.out.println(untilDateTime);
                    while (startCalendar.getTimeInMillis() - untilDateTime
                            .toJavaDate().getTime() < 0) {
                        int dayOfWeek = startCalendar.get(GregorianCalendar.DAY_OF_WEEK);
                        if (daysOfWeek.contains("" + (dayOfWeek - 1))) {
                            DateTime startTemp = createDateTime(startCalendar);
                            startDateVector.addElement(startTemp);
                            DateTime endTemp = createDateTime(endCalendar);
                            endDateVector.addElement(endTemp);
                            repeatFor++;
                        }

                        startCalendar.add(GregorianCalendar.DATE, 1);
                        endCalendar.add(GregorianCalendar.DATE, 1);
                        dayOfWeek = startCalendar.get(GregorianCalendar.DAY_OF_WEEK);

                        while (dayOfWeek != GregorianCalendar.SUNDAY) {
                            if (startCalendar.getTimeInMillis() - untilDateTime
                                    .toJavaDate().getTime() >= 0) {
                                break;
                            }
                            if (daysOfWeek.contains("" + (dayOfWeek - 1))) {
                                DateTime startTemp = createDateTime(startCalendar);
                                startDateVector.addElement(startTemp);
                                DateTime endTemp = createDateTime(endCalendar);
                                endDateVector.addElement(endTemp);
                                repeatFor++;
                            }
                            startCalendar.add(GregorianCalendar.DATE, 1);
                            endCalendar.add(GregorianCalendar.DATE, 1);
                            dayOfWeek = startCalendar.get(GregorianCalendar.DAY_OF_WEEK);
                        }
                        startCalendar.add(GregorianCalendar.DATE, 7 * (interval - 1));
                        endCalendar.add(GregorianCalendar.DATE, 7 * (interval - 1));
                    }
                } else {
                    // todo
                }

            }

            setRepeatAdjust(daysOfWeek);
            setRepeatFor(repeatFor);
            setRepeatForUnit(REPEAT_FOR_WEEKS);
            setRecurrence();

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     *
     */
    private void createDailyRecurrence(RecurrencePattern rp, String dtStart, String dtEnd)
            throws DominoDocumentException {
        try {

            setRepeatUnit(REPEAT_UNIT_DAILY);
            int repeatFor = 0;

            // set interval: every day, every 2 days, ...
            int interval = rp.getInterval();
            if (interval >= 1 && interval <= 31) {
                setRepeatInterval(interval);
            } else if (interval > 31) {
                setRepeatInterval(31);
                interval = 31;
            } else if (interval < 1) {
                setRepeatInterval(1);
                interval = 1;
            }

            short weekMask = rp.getDayOfWeekMask();
            startDateVector = new Vector();
            endDateVector = new Vector();

            DateTime startDateTime = foundationDateToDominoDateTime(dtStart);
            DateTime endDateTime = foundationDateToDominoDateTime(dtEnd);

            setStartDate(startDateTime);
            setStartTime(startDateTime);
            setEndDate(endDateTime);
            setEndTime(endDateTime);

            if (rp.isNoEndDate()) {

                for (int i = 0; i < FOR_EVER_VALUE; i++) {
                    DateTime startTemp = createDateTime(startDateTime, getTimeZone());
                    DateTime endTemp = createDateTime(endDateTime, getTimeZone());
                    startDateVector.addElement(startTemp);
                    endDateVector.addElement(endTemp);
                    startDateTime.adjustDay(interval, true);
                    endDateTime.adjustDay(interval, true);
                }
                repeatFor = FOR_EVER_VALUE;

            } else {
                if (rp.getInstance() == 0) {

                    DateTime untilDateTime = foundationDateToDominoDateTime(rp.getEndDatePattern());
                    untilDateTime.adjustHour(24, true);     // in order to get the last instance
                    repeatFor = 0;
                    while (startDateTime.timeDifference(untilDateTime) < 0) {
                        DateTime startTemp = createDateTime(startDateTime, getTimeZone());
                        DateTime endTemp = createDateTime(endDateTime, getTimeZone());
                        startDateVector.addElement(startTemp);
                        endDateVector.addElement(endTemp);
                        startDateTime.adjustDay(interval, true);
                        endDateTime.adjustDay(interval, true);
                        repeatFor++;
                    }
                } else {
                    // to do
                }
            }


            if (weekMask == WEEK_DAYS_MASK) {

                int dayOfWeek;

                Vector itemsToDelete = new Vector();
                repeatFor = 0;
                for (Iterator it = startDateVector.iterator(); it.hasNext();) {
                    DateTime startDate = (DateTime) it.next();
                    GregorianCalendar startCalendar = new GregorianCalendar();
                    if (getTimeZone() != null) {
                        startCalendar.setTimeZone(getTimeZone());
                    }
                    startCalendar.setTime(startDate.toJavaDate());
                    dayOfWeek = startCalendar.get(GregorianCalendar.DAY_OF_WEEK);
                    if (dayOfWeek == GregorianCalendar.SUNDAY ||
                            dayOfWeek == GregorianCalendar.SATURDAY) {
                        itemsToDelete.add(startDate);
                    } else {
                        repeatFor++;
                    }
                }
                for (Iterator it = itemsToDelete.iterator(); it.hasNext();) {
                    Object elem = it.next();
                    startDateVector.remove(elem);
                }

                itemsToDelete = new Vector();
                repeatFor = 0;
                for (Iterator it = endDateVector.iterator(); it.hasNext();) {
                    DateTime endDate = (DateTime) it.next();
                    GregorianCalendar endCalendar = new GregorianCalendar();
                    if (getTimeZone() != null) {
                        endCalendar.setTimeZone(getTimeZone());
                    }
                    endCalendar.setTime(endDate.toJavaDate());
                    dayOfWeek = endCalendar.get(GregorianCalendar.DAY_OF_WEEK);
                    if (dayOfWeek == GregorianCalendar.SUNDAY ||
                            dayOfWeek == GregorianCalendar.SATURDAY)
                        itemsToDelete.add(endDate);
                    repeatFor++;
                }
                for (Iterator it = itemsToDelete.iterator(); it.hasNext();) {
                    Object elem = it.next();
                    endDateVector.remove(elem);
                }

                setRepeatWeekends(REPEAT_WEEKENDS_DELETE);
            } else {
                setRepeatWeekends(REPEAT_WEEKENDS_DO_NOT_MOVE);
            }


            setRepeatAdjust(new Vector());
            setRepeatFor(repeatFor);
            setRepeatForUnit(REPEAT_FOR_DAYS);
            setRecurrence();
        } catch (Exception e) {
            throw new DominoDocumentException(e);
        }
    }

    private void createParent() throws NotesException, DominoDocumentException {

        setAttachmentNames("QPNULL");
        setAttachmentLength("QPNULL");
        setPublicAccess("1");
        setBusyPriority("1");
        setPreventReplies("0");
        Vector excludeFromViewVector = new Vector();
        excludeFromViewVector.addElement("D");
        excludeFromViewVector.addElement("S");
        setExcludeFromView(excludeFromViewVector);
        setSequenceNum(1);
        setUpdateSeq(1);
        Vector cswislVect = new Vector();
        cswislVect.add("$S:1");
        cswislVect.add("$L:1");
        cswislVect.add("$B:1");
        cswislVect.add("$R:1");
        cswislVect.add("$E:1");
        setCSWISL(cswislVect);
        setCsVersion("2");
        setRepeats("1");
        setOrgRepeat("1");

        Document parentDoc = getDatabase().createDocument();
        String uid = parentDoc.getUniversalID();
        setApptUNID(uid);

        parent = new SimpleDocument(parentDoc, getTimeZone());
        document.copyAllItems(parentDoc, false);

        parent.setRepeatCustom("");
        parent.setCsFlags("c");
        parent.getDocument().removeItem(CALENDAR_DATE_TIME);
        parent.getDocument().computeWithForm(true, false);
        parent.getDocument().save(true, true, true);

        setChair(getUserName());
        setAltChair(getUserName());
        setAltPrincipal(getUserName());
        setRef(this.parent.getApptUNID());
        setRefOptions("1");
        setCsFlags("i");

        document.makeResponse(parent.getDocument());

    }

    private void setRecurrence() throws NotesException {

        setRepeatInstanceDates(startDateVector);
        setStartDateTime(startDateVector);
        setCalendarDateTime(startDateVector);
        setEndDateTime(endDateVector);
        setRepeatHow(REPEAT_HOW_UNTIL);

        parent.setRepeatInstanceDates(startDateVector);
        parent.setRepeatStartDate(startDateVector.firstElement());
        parent.setRepeatUntil(startDateVector.lastElement());
        parent.setStartDateTime(startDateVector.firstElement());
        parent.setEndDateTime(endDateVector.firstElement());
        parent.setRepeatDates(startDateVector);
        parent.setRepeatEndDates(endDateVector);
        parent.getDocument().computeWithForm(true, false);
        parent.getDocument().save(true, true, true);

    }


    private Vector decodeWeekMask(int weekMask) {

        Vector daysOfWeek = new Vector();
        if (weekMask > 0) {

            int rest;
            for (int i = 0; i < 7 && weekMask > 0; i++) {

                rest = weekMask % 2;
                weekMask /= 2;

                if (rest > 0)
                    daysOfWeek.addElement(String.valueOf(i));

            }

        }
        return daysOfWeek;
    }

}
