/**
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.domino.engine.source;

import com.funambol.domino.engine.cache.SyncItemInfo;

import java.io.Serializable;
import java.io.ByteArrayInputStream;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.logging.Level;

import com.funambol.framework.engine.*;
import com.funambol.framework.engine.source.*;
import com.funambol.framework.tools.beans.*;
import com.funambol.domino.items.model.ItemCalendar;
import com.funambol.foundation.pdi.parser.ICalendarParser;
import com.funambol.foundation.pdi.parser.XMLEventParser;
import com.funambol.foundation.pdi.converter.CalendarToXML;
import com.funambol.foundation.pdi.converter.CalendarToIcalendar;
import com.funambol.foundation.pdi.utils.SourceUtils;


import com.funambol.domino.items.manager.CalendarManager;
import com.funambol.domino.exception.*;
import com.funambol.domino.util.*;
import com.funambol.foundation.common.merge.MergeResult;
import com.funambol.foundation.pdi.event.Calendar;

/**
 * This class define <i>SyncSource</i>
 * between SyncServer and Domino Server calendar items
 *
 * @author Gilberto Migliavacca
 * @version $Id: DominoCalendarSyncSource.java,v 1.1 2006/08/08 08:31:15 gibi Exp $
 */
public final class DominoCalendarSyncSource
        extends DominoSyncSource
        implements MergeableSyncSource, Serializable, LazyInitBean {

    // ----------------------------------------------------------- Constants

    // ----------------------------------------------------------- Private data

    private CalendarManager cm = null;

    // ----------------------------------------------------------- Public Methods

    /**
     * Init the syncSource
     *
     * @throws BeanInitializationException
     */
    public void init() throws BeanInitializationException {

        super.init();
    }

    /* @see com.funambol.framework.engine.source.SyncSource */
    public SyncItemKey[] getSyncItemKeysFromTwin(SyncItem syncItem) throws SyncSourceException {
        if (log.isLoggable(Level.FINEST)) {
            log.finest("getSyncItemFromTwin [" + sourceURI + "]");
        }

        //Document doc      = null;
        SyncItemKey keys[] = new SyncItemKey[0];
        String content = null;
        String id = syncItem.getKey().getKeyAsString();

        // get content of the item
        try {
            content = Utility.getContentFromSyncItem(syncItem, false);
        } catch (Exception e) {
            throw new SyncSourceException("Error getting the item " +
                    syncItem, e);
        }


        try {
            this.cm = getEventManager();
            // @todo TASK next release
            //this.tm = getTodoManager();
        } catch (DominoAccessException e) {
            throw new SyncSourceException("Error opening CalendarManager/TodoManager", e);
        }


        boolean iscal = false;
        try {
            iscal = Utility.isCalendar(content, this.getItemFormat());
        } catch (EntityException e) {
            throw new SyncSourceException("Error converting foundation: " +
                    e.getMessage(), e);
        }

        ItemCalendar ctmp = new ItemCalendar();
        try {
            if (this.getItemFormat().equals(Def.ICAL_TYPE)) {
                ctmp = ical2Calendar(id, content);
            } else {
                ctmp = sife2Calendar(id, content);
            }
        } catch (EntityException e) {
            throw new SyncSourceException("Error converting foundation: " +
                    e.getMessage(), e);
        }

        // CALENDAR
        if (iscal) {

            ItemCalendar twins[];
            try {
                twins = this.cm.getItemsByWhereClause(ctmp,
                        this.getItemFormat());
            } catch (EntityException e) {
                throw new SyncSourceException("Error getting twin", e);
            }

            // converts Calendar in SyncItem
            keys = new SyncItemKey[twins.length];
            for (int i = 0; i < twins.length; i++)
                keys[i] = new SyncItemKey(twins[i].getId());

            return keys;
        }
        return new SyncItemKey[0];
    }

    /* @see com.funambol.framework.engine.source.SyncSource */
    public SyncItem updateSyncItem(SyncItem syncItem) throws SyncSourceException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("setSyncItem [" + sourceURI + "]");
        }

        SyncItem syncItemOut = null;
        String content = null;
        String id = null;
        String operation = Def.OPERATION_NOP;
        char status = ' ';

        id = syncItem.getKey().getKeyAsString();

        // get content of the item
        try {
            content = Utility.getContentFromSyncItem(syncItem);
        } catch (Exception e) {
            throw new SyncSourceException("Error getting the item " +
                    syncItem, e);
        }

        if (log.isLoggable(Level.FINEST)) {
            log.finest("setSyncItem content \n" + content);
        }


        ItemCalendar clientItem = new ItemCalendar();
        try {
            if (this.getItemFormat().equals(Def.ICAL_TYPE)) {
                clientItem = ical2Calendar(id, content);
            } else {
                clientItem = sife2Calendar(id, content);
            }
        } catch (EntityException e) {
            throw new SyncSourceException("Error converting foundation: " +
                    e.getMessage(), e);
        }

        // start pre-insert

        try {
            this.cm = getEventManager();
            // @todo TASK next release
            //this.tm = getTodoManager();
        } catch (DominoAccessException e) {
            throw new SyncSourceException("Error connecting calendar/task: " +
                    e.getMessage(), e);
        }

        boolean iscal = false;
        try {
            iscal = Utility.isCalendar(content, this.getItemFormat());
        } catch (EntityException e) {
            throw new SyncSourceException("Error converting foundation: " +
                    e.getMessage(), e);
        }


        ItemCalendar serverItem = null;

        // CALENDAR
        if (iscal) {

            try {
                serverItem = this.cm.getItemByUID(id, this.getItemFormat());
                operation = Def.OPERATION_UPDATE;
            } catch (Exception ex) {
                String errorMessage = "The item '" + id +
                        "' is mapped but there is a error getting domino document ["
                        + ex.getMessage() + "]";
                if (log.isLoggable(Level.SEVERE)) {
                    log.severe(errorMessage);
                }
                throw new SyncSourceException(errorMessage, ex);
            }


            try {
                if (operation.equals(Def.OPERATION_UPDATE)) {
                    status = SyncItemState.UPDATED;


                    if (conflictResolution !=
                            SyncContext.CONFLICT_RESOLUTION_CLIENT_WINS) {

                        serverItem.copyCommonProperties(clientItem);

                        this.cm.updateItem(serverItem,
                                serverItem,
                                this.getItemFormat(),
                                iscal);

                    } else {

                        this.cm.updateItem(serverItem,
                                clientItem,
                                this.getItemFormat(),
                                iscal);
                    }


                } else {
                    status = SyncItemState.NEW;
                    serverItem = this.cm.insertItem(clientItem,
                            this.getItemFormat(),
                            iscal);
                }

            } catch (Exception e) {
                String errorMessage = "Error setting the item: " + syncItem;
                if (log.isLoggable(Level.SEVERE)) {
                    log.severe(errorMessage);
                }
                throw new SyncSourceException(errorMessage, e);
            }

            if (log.isLoggable(Level.FINEST)) {
                log.finest(" end insert/update. start create SyncItem");
            }

            // create the output syncItem
            try {
                syncItemOut = createSyncItemFromCalendar(serverItem, status);
            } catch (Exception e) {
                String errorMessage = "Error creating SyncItem: " + syncItem;
                throw new SyncSourceException(errorMessage, e);
            }
        }
        // @todo TASK next release
        /*
        else{
         
            // check if there is a mapped key. If this happend, we must do an update
            if (syncItemExt.getMappedKey() != null) {
                try {
                    ctmpOld = this.tm.getItemByUID(id,this.getItemFormat());
                    operation = Def.OPERATION_UPDATE;
                } catch (Exception ex) {
                    String errorMessage = "The item '" + id +
                        "' is mapped but there is a error getting domino document ["
                        + ex.getMessage() + "]";
                    if (log.isLoggable(Level.SEVERE)) {
                        log.severe(errorMessage);
                    }
                    throw new SyncSourceException(errorMessage, ex);
                }
            } else {
                operation = Def.OPERATION_INSERT;
            }
         
            try {
                if (operation.equals(Def.OPERATION_UPDATE)) {
                    status = SyncItemState.UPDATED;
                    this.tm.updateItem(ctmpOld,
                                       ctmpNew,
                                       this.getItemFormat(),
                                       deviceTimezone);
                } else {
                    status = SyncItemState.NEW;
                    ctmpOld = this.tm.insertItem(ctmpNew,
                                                 this.getItemFormat(),
                                                 deviceTimezone);
                }
            } catch (Exception e) {
                String errorMessage = "Error setting the item: " + syncItemExt;
                if (log.isLoggable(Level.SEVERE)) {
                    log.severe(errorMessage);
                }
                throw new SyncSourceException(errorMessage, e);
            }
         
            if (log.isLoggable(Level.FINEST)) {
                log.finest(" end insert/update. start create SyncItem" );
            }
         
            // create the output syncitem
            try {
                syncItem = createSyncItemFromTodo(ctmpOld, status, deviceTimezone);
            } catch (Exception e) {
                String errorMessage = "Error creating SyncItem: " + syncItemExt;
                throw new SyncSourceException(errorMessage, e);
            }
         
        }
         */

        return syncItemOut;
    }

    /* @see com.funambol.framework.engine.source.SyncSource */
    public SyncItem addSyncItem(SyncItem syncItem) throws SyncSourceException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("setSyncItem [" + sourceURI + "]");
        }

        SyncItem syncItemOut = null;
        String content = null;
        String id = null;
        String operation = Def.OPERATION_NOP;
        char status = ' ';

        id = syncItem.getKey().getKeyAsString();

        // get content of the item
        try {
            content = Utility.getContentFromSyncItem(syncItem);
        } catch (Exception e) {
            throw new SyncSourceException("Error getting the item " +
                    syncItem, e);
        }

        if (log.isLoggable(Level.FINEST)) {
            log.finest("setSyncItem content \n" + content);
        }


        ItemCalendar ctmpNew = new ItemCalendar();
        try {
            if (this.getItemFormat().equals(Def.ICAL_TYPE)) {
                ctmpNew = ical2Calendar(id, content);
            } else {
                ctmpNew = sife2Calendar(id, content);
            }
        } catch (EntityException e) {
            throw new SyncSourceException("Error converting foundation: " +
                    e.getMessage(), e);
        }

        // start pre-insert

        try {
            this.cm = getEventManager();
            // @todo TASK next release
            //this.tm = getTodoManager();
        } catch (DominoAccessException e) {
            throw new SyncSourceException("Error connecting calendar/task: " +
                    e.getMessage(), e);
        }

        boolean iscal = false;
        try {
            iscal = Utility.isCalendar(content, this.getItemFormat());
        } catch (EntityException e) {
            throw new SyncSourceException("Error converting foundation: " +
                    e.getMessage(), e);
        }


        ItemCalendar ctmpOld = null;

        // CALENDAR
        if (iscal) {

            operation = Def.OPERATION_INSERT;

            try {
                if (operation.equals(Def.OPERATION_UPDATE)) {
                    status = SyncItemState.UPDATED;
                    this.cm.updateItem(ctmpOld,
                            ctmpNew,
                            this.getItemFormat(),
                            iscal);
                } else {
                    status = SyncItemState.NEW;
                    ctmpOld = this.cm.insertItem(ctmpNew,
                            this.getItemFormat(),
                            iscal);
                }

            } catch (Exception e) {
                String errorMessage = "Error setting the item: " + syncItem;
                if (log.isLoggable(Level.SEVERE)) {
                    log.severe(errorMessage);
                }
                throw new SyncSourceException(errorMessage, e);
            }

            if (log.isLoggable(Level.FINEST)) {
                log.finest(" end insert/update. start create SyncItem");
            }

            // create the output syncItem
            try {
                syncItemOut = createSyncItemFromCalendar(ctmpOld, status);
            } catch (Exception e) {
                String errorMessage = "Error creating SyncItem: " + syncItem;
                throw new SyncSourceException(errorMessage, e);
            }
        }
        // @todo TASK next release
        /*
        else{
         
            // check if there is a mapped key. If this happend, we must do an update
            if (syncItemExt.getMappedKey() != null) {
                try {
                    ctmpOld = this.tm.getItemByUID(id,this.getItemFormat());
                    operation = Def.OPERATION_UPDATE;
                } catch (Exception ex) {
                    String errorMessage = "The item '" + id +
                        "' is mapped but there is a error getting domino document ["
                        + ex.getMessage() + "]";
                    if (log.isLoggable(Level.SEVERE)) {
                        log.severe(errorMessage);
                    }
                    throw new SyncSourceException(errorMessage, ex);
                }
            } else {
                operation = Def.OPERATION_INSERT;
            }
         
            try {
                if (operation.equals(Def.OPERATION_UPDATE)) {
                    status = SyncItemState.UPDATED;
                    this.tm.updateItem(ctmpOld,
                                       ctmpNew,
                                       this.getItemFormat(),
                                       deviceTimezone);
                } else {
                    status = SyncItemState.NEW;
                    ctmpOld = this.tm.insertItem(ctmpNew,
                                                 this.getItemFormat(),
                                                 deviceTimezone);
                }
            } catch (Exception e) {
                String errorMessage = "Error setting the item: " + syncItemExt;
                if (log.isLoggable(Level.SEVERE)) {
                    log.severe(errorMessage);
                }
                throw new SyncSourceException(errorMessage, e);
            }
         
            if (log.isLoggable(Level.FINEST)) {
                log.finest(" end insert/update. start create SyncItem" );
            }
         
            // create the output syncitem
            try {
                syncItem = createSyncItemFromTodo(ctmpOld, status, deviceTimezone);
            } catch (Exception e) {
                String errorMessage = "Error creating SyncItem: " + syncItemExt;
                throw new SyncSourceException(errorMessage, e);
            }
         
        }
         */

        return syncItemOut;
    }


    /* @see com.funambol.framework.engine.source.SyncSource */
    public void removeSyncItem(SyncItemKey syncItemKey, Timestamp timestamp,
                               boolean softDelete)
            throws SyncSourceException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("remove item softDelete: " + softDelete);
        }

        if (softDelete) {
            // nothing
        } else {
            // if false
            removeItem(syncItemKey, timestamp);
        }
    }

    /* @see com.funambol.framework.engine.source.SyncSource */
    public SyncItem getSyncItemFromId(SyncItemKey syncItemKey) throws SyncSourceException {
        if (log.isLoggable(Level.FINEST)) {
            log.finest("getSyncItemFromId start");
        }

        String id = null;
        SyncItem syncItem = null;

        id = syncItemKey.getKeyAsString();

        try {
            this.cm = getEventManager();
            // @todo TASK next release
            //this.tm = getTodoManager();
        } catch (DominoAccessException e) {
            throw new SyncSourceException("Error connecting calendar/task: " +
                    e.getMessage(), e);
        }

        boolean iscal = false;
        try {
            iscal = this.cm.isCalendar(this.getDominoWrapper(), id);
        } catch (EntityException e) {
            throw new SyncSourceException("Error converting foundation: " +
                    e.getMessage(), e);
        }

        ItemCalendar ctmp = null;
        try {
            // CALENDAR
            if (iscal) {
                ctmp = this.cm.getItemByUID(id, this.getItemFormat());
                syncItem = createSyncItemFromCalendar(ctmp, SyncItemState.NEW);
            }
            // @todo TASK next release
            /*
            else {
                ctmp = this.tm.getItemByUID(id, this.getItemFormat());
                syncItem = createSyncItemFromTodo(ctmp, SyncItemState.NEW);
            }
             */
        } catch (Exception e) {
            throw new SyncSourceException("Error in creating SyncItem ", e);
        }

        return syncItem;
    }


    /**
     * @return SyncItemInfo[]
     * @throws SyncSourceException
     */
    public SyncItemInfo[] getAllSyncItemInfo() throws
            SyncSourceException {
        if (log.isLoggable(Level.FINEST)) {
            log.finest("getAllSyncItemInfo start");
        }

        SyncItemInfo[] syncItemInfo = null;
        SyncItemInfo[] calendarInfo = null;
        SyncItemInfo[] taskInfo = null;

        try {
            this.cm = getEventManager();
            // @todo TASK next release
            //this.tm      = getTodoManager();
        } catch (Exception e) {
            throw new SyncSourceException("Error connecting calendars: " +
                    e.getMessage(), e);
        }

        try {

            // only calendar
            calendarInfo = this.cm.getAllSyncItemInfo(Def.CALENDAR_VIEW);
            /*
            if (this.isEventHandler() && this.isTodoHandler()){
               calendarInfo = this.cm.getAllSyncItemInfo(Def.CALENDAR_VIEW,
                                                         this.getItemFormat());
               taskInfo     = this.tm.getAllSyncItemInfo(Def.TODO_VIEW,
                                                         this.getItemFormat());
            }
            if (this.isEventHandler()){
               calendarInfo = this.cm.getAllSyncItemInfo(Def.CALENDAR_VIEW,
                                                         this.getItemFormat());
            }
            if (this.isTodoHandler()){
               taskInfo     = this.tm.getAllSyncItemInfo(Def.TODO_VIEW,
                                                         this.getItemFormat());
            }
             */

        } catch (Exception e) {
            throw new SyncSourceException("Error reading calendars: " +
                    e.getMessage(), e);
        }

        try {
            syncItemInfo = Utility.mergeArray(calendarInfo, taskInfo);
        } catch (Exception e) {
            throw new SyncSourceException("Error creating calendars: " +
                    e.getMessage(), e);
        }

        return syncItemInfo;
    }

    // ---------------------------------------------------------- Private methods

    /**
     * Create CalendarManager istance
     * (if not already exist)
     *
     * @return CalendarManager
     * @throws DominoAccessException
     */
    private CalendarManager getEventManager() throws DominoAccessException {
        if (this.cm == null) {
            this.cm = new CalendarManager(this.getDominoWrapper(),
                    this.deviceTimeZone);
        }
        return this.cm;

    }

    /**
     * create a single syncitem from a single
     * Domino Document
     *
     * @param ctmp   Document
     * @param status     char
     * @return SyncItem
     * @throws EntityException
     */
    private SyncItem createSyncItem(ItemCalendar ctmp,
                                    char status)
            throws EntityException {

        SyncItem syncItem = null;

        /**
         * @todo check if ctmp is Event or Todo
         */
        boolean iscal = true;

        if (iscal) {
            if (log.isLoggable(Level.FINEST)) {
                log.finest("create EVENT from Calendar");
            }
            syncItem = createSyncItemFromCalendar(ctmp, status);
        }

        return syncItem;
    }


    /**
     * create a single syncitem with
     * UID and the empty content.
     * This SyncItem is used in the getDeleted
     * operation
     * @param uid
     * @param status
     * @return SyncItem
     */
    private SyncItem createSyncItem(String uid, char status) {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("CalendarManager createSyncItem start ");
        }

        SyncItem syncItem = null;
        String calendarStream = "";


        syncItem = new SyncItemImpl(this, uid, status);

        syncItem.setContent(calendarStream.getBytes());

        if (log.isLoggable(Level.FINEST)) {
            log.finest("ContactManager createSyncItem end ");
        }

        return syncItem;
    }


    /**
     * Create SyncItem from Contact
     *
     * @param calendars  ArrayList
     * @param itemformat String
     * @param dcs        DominoCalendarSyncSource
     * @param status     char
     * @param delete     boolean
     * @return SyncItem[]
     * @throws EntityException
     */
    private SyncItem[] createSyncItems(ArrayList calendars,
                                       char status,
                                       boolean delete)
            throws EntityException {


        SyncItem[] syncItems = null;
        ItemCalendar calendar = null;
        String calendarID = null;
        int l = 0;

        l = calendars.size();
        syncItems = new SyncItem[l];

        if (log.isLoggable(Level.FINEST)) {
            log.finest("ContactManager createSyncItems items number " + l);
        }

        try {
            for (int i = 0; i < l; i++) {
                if (delete) {
                    calendarID = (String) calendars.get(i);
                    syncItems[i] = createSyncItem(calendarID, status);
                } else {
                    calendar = (ItemCalendar) calendars.get(i);
                    syncItems[i] = createSyncItem(calendar, status);
                }
            }
        } catch (Exception e) {
            throw new EntityException(e.getMessage());
        }

        return syncItems;
    }

    /**
     * Removes an SyncItem
     *
     * @param syncItemKey SyncItemKey
     * @param timestamp   Timestamp
     * @throws SyncSourceException
     */
    private void removeItem(SyncItemKey syncItemKey, Timestamp timestamp)
            throws SyncSourceException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("DominoCalendarSyncSource removeSyncItem [" + sourceURI +
                    "]");
        }

        String content = null;
        String id = syncItemKey.getKeyAsString();
        boolean iscal = false;

        if (log.isLoggable(Level.FINEST)) {
            log.finest("remove item with UID: " + id);
        }

        // get content of the item
        try {
            SyncItem syncItem = getSyncItemFromId(syncItemKey);
            content = Utility.getContentFromSyncItem(syncItem);
        } catch (Exception e) {
            throw new SyncSourceException("Error getting the item " +
                    syncItemKey, e);
        }


        try {
            this.cm = getEventManager();
            // @todo TASK next release
            //this.tm = getTodoManager();
        } catch (DominoAccessException e) {
            throw new SyncSourceException("Error connecting calendar/task: " +
                    e.getMessage(), e);
        }

        try {
            iscal = Utility.isCalendar(content, this.getItemFormat());
        } catch (EntityException e) {
            throw new SyncSourceException("Error selectin calendar type: " +
                    e.getMessage(), e);
        }

        try {
            // CALENDAR
            if (iscal) {
                this.cm.removeItem(id);
            }
            // @todo TASK next release
            /*
            else{
                this.tm.removeItem(id);
            }
             */
        } catch (EntityException e) {
            throw new SyncSourceException("Error removing item", e);
        }

    }

    /**
     * Removes all Items
     */
    protected void removeAllItems() throws SyncSourceException {

        try {

            getEventManager().removeAll(Def.CALENDAR_VIEW);

        } catch (Exception ex) {
            throw new SyncSourceException("Error while removing all items", ex);
        }

    }

    /**
     * Get Data from VCard message
     * converting the i-cal item into a Calendar object
     * <p/>
     * the contact object is a com.funambol.domino.items.model.ItemCalendar
     *
     * @param content String
     * @return ItemCalendar
     * @throws EntityException
     */
    private ItemCalendar ical2Calendar(String id, String content)
            throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("ical2Calendar ");
        }

        ByteArrayInputStream buffer = null;
        ICalendarParser parser = null;
        Calendar calendar = null;

        content = SourceUtils.handleLineDelimiting(content);

        try {
            calendar = new Calendar();
            buffer = new ByteArrayInputStream(content.getBytes());
            if ((content.getBytes()).length > 0) {
                parser = new ICalendarParser(buffer,
                        this.deviceTimeZoneDescr,
                        this.deviceCharset);
                calendar = (Calendar) parser.iCalendar();
            }
        } catch (Exception e) {
            throw new EntityException(e.getMessage());
        }


        return new ItemCalendar(id, calendar);
    }

    /**
     * Get Data from XML message
     * converting the xml item into a Calendar object
     * <p/>
     * the contact object is a ItemCalendar
     *
     * @param content String
     * @return Contact
     * @throws EntityException
     */
    private ItemCalendar sife2Calendar(String id, String content)
            throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("sife2Calendar ");
        }

        ByteArrayInputStream buffer = null;
        XMLEventParser parser = null;
        Calendar calendar = null;
        try {
            calendar = new Calendar();
            buffer = new ByteArrayInputStream(content.getBytes());

            if ((content.getBytes()).length > 0) {
                parser = new XMLEventParser(buffer);
                calendar = (Calendar) parser.parse();
            }
        } catch (Exception e) {
            throw new EntityException(e.toString());
        }
        return new ItemCalendar(id, calendar);
    }

    /**
     * Get Data from ItemContact
     * converting the Contact object into an xml item
     *
     * @param calendar ItemCalendar
     * @return String
     * @throws EntityException
     */
    private String calendar2sife(ItemCalendar calendar)
            throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("calendar2sife ");
        }
        String xml = null;
        try {
            CalendarToXML c2xml =
                    new CalendarToXML(this.deviceTimeZone,
                            this.deviceCharset);
            xml = c2xml.convert(calendar);
        } catch (Exception e) {
            throw new EntityException(e);
        }
        return xml;
    }


    /**
     * Get Data from ItemCalendar
     * converting the Calendar object into a i-cal item
     *
     * @param calendar ItemCalendar
     * @return String
     * @throws EntityException
     */
    protected String calendar2ical(ItemCalendar calendar)
            throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("calendar2ical ");
        }
        String icard = null;
        try {
            CalendarToIcalendar c2ical =
                    new CalendarToIcalendar(this.deviceTimeZone,
                            this.deviceCharset);
            icard = c2ical.convert(calendar);
        } catch (Exception e) {
            throw new EntityException(e);
        }
        return icard;
    }


    /**
     * create a single syncitem from a single
     * Domino Document
     * <p/>
     * WORKAROUND:
     * in this case DominoTaskSyncSource = null
     *
     * @param ctmp  Document
     * @param status     char
     * @return SyncItem
     * @throws EntityException
     */
    private SyncItem createSyncItemFromCalendar(ItemCalendar ctmp, char status)
            throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("createSyncItemFromCalendar start");
        }

        SyncItem syncItem = null;
        String stream = null;

        if (this.getItemFormat().equals(Def.ICAL_TYPE)) {
            stream = calendar2ical(ctmp);
        } else {
            stream = calendar2sife(ctmp);
        }

        if (log.isLoggable(Level.FINEST)) {
            log.finest("FOUNDATION BODY " + stream);
        }


        try {
            syncItem = new SyncItemImpl(this, ctmp.getId(), status);
        } catch (Exception e) {
            throw new EntityException(e);
        }

        syncItem.setContent(stream.getBytes());

        if (this.getItemFormat().equals(Def.ICAL_TYPE)) {
            syncItem.setType(Def.ICAL_TYPE);
        } else {
            syncItem.setType(Def.SIFE_TYPE);
        }


        if (log.isLoggable(Level.FINEST)) {
            log.finest("setSyncItemFromStream - end ");
        }

        return syncItem;
    }

    /**
     * Merges server item and client item.
     *
     * @param serverKey  the item's key on the server
     * @param clientItem the item on the client
     * @return true is the content of the client item is changed
     * @throws SyncSourceException
     */
    public boolean mergeSyncItems(SyncItemKey serverKey, SyncItem clientItem)
            throws SyncSourceException {

        //return false;

        ItemCalendar clientCalendar = null;
        ItemCalendar serverCalendar = null;

        Timestamp t = clientItem.getTimestamp();

        try {


            byte[] itemContent =
                    (byte[]) clientItem.getContent();

            if (itemContent == null) {
                itemContent = new byte[0];
            }

            String content = new String(itemContent);

            //
            // 1. get the Calendar object from the itemA (this is the item sent by the client)
            //
            clientCalendar = getCalendar(clientItem.getKey().getKeyAsString(), content);

            //
            // 3. Read the server calendar
            //
            try {
                serverCalendar = getCalendarFromSyncItem(getSyncItemFromId(serverKey));
            } catch (Exception ex) {
                throw new SyncSourceException("Error reading the calendar '" +
                        serverKey.getKeyAsString() + "'", ex);
            }

            MergeResult mergeResult = null;

            try {
                mergeResult = clientCalendar.getEvent().merge(serverCalendar.getEvent());
            } catch (Exception ex) {
                throw new SyncSourceException("Error merging the calendars '" +
                        serverKey.getKeyAsString() + "'", ex);
            }

            if (mergeResult.isSetBRequired()) {
                //
                // We have to update the server calendar
                //
                SyncItem serverSyncItem = createSyncItem(serverCalendar, SyncItemState.SYNCHRONIZED);
                serverSyncItem.setTimestamp(t);
                updateSyncItem(serverSyncItem);

            }

            if (mergeResult.isSetARequired()) {
                //converting the Calendar object into a vcard string

                SyncItem clientSyncItem = createSyncItem(clientCalendar, SyncItemState.SYNCHRONIZED);
                clientSyncItem.setTimestamp(t);
                clientItem = clientSyncItem;
            }

            if (log.isLoggable(Level.FINEST)) {
                log.finest("MergeResult: " + mergeResult);
            }

            return mergeResult.isSetARequired();

        } catch (Exception e) {

            if (log.isLoggable(Level.FINEST)) {
                log.severe("SyncSource error: " + e.getMessage());
                log.throwing(getClass().getName(), "mergeSyncItems", e);
            }

            throw new SyncSourceException("Error merging the calendars", e);
        }

    }


    /**
     * Creates a new Calendar from the given syncItem
     *
     * @param syncItem SyncItem
     * @return ItemCalendar
     */
    private ItemCalendar getCalendarFromSyncItem(SyncItem syncItem)
            throws SyncSourceException {

        String content = null;
        ItemCalendar calendar = null;

        String itemKey = syncItem.getKey().getKeyAsString();

        byte[] itemContent =
                (byte[]) syncItem.getContent();

        if (itemContent == null) {
            itemContent = new byte[0];
        }

        content = new String(itemContent);

        calendar = getCalendar(itemKey, content);

        return calendar;
    }

    private ItemCalendar getCalendar(String id,
                                     String content)
            throws SyncSourceException {

        ItemCalendar calendar = new ItemCalendar();

        if (content != null && content.length() > 0) {

            try {

                if (TYPE_ICAL.equals(getType())) {
                    calendar = ical2Calendar(id, content);
                } else {
                    calendar = sife2Calendar(id, content);
                }
            } catch (EntityException ex) {
                throw new SyncSourceException(ex);
            }
        }

        return calendar;

    }
}