/**
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.domino.engine.source;


import java.io.Serializable;

import java.sql.Timestamp;

import java.util.ArrayList;
import java.util.logging.Level;
import java.util.Date;

import com.funambol.framework.engine.*;
import com.funambol.framework.engine.source.*;
import com.funambol.framework.tools.beans.*;

import com.funambol.domino.items.model.ItemTask;
import com.funambol.domino.engine.cache.SyncItemInfo;
import com.funambol.domino.exception.*;
import com.funambol.domino.items.manager.TaskManager;
import com.funambol.domino.util.*;

import com.funambol.foundation.common.merge.MergeResult;
import com.funambol.foundation.pdi.converter.CalendarToIcalendar;

/**
 * This class define <i>SyncSource</i>
 * between SyncServer and Domino Server task items
 *
 * @author Gilberto Migliavacca
 * @version $Id: DominoTaskSyncSource.java,v 1.1 2006/08/08 08:31:15 gibi Exp $
 */
public final class DominoTaskSyncSource
        extends DominoSyncSource
        implements MergeableSyncSource, Serializable, LazyInitBean {

    // ----------------------------------------------------------- Constants

    // ----------------------------------------------------------- Private data

    private TaskManager manager = null;

    // ----------------------------------------------------------- Public Methods

    /**
     * Init the syncSource
     *
     * @throws BeanInitializationException
     */
    public void init() throws BeanInitializationException {

        super.init();
    }

    /* @see com.funambol.framework.engine.source.SyncSource */
    public SyncItemKey[] getSyncItemKeysFromTwin(SyncItem syncItem) throws SyncSourceException {
        if (log.isLoggable(Level.FINEST)) {
            log.finest("getSyncItemFromTwin [" + sourceURI + "]");
        }

        //Document doc      = null;
        SyncItemKey keys[] = new SyncItemKey[0];
        String content;
        String id = syncItem.getKey().getKeyAsString();
        Date lastModified = syncItem.getTimestamp();

        // get content of the item
        try {
            content = Utility.getContentFromSyncItem(syncItem, false);
        } catch (Exception e) {
            throw new SyncSourceException("Error getting the item " +
                    syncItem, e);
        }


        try {

            this.manager = getManager();

        } catch (DominoAccessException e) {
            throw new SyncSourceException("Error opening TaskManager/TodoManager", e);
        }


        boolean istask;
        try {
            istask = Utility.isTask(content, this.getItemFormat());
        } catch (EntityException e) {
            throw new SyncSourceException("Error converting foundation: " +
                    e.getMessage(), e);
        }

        ItemTask tmpTask = new ItemTask();
        try {
            if (this.getItemFormat().equals(Def.ICAL_TYPE)) {
                tmpTask = ical2Task(id, content);
            } else {
                tmpTask.getPPTask(id, content, lastModified, deviceTimeZone);
            }
        } catch (Exception e) {
            throw new SyncSourceException("Error converting foundation: " +
                    e.getMessage(), e);
        }

        // TASK
        if (istask) {

            ItemTask[] outTasks;
            try {
                outTasks = this.manager.getItemsByWhereClause(tmpTask,
                        this.getItemFormat());
            } catch (EntityException e) {
                throw new SyncSourceException("Error getting twin", e);
            }

            keys = new SyncItemKey[outTasks.length];

            for (int i = 0; i < outTasks.length; i++) {

                keys[i] = new SyncItemKey(outTasks[i].getId());
            }
            return keys;
        }
        // @todo TASK next release
        /*
        else {
         
            Task ctmpout = null;
         
            try {
                ctmpout = this.tm.getItemByWhereClause(ctmp,
                                                       this.getItemFormat(),
                                                       deviceTimezone);
            } catch (EntityException e) {
                throw new SyncSourceException("Error getting twin", e);
            }
         
            if (ctmpout != null) {
                try {
                    syncItem = createSyncItemFromTodo(ctmpout,
                                                      SyncItemState.NEW,
                                                      deviceTimezone);
                } catch (Exception e) {
                    throw new SyncSourceException("Error creating SyncItem", e);
                }
            }
         
        }
         */

        return keys;
    }

    /* @see com.funambol.framework.engine.source.SyncSource */
    public SyncItem updateSyncItem(SyncItem syncItem) throws SyncSourceException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("setSyncItem [" + sourceURI + "]");
        }

        SyncItem syncItemOut = null;
        String content;
        Date lastModified = syncItem.getTimestamp();
        String id;
        String operation;
        char status;

        id = syncItem.getKey().getKeyAsString();

        // get content of the item
        try {
            content = Utility.getContentFromSyncItem(syncItem);
        } catch (Exception e) {
            throw new SyncSourceException("Error getting the item " +
                    syncItem, e);
        }

        if (log.isLoggable(Level.FINEST)) {
            log.finest("setSyncItem content \n" + content);
        }


        ItemTask clientItem = new ItemTask();
        try {
            if (this.getItemFormat().equals(Def.ICAL_TYPE)) {
                clientItem = ical2Task(id, content);
            } else {
                clientItem.getPPTask(id, content, lastModified, deviceTimeZone);
            }
        } catch (Exception e) {
            throw new SyncSourceException("Error converting foundation: " +
                    e.getMessage(), e);
        }

        // start pre-insert

        try {
            this.manager = getManager();
            // @todo TASK next release
            //this.tm = getTodoManager();
        } catch (DominoAccessException e) {
            throw new SyncSourceException("Error connecting task/task: " +
                    e.getMessage(), e);
        }

        boolean istask;
        try {
            istask = Utility.isTask(content, this.getItemFormat());
        } catch (EntityException e) {
            throw new SyncSourceException("Error converting foundation: " +
                    e.getMessage(), e);
        }


        ItemTask serverItem;

        // Task
        if (istask) {

            try {
                serverItem = this.manager.getItemByUID(id, this.getItemFormat());
                operation = Def.OPERATION_UPDATE;
            } catch (Exception ex) {
                String errorMessage = "The item '" + id +
                        "' is mapped but there is a error getting domino document ["
                        + ex.getMessage() + "]";
                if (log.isLoggable(Level.SEVERE)) {
                    log.severe(errorMessage);
                }
                throw new SyncSourceException(errorMessage, ex);
            }


            try {
                if (operation.equals(Def.OPERATION_UPDATE)) {
                    status = SyncItemState.UPDATED;


                    if (conflictResolution !=
                            SyncContext.CONFLICT_RESOLUTION_CLIENT_WINS) {

                        serverItem.copyCommonProperties(clientItem);

                        this.manager.updateItem(serverItem,
                                serverItem,
                                this.getItemFormat(),
                                istask);

                    } else {

                        this.manager.updateItem(serverItem,
                                clientItem,
                                this.getItemFormat(),
                                istask);
                    }


                } else {
                    status = SyncItemState.NEW;
                    serverItem = this.manager.insertItem(clientItem,
                            this.getItemFormat(),
                            istask);
                }

            } catch (Exception e) {
                String errorMessage = "Error setting the item: " + syncItem;
                if (log.isLoggable(Level.SEVERE)) {
                    log.severe(errorMessage);
                }
                throw new SyncSourceException(errorMessage, e);
            }

            if (log.isLoggable(Level.FINEST)) {
                log.finest(" end insert/update. start create SyncItem");
            }

            // create the output syncItem
            try {
                syncItemOut = createSyncItemFromTask(serverItem, status);
            } catch (Exception e) {
                String errorMessage = "Error creating SyncItem: " + syncItem;
                throw new SyncSourceException(errorMessage, e);
            }
        }
        // @todo TASK next release
        /*
        else{
         
            // check if there is a mapped key. If this happend, we must do an update
            if (syncItemExt.getMappedKey() != null) {
                try {
                    ctmpOld = this.tm.getItemByUID(id,this.getItemFormat());
                    operation = Def.OPERATION_UPDATE;
                } catch (Exception ex) {
                    String errorMessage = "The item '" + id +
                        "' is mapped but there is a error getting domino document ["
                        + ex.getMessage() + "]";
                    if (log.isLoggable(Level.SEVERE)) {
                        log.severe(errorMessage);
                    }
                    throw new SyncSourceException(errorMessage, ex);
                }
            } else {
                operation = Def.OPERATION_INSERT;
            }
         
            try {
                if (operation.equals(Def.OPERATION_UPDATE)) {
                    status = SyncItemState.UPDATED;
                    this.tm.updateItem(ctmpOld,
                                       ctmpNew,
                                       this.getItemFormat(),
                                       deviceTimezone);
                } else {
                    status = SyncItemState.NEW;
                    ctmpOld = this.tm.insertItem(ctmpNew,
                                                 this.getItemFormat(),
                                                 deviceTimezone);
                }
            } catch (Exception e) {
                String errorMessage = "Error setting the item: " + syncItemExt;
                if (log.isLoggable(Level.SEVERE)) {
                    log.severe(errorMessage);
                }
                throw new SyncSourceException(errorMessage, e);
            }
         
            if (log.isLoggable(Level.FINEST)) {
                log.finest(" end insert/update. start create SyncItem" );
            }
         
            // create the output syncitem
            try {
                syncItem = createSyncItemFromTodo(ctmpOld, status, deviceTimezone);
            } catch (Exception e) {
                String errorMessage = "Error creating SyncItem: " + syncItemExt;
                throw new SyncSourceException(errorMessage, e);
            }
         
        }
         */

        return syncItemOut;
    }

    /* @see com.funambol.framework.engine.source.SyncSource */
    public SyncItem addSyncItem(SyncItem syncItem) throws SyncSourceException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("setSyncItem [" + sourceURI + "]");
        }

        SyncItem syncItemOut = null;
        String content;
        String id;
        String operation;
        Date lastModified = syncItem.getTimestamp();
        char status;

        id = syncItem.getKey().getKeyAsString();

        // get content of the item
        try {
            content = Utility.getContentFromSyncItem(syncItem);
        } catch (Exception e) {
            throw new SyncSourceException("Error getting the item " +
                    syncItem, e);
        }

        if (log.isLoggable(Level.FINEST)) {
            log.finest("setSyncItem content \n" + content);
        }


        ItemTask tmpTask = new ItemTask();
        try {
            if (this.getItemFormat().equals(Def.ICAL_TYPE)) {
                tmpTask = ical2Task(id, content);
            } else {
                tmpTask.getPPTask(id, content, lastModified, deviceTimeZone);
            }
        } catch (Exception e) {
            throw new SyncSourceException("Error converting foundation: " +
                    e.getMessage(), e);
        }

        // start pre-insert

        try {
            this.manager = getManager();
            // @todo TASK next release
            //this.tm = getTodoManager();
        } catch (DominoAccessException e) {
            throw new SyncSourceException("Error connecting task: " +
                    e.getMessage(), e);
        }

        boolean istask;
        try {
            istask = Utility.isTask(content, this.getItemFormat());
        } catch (EntityException e) {
            throw new SyncSourceException("Error converting foundation: " +
                    e.getMessage(), e);
        }


        ItemTask ctmpOld = null;

        // Task
        if (istask) {

            operation = Def.OPERATION_INSERT;

            try {
                if (operation.equals(Def.OPERATION_UPDATE)) {
                    status = SyncItemState.UPDATED;
                    this.manager.updateItem(ctmpOld,
                            tmpTask,
                            this.getItemFormat(),
                            istask);
                } else {
                    status = SyncItemState.NEW;
                    ctmpOld = this.manager.insertItem(tmpTask,
                            this.getItemFormat(),
                            istask);
                }

            } catch (Exception e) {
                String errorMessage = "Error setting the item: " + syncItem;
                if (log.isLoggable(Level.SEVERE)) {
                    log.severe(errorMessage);
                }
                throw new SyncSourceException(errorMessage, e);
            }

            if (log.isLoggable(Level.FINEST)) {
                log.finest(" end insert/update. start create SyncItem");
            }

            // create the output syncItem
            try {
                syncItemOut = createSyncItemFromTask(ctmpOld, status);
            } catch (Exception e) {
                String errorMessage = "Error creating SyncItem: " + syncItem;
                throw new SyncSourceException(errorMessage, e);
            }
        }
        // @todo TASK next release
        /*
        else{
         
            // check if there is a mapped key. If this happend, we must do an update
            if (syncItemExt.getMappedKey() != null) {
                try {
                    ctmpOld = this.tm.getItemByUID(id,this.getItemFormat());
                    operation = Def.OPERATION_UPDATE;
                } catch (Exception ex) {
                    String errorMessage = "The item '" + id +
                        "' is mapped but there is a error getting domino document ["
                        + ex.getMessage() + "]";
                    if (log.isLoggable(Level.SEVERE)) {
                        log.severe(errorMessage);
                    }
                    throw new SyncSourceException(errorMessage, ex);
                }
            } else {
                operation = Def.OPERATION_INSERT;
            }
         
            try {
                if (operation.equals(Def.OPERATION_UPDATE)) {
                    status = SyncItemState.UPDATED;
                    this.tm.updateItem(ctmpOld,
                                       ctmpNew,
                                       this.getItemFormat(),
                                       deviceTimezone);
                } else {
                    status = SyncItemState.NEW;
                    ctmpOld = this.tm.insertItem(ctmpNew,
                                                 this.getItemFormat(),
                                                 deviceTimezone);
                }
            } catch (Exception e) {
                String errorMessage = "Error setting the item: " + syncItemExt;
                if (log.isLoggable(Level.SEVERE)) {
                    log.severe(errorMessage);
                }
                throw new SyncSourceException(errorMessage, e);
            }
         
            if (log.isLoggable(Level.FINEST)) {
                log.finest(" end insert/update. start create SyncItem" );
            }
         
            // create the output syncitem
            try {
                syncItem = createSyncItemFromTodo(ctmpOld, status, deviceTimezone);
            } catch (Exception e) {
                String errorMessage = "Error creating SyncItem: " + syncItemExt;
                throw new SyncSourceException(errorMessage, e);
            }
         
        }
         */

        return syncItemOut;
    }


    /* @see com.funambol.framework.engine.source.SyncSource */
    public void removeSyncItem(SyncItemKey syncItemKey, Timestamp timestamp,
                               boolean softDelete)
            throws SyncSourceException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("remove item softDelete: " + softDelete);
        }

        if (softDelete) {
            // nothing
        } else {
            // if false
            removeItem(syncItemKey, timestamp);
        }
    }

    /* @see com.funambol.framework.engine.source.SyncSource */
    public SyncItem getSyncItemFromId(SyncItemKey syncItemKey) throws SyncSourceException {
        if (log.isLoggable(Level.FINEST)) {
            log.finest("getSyncItemFromId start");
        }

        String id;
        SyncItem syncItem = null;

        id = syncItemKey.getKeyAsString();

        try {
            this.manager = getManager();
            // @todo TASK next release
            //this.tm = getTodoManager();
        } catch (DominoAccessException e) {
            throw new SyncSourceException("Error connecting task: " +
                    e.getMessage(), e);
        }

        boolean istask;
        try {
            istask = this.manager.isTask(this.getDominoWrapper(), id);
        } catch (EntityException e) {
            throw new SyncSourceException("Error converting foundation: " +
                    e.getMessage(), e);
        }

        ItemTask ctmp;
        try {
            // Task
            if (istask) {
                ctmp = this.manager.getItemByUID(id, this.getItemFormat());
                syncItem = createSyncItemFromTask(ctmp, SyncItemState.NEW);
            }
            // @todo TASK next release
            /*
            else {
                ctmp = this.tm.getItemByUID(id, this.getItemFormat());
                syncItem = createSyncItemFromTodo(ctmp, SyncItemState.NEW);
            }
             */
        } catch (Exception e) {
            throw new SyncSourceException("Error in creating SyncItem ", e);
        }

        return syncItem;
    }


    /**
     * @return SyncItemInfo[]
     * @throws SyncSourceException
     */
    public SyncItemInfo[] getAllSyncItemInfo() throws
            SyncSourceException {
        if (log.isLoggable(Level.FINEST)) {
            log.finest("getAllSyncItemInfo start");
        }

        SyncItemInfo[] taskInfo;

        try {
            this.manager = getManager();

        } catch (Exception e) {
            throw new SyncSourceException("Error connecting tasks: " +
                    e.getMessage(), e);
        }

        try {

            // only task
            taskInfo = this.manager.getAllSyncItemInfo(Def.TODO_VIEW);


        } catch (Exception e) {
            throw new SyncSourceException("Error reading tasks: " +
                    e.getMessage(), e);
        }


        return taskInfo;
    }

    // ---------------------------------------------------------- Private methods

    /**
     * Create TaskManager istance
     * (if not already exist)
     *
     * @return TaskManager
     * @throws DominoAccessException
     */
    private TaskManager getManager() throws DominoAccessException {

        if (this.manager == null) {
            this.manager = new TaskManager(this.getDominoWrapper());
        }
        return this.manager;

    }


    private SyncItem createSyncItem(ItemTask tmpTask,
                                    char status)
            throws EntityException {

        SyncItem syncItem;
        if (log.isLoggable(Level.FINEST)) {
            log.finest("create TASK from Task");
        }
        syncItem = createSyncItemFromTask(tmpTask, status);
        return syncItem;
    }

    /*
    private SyncItem createSyncItem(String uid, char status) {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("TaskManager createSyncItem start ");
        }

        SyncItem syncItem;
        String taskStream = "";


        syncItem = new SyncItemImpl(this, uid, status);

        syncItem.setContent(taskStream.getBytes());

        if (log.isLoggable(Level.FINEST)) {
            log.finest("TaskManager createSyncItem end ");
        }

        return syncItem;
    }

    private SyncItem[] createSyncItems(ArrayList tasks,
                                       char status,
                                       boolean delete)
            throws EntityException {


        SyncItem[] syncItems;
        ItemTask task;
        String taskId;
        int l;

        l = tasks.size();
        syncItems = new SyncItem[l];

        if (log.isLoggable(Level.FINEST)) {
            log.finest("TaskManager createSyncItems items number " + l);
        }

        try {
            for (int i = 0; i < l; i++) {
                if (delete) {
                    taskId = (String) tasks.get(i);
                    syncItems[i] = createSyncItem(taskId, status);
                } else {
                    task = (ItemTask) tasks.get(i);
                    syncItems[i] = createSyncItem(task, status);
                }
            }
        } catch (Exception e) {
            throw new EntityException(e.getMessage());
        }

        return syncItems;
    }
    */

    /**
     * Removes an SyncItem
     *
     * @param syncItemKey SyncItemKey
     * @param timestamp   Timestamp
     * @throws SyncSourceException
     * @noinspection UNUSED_SYMBOL
     */
    private void removeItem(SyncItemKey syncItemKey, Timestamp timestamp)
            throws SyncSourceException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("DominoTaskSyncSource removeSyncItem [" + sourceURI +
                    "]");
        }

        String content;
        String id = syncItemKey.getKeyAsString();
        boolean istask;

        if (log.isLoggable(Level.FINEST)) {
            log.finest("remove item with UID: " + id);
        }

        // get content of the item
        try {
            SyncItem syncItem = getSyncItemFromId(syncItemKey);
            content = Utility.getContentFromSyncItem(syncItem);
        } catch (Exception e) {
            throw new SyncSourceException("Error getting the item " +
                    syncItemKey, e);
        }


        try {
            this.manager = getManager();

        } catch (DominoAccessException e) {
            throw new SyncSourceException("Error connecting task: " +
                    e.getMessage(), e);
        }

        try {
            istask = Utility.isTask(content, this.getItemFormat());
        } catch (EntityException e) {
            throw new SyncSourceException("Error selectin task type: " +
                    e.getMessage(), e);
        }

        try {

            if (istask) {
                this.manager.removeItem(id);
            }

        } catch (EntityException e) {
            throw new SyncSourceException("Error removing item", e);
        }

    }

    /**
     * Removes all Items
     */
    protected void removeAllItems() throws SyncSourceException {

        try {

            getManager().removeAll(Def.TODO_VIEW);

        } catch (Exception ex) {
            throw new SyncSourceException("Error while removing all items", ex);
        }

    }

    /**
     * Get Data from ICal message
     * converting the i-cal item into a Task object
     * <p/>
     * the task object is a com.funambol.domino.items.model.ItemTask
     * todo
     *
     * @param content String
     * @return ItemTask
     * @throws EntityException
     * @noinspection UNUSED_THROWS,UNUSED_SYMBOL
     */
    private ItemTask ical2Task(String id, String content)
            throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("ical2Task ");
        }
        return null;
    }


    /**
     * Get Data from ItemTask
     * converting the Task object into a i-cal item
     *
     * @param task ItemTask
     * @return String
     * @throws EntityException
     */
    protected String task2ical(ItemTask task)
            throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("task2ical ");
        }

        //TODO
        String icard;
        try {
            CalendarToIcalendar task2ical =
                    new CalendarToIcalendar(this.deviceTimeZone,
                            this.deviceCharset);
            icard = task2ical.convert(task);
        } catch (Exception e) {
            throw new EntityException(e);
        }
        return icard;
    }


    private SyncItem createSyncItemFromTask(ItemTask tmpTask, char status)
            throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("createSyncItemFromTask start");
        }

        SyncItem syncItem;
        String stream;
        try {

            if (this.getItemFormat().equals(Def.ICAL_TYPE)) {
                stream = task2ical(tmpTask);
            } else {
                stream = tmpTask.toXml();
            }

            if (log.isLoggable(Level.FINEST)) {
                log.finest("FOUNDATION BODY " + stream);
            }


            syncItem = new SyncItemImpl(this, tmpTask.getId(), status);


            syncItem.setContent(stream.getBytes());

            if (this.getItemFormat().equals(Def.ICAL_TYPE)) {
                syncItem.setType(Def.ICAL_TYPE);
            } else {
                syncItem.setType(Def.SIFT_TYPE);
            }

        } catch (Exception e) {
            throw new EntityException(e);
        }


        if (log.isLoggable(Level.FINEST)) {
            log.finest("setSyncItemFromStream - end ");
        }

        return syncItem;
    }

    /**
     * Merges server item and client item.
     *
     * @param serverKey  the item's key on the server
     * @param clientItem the item on the client
     * @return true is the content of the client item is changed
     * @throws SyncSourceException
     */
    public boolean mergeSyncItems(SyncItemKey serverKey, SyncItem clientItem)
            throws SyncSourceException {

        //return false;

        ItemTask clientTask;
        ItemTask serverTask;

        Timestamp t = clientItem.getTimestamp();

        try {


            byte[] itemContent = clientItem.getContent();

            if (itemContent == null) {
                itemContent = new byte[0];
            }

            String content = new String(itemContent);

            //
            // 1. get the Task object from the itemA (this is the item sent by the client)
            //
            clientTask = getTask(clientItem.getKey().getKeyAsString(),
                    content,
                    clientItem.getTimestamp());

            //
            // 3. Read the server task
            //
            try {
                serverTask = getTaskFromSyncItem(getSyncItemFromId(serverKey));
            } catch (Exception ex) {
                throw new SyncSourceException("Error reading the task '" +
                        serverKey.getKeyAsString() + "'", ex);
            }

            MergeResult mergeResult;

            try {
                mergeResult = clientTask.merge(serverTask);
            } catch (Exception ex) {
                throw new SyncSourceException("Error merging the tasks '" +
                        serverKey.getKeyAsString() + "'", ex);
            }

            if (mergeResult.isSetBRequired()) {
                //
                // We have to update the server task
                //
                SyncItem serverSyncItem = createSyncItem(serverTask, SyncItemState.SYNCHRONIZED);
                serverSyncItem.setTimestamp(t);
                updateSyncItem(serverSyncItem);

            }

            if (mergeResult.isSetARequired()) {
                //converting the Task object into a ical string

                SyncItem clientSyncItem = createSyncItem(clientTask, SyncItemState.SYNCHRONIZED);
                clientSyncItem.setTimestamp(t);
            }

            if (log.isLoggable(Level.FINEST)) {
                log.finest("MergeResult: " + mergeResult);
            }

            return mergeResult.isSetARequired();

        } catch (Exception e) {

            if (log.isLoggable(Level.FINEST)) {
                log.severe("SyncSource error: " + e.getMessage());
                log.throwing(getClass().getName(), "mergeSyncItems", e);
            }

            throw new SyncSourceException("Error merging the tasks", e);
        }

    }


    /**
     * Creates a new Task from the given syncItem
     *
     * @param syncItem SyncItem
     * @return ItemTask
     */
    private ItemTask getTaskFromSyncItem(SyncItem syncItem)
            throws SyncSourceException {

        String content;
        ItemTask task;

        String itemKey = syncItem.getKey().getKeyAsString();

        byte[] itemContent = syncItem.getContent();

        if (itemContent == null) {
            itemContent = new byte[0];
        }

        content = new String(itemContent);

        task = getTask(itemKey, content, syncItem.getTimestamp());

        return task;
    }

    private ItemTask getTask(String id, String content, Date lastModified)
            throws SyncSourceException {

        ItemTask task = new ItemTask();

        if (content != null && content.length() > 0) {

            try {

                if (TYPE_ICAL.equals(getType())) {
                    task = ical2Task(id, content);
                } else {
                    task.getPPTask(id, content, lastModified, deviceTimeZone);
                }
            } catch (Exception ex) {
                throw new SyncSourceException(ex);
            }
        }

        return task;

    }
}