/**
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.domino.items.dao;

import java.util.ArrayList;
import java.util.logging.Level;

import com.funambol.domino.engine.cache.SyncItemInfo;
import lotus.domino.NotesException;

import lotus.domino.View;
import lotus.domino.Document;

import com.funambol.domino.exception.*;
import com.funambol.domino.util.Def;
import com.funambol.domino.util.DominoWrapper;
import com.funambol.domino.util.Utility;
import lotus.domino.Database;
import lotus.domino.DocumentCollection;

/*
 * This class implements methods to access calendar data
 * in domino server datastore
 *
 * @@author Gilberto Migliavacca
 * @@version $Id: CalendarDAO.java,v 1.1 2006/08/08 08:32:36 gibi Exp $
 *
 **/
public class CalendarDAO extends EntityDAO {
    
    
    //------------------------------------------------------------- Costructors
    /**
     *
     */
    public CalendarDAO(DominoWrapper dominoWrapper){
        super(dominoWrapper);
    }
    
    
    //------------------------------------------------------------- Public methods
    /**
     *
     * @return SyncItemInfo[]
     * @throws EntityException
     */
    public SyncItemInfo[] getAllSyncItemInfo(View views) throws EntityException{
        
        if (log.isLoggable(Level.FINEST)) {
            log.finest("CalendarDAO start getAllSyncItemInfo");
        }
        
        ArrayList allSyncItemInfo = new ArrayList() ;
        SyncItemInfo syncItemInfo = null;
        try {
            
            Database db = wrapper.getDatabase();
            DocumentCollection view = db.search("Form =\"Appointment\" & " +
                    "($CSFlags=\"i\" | $CSFlags=LEI | Repeats!=\"1\")");
            
            Document doc = view.getFirstDocument();
            
            while (doc != null) {
                
                syncItemInfo = new SyncItemInfo();
                syncItemInfo.setLastModified(
                        Utility.dominoDateTimeToTimestamp(doc.getLastModified()));
                syncItemInfo.setKey(doc.getUniversalID());
                allSyncItemInfo.add(syncItemInfo);
                
                doc = view.getNextDocument(doc);
            }
            
        }catch (Exception e){
            throw new EntityException(e);
        }
        
        if (log.isLoggable(Level.FINEST)) {
            log.finest("CalendarDAO end getAllSyncItemInfo");
        }
        
        return (SyncItemInfo[])allSyncItemInfo.toArray(new SyncItemInfo[0]);
    }
    
    
    /**
     *
     * @param UID String
     * @param dominoWrapper DominoWrapper
     * @throws EntityException
     */
    public void removeItem(String UID,DominoWrapper dominoWrapper)
    throws EntityException {
        try {
            
            Document docToDelete = getItemByUID(UID,dominoWrapper);
            
            if(docToDelete.isResponse()) {
                try {
                    
                    Database db = wrapper.getDatabase();
                    Document parent = db.getDocumentByUNID(
                            docToDelete.getParentDocumentUNID());
                    if(parent != null)
                         parent.remove(true);
                    
                } catch (NotesException ex) {
                    log.finest(
              "CalendarDAO - Parent doc don't exist or has been already removed");
                }
            }
            
            docToDelete.remove(true);
            
        }catch (Exception e){
            throw new EntityException(e);
        }
    }
    
}
