/**
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.domino.items.dao;


import com.funambol.domino.engine.cache.SyncItemInfo;
import java.util.logging.Logger;
import java.util.logging.Level;

import lotus.domino.View;
import lotus.domino.Document;
import lotus.domino.NotesException;

import com.funambol.domino.exception.EntityException;
import com.funambol.domino.util.DominoWrapper;
import com.funambol.domino.util.Def;
import com.funambol.framework.logging.Sync4jLogger;

/**
 * This class implements methods to access data
 * in domino server datastore.
 *
 * @author Gilberto Migliavacca
 * @version $Id: EntityDAO.java,v 1.1 2006/08/08 08:32:37 gibi Exp $
 */
public abstract class EntityDAO {
    
    
    // ------------------------------------------------------------ Private data
    
    protected Logger  log = Sync4jLogger.getLogger(Def.LOGGER_NAME);
    
    protected DominoWrapper wrapper = null;
    // -------------------------------------------------------------- Properties
    
    /**
     *
     */
    public EntityDAO(DominoWrapper dominoWrapper){
        wrapper = dominoWrapper;
    }
    
    
    // -----------------------------------------------------------Public Methods
    
    
    /**
     *
     * @param UID String
     * @param dominoWrapper DominoWrapper
     * @throws EntityException
     */
    public void removeItem(String UID,DominoWrapper dominoWrapper)
    throws EntityException {
        try {
            
            Document docToDelete = getItemByUID(UID,dominoWrapper);
            
            docToDelete.remove(true);
            
        }catch (Exception e){
            throw new EntityException(e);
        }
    }
    
    /**
     * Clears all items on server
     * @param viewName String
     * @throws EntityException
     */
    public void removeAll(String viewName, DominoWrapper dominoWrapper)
    throws EntityException {
        try {
            
            View view = dominoWrapper.getDatabase().getView(viewName);
            view.refresh();
            view.getAllEntries().removeAll(true);
        } catch (Exception e) {
            throw new EntityException(e);
        }
        
    }
    
    /**
     * saves the document in the DB
     *
     * @param docToUptdate Document
     * @throws EntityException
     */
    public void updateItem(Document docToUptdate) throws EntityException {
        
        if (log.isLoggable(Level.FINEST)) {
            log.finest("DAO start updateItem ");
        }
        
        try {
            
            docToUptdate.save();
        } catch (Exception e) {
            throw new EntityException(e);
        }
    }
    
    
    /**
     *
     * @param doc Document
     * @return Document
     * @throws EntityException
     */
    public Document insertItem(Document doc)throws EntityException {
        
        if (log.isLoggable(Level.FINEST)) {
            log.finest("DAO start insertItem");
        }
        
        boolean ret      = false;
        try {
            // It doesn't throw the error if check fails
            ret = doc.computeWithForm(true, false);
            // force insert
            ret = true;
            if (ret) {
                doc.save(true, true, true);
            }
        } catch (NotesException e) {
            throw new EntityException(e);
        }
        return doc;
    }
    
    
    
    /**
     * serch a document using UID
     *
     * @param UID String
     * @param dominoWrapper DominoWrapper
     * @return Document
     * @throws EntityException
     */
    public Document getItemByUID(String UID, DominoWrapper dominoWrapper)
    throws EntityException {
        
        if (log.isLoggable(Level.FINEST)) {
            log.finest("DAO start getItemByUID " + UID);
        }
        Document doc = null;
        try {
            doc = dominoWrapper.getDatabase().getDocumentByUNID(UID);
        }catch (Exception e){
            throw new EntityException("EntityDao - Error getting Domino Doc",e);
        }
        
        return doc;
    }
    
    /**
     *
     * @return SyncItemInfo[]
     * @throws EntityException
     */
    public abstract SyncItemInfo[] getAllSyncItemInfo(View views) throws EntityException;
}