/**
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.domino.items.manager ;

import com.funambol.domino.document.RecurrentDocument;
import com.funambol.foundation.pdi.event.RecurrencePattern;

import java.util.logging.Level;

import com.funambol.foundation.pdi.event.Reminder;
import com.funambol.domino.engine.cache.SyncItemInfo;


import lotus.domino.Document;
import lotus.domino.DocumentCollection;
import lotus.domino.DateTime;
import lotus.domino.NotesException;
import lotus.domino.View;
import lotus.domino.Database;
import lotus.domino.RichTextItem;


import com.funambol.domino.items.dao.CalendarDAO;
import com.funambol.domino.exception.*;
import com.funambol.domino.items.model.ItemCalendar;
import com.funambol.domino.util.*;

import java.util.TimeZone;

/**
 * @author Gilberto Migliavacca
 */
public class CalendarManager extends EntityManager {


    /**
     *
     */
    public CalendarManager() {
    }

    /**
     * @param wrapper DominoWrapper
     * @throws DominoAccessException
     */
    public CalendarManager(DominoWrapper wrapper, TimeZone timeZone)
            throws DominoAccessException {

        dominoWrapper = wrapper;
        this.timezone = timeZone;
        dao = new CalendarDAO(wrapper);
    }

    // ---------------------------------------------------------- Public Methods


    /**
     * @param UID String
     * @return ItemCalendar
     * @throws EntityException
     */
    public ItemCalendar getItemByUID(String UID,
                                     String itemFormat)
            throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("getItemByUID start " + UID);
        }
        Document doc;
        try {
            doc = dao.getItemByUID(UID, this.dominoWrapper);
        } catch (Exception e) {
            throw new EntityException(e);
        }

        // converts domino doc in event
        ItemCalendar ctmpout = null;
        if (doc != null) {
            try {
                ctmpout = this.createFoundationEvent(doc, itemFormat);
            } catch (Exception e) {
                throw new EntityException(e);
            }
        }

        return ctmpout;
    }


    /**
     * @return SyncItemInfo[]
     * @throws EntityException
     */
    public SyncItemInfo[] getAllSyncItemInfo(String viewname)
            throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("getAllSyncItemInfo Start");
        }

        SyncItemInfo[] syncItemInfos;
        try {
            View view = this.dominoWrapper.getView(viewname);
            syncItemInfos = dao.getAllSyncItemInfo(view);
        } catch (Exception e) {
            throw new EntityException(e);
        }

        if (log.isLoggable(Level.FINEST)) {
            log.finest("Domino List with " + syncItemInfos.length + " Document");
        }

        return syncItemInfos;

    }


    /**
     * updates a document defined by UID
     * In the parameter content there are all the
     * value from the client
     *
     * @param oldCalendar ItemCalendar
     * @param newCalendar ItemCalendar
     * @param itemFormat  String
     * @param iscal       boolean
     * @throws EntityException
     */
    public void updateItem(ItemCalendar oldCalendar,
                           ItemCalendar newCalendar,
                           String itemFormat,
                           boolean iscal)
            throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("Start - Calendar updateItem " +
                    "itemFormat (" + itemFormat + ") " +
                    "is calendar type (" + iscal + ")");
        }

        Document doc = null;
        String uid = oldCalendar.getId();
        Document docToUptdate = dao.getItemByUID(uid, this.dominoWrapper);

        if (itemFormat.equals(Def.ICAL_TYPE)) {
            // check if the content contains VEVENT or VTODO
            if (iscal) {
                doc = updateDominoCalendar(docToUptdate,
                        newCalendar,
                        itemFormat);
            }
        } else { // @todo check XML
            doc = updateDominoCalendar(docToUptdate,
                    newCalendar,
                    itemFormat);
        }

        try {
            dao.updateItem(doc);
        } catch (EntityException e) {
            throw new EntityException(e);
        }

    }


    /**
     * @param calendar   ItemCalendar
     * @param itemFormat String
     * @param iscal      boolean
     * @return ItemCalendar
     * @throws EntityException
     */
    public ItemCalendar insertItem(ItemCalendar calendar,
                                   String itemFormat,
                                   boolean iscal)
            throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("Start - Calendar insertItem " +
                    "format (" + itemFormat + ")" +
                    "is calendar type (" + iscal + ")");
        }

        Document docNew;
        Document doc = null;

        if (itemFormat.equals(Def.ICAL_TYPE)) {
            // check if the content contains VEVENT or VTODO
            if (iscal) {
                doc = createDominoCalendar(calendar,
                        itemFormat,
                        Def.CALENDAR_FORM_NAME);
            }
        } else { // @todo check XML
            doc = createDominoCalendar(calendar,
                    itemFormat,
                    Def.CALENDAR_FORM_NAME);
        }

        try {
            docNew = dao.insertItem(doc);
        } catch (EntityException e) {
            throw new EntityException(e);
        }

        // converts domino doc in event
        ItemCalendar ctmpout;
        try {
            ctmpout = this.createFoundationEvent(docNew, itemFormat);
        } catch (Exception e) {
            throw new EntityException(e);
        }

        return ctmpout;
    }

    /**
     * checks if a calendar is already inserted in the db
     * and returns the Document in the DB
     * we use: startdate, enddate, location as search criteria
     *
     * @param ctmp       ItemCalendar
     * @param itemFormat String
     * @return found ItemCalendar already present
     */
    public ItemCalendar[] getItemsByWhereClause(ItemCalendar ctmp,
                                                String itemFormat)
            throws EntityException {


        if (log.isLoggable(Level.FINEST)) {
            log.finest("getItemByWhereClause start ");
        }

        ItemCalendar calendars[] = null;

        // start date
        String clientStartDate = ctmp.getEvent().getDtStart().getPropertyValueAsString();
        // end date
        String clientEndDate = ctmp.getEvent().getDtEnd().getPropertyValueAsString();
        // location
        String location = ctmp.getEvent().getLocation().getPropertyValueAsString();
        // subject
        String subject = ctmp.getEvent().getSummary().getPropertyValueAsString();

        if (log.isLoggable(Level.FINEST)) {
            log.finest("baseline parameters: " +
                    "\nsubject:    " + subject +
                    "\nlocation:   " + location +
                    "\nstart date: " + clientStartDate +
                    "\nend date:   " + clientEndDate);
        }

        // the formula is different
        String formula = "Form =\"Appointment\" & " +
                "($CSFlags=\"i\" | $CSFlags=LEI | Repeats!=\"1\") & ";

        if (ctmp.getEvent().isAllDay()) {
            DateTime domStartDate = Utility.setAllDayDomino_Date(dominoWrapper,
                    clientStartDate);
            String startDate = "" + domStartDate;

            formula += Def.SUBJECT + " = \"" + subject + "\" & " +
                    Def.LOCATION + " = \"" + location + "\" & " +
                    Def.START_DATE + " = @Date([" + startDate + "])";


        } else {

            DateTime domStartDate = Utility.setDominoDateTime(dominoWrapper,
                    clientStartDate);
            String startDate = "" + domStartDate;

            DateTime domEndDate = Utility.setDominoDateTime(dominoWrapper,
                    clientEndDate);
            String endDate = "" + domEndDate;

            formula += Def.SUBJECT + " = \"" + subject + "\" & " +
                    Def.LOCATION + " = \"" + location + "\" & " +
                    Def.START_DATE_TIME + "=@Date([" + startDate + "]) & " +
                    Def.END_DATE_TIME + "=@Date([" + endDate + "]) ";
        }


        try {

            if (log.isLoggable(Level.FINEST)) {
                log.finest("Domino Formula " + formula);
            }

            DocumentCollection dc = dominoWrapper.getDatabase().search(formula);

            if (log.isLoggable(Level.FINEST)) {
                log.finest("Found calendar number " + dc.getCount());
            }

            int itemsCount = dc.getCount();
            if (itemsCount > 0) {
                if (log.isLoggable(Level.FINEST)) {
                    log.finest("Calendar already Exist");
                }

                calendars = new ItemCalendar[itemsCount];
                for (int i = 0; i < itemsCount; i++) {
                    calendars[i] = this.createFoundationEvent(dc.getNthDocument(i + 1), itemFormat);
                }
            }

        } catch (Exception e) {
            throw new EntityException(e);
        }

        if (calendars == null) {
            return new ItemCalendar[0];
        }

        return calendars;
    }

// ---------------------------------------------------------- Public Methods

    /**
     * get Documento from Domino using UID
     * check if the Docoment.FORM property is: Appointment or Task
     *
     * @param dominoWrapper DominoWrapper
     * @param UID           String
     * @throws EntityException
     */
    public boolean isCalendar(DominoWrapper dominoWrapper, String UID)
            throws EntityException {


        Document doc;
        try {
            doc = dao.getItemByUID(UID, dominoWrapper);
        } catch (Exception e) {
            throw new EntityException(e);
        }

        String form;
        try {
            form = doc.getItemValueString(Def.FORM);
        } catch (Exception e) {
            throw new EntityException("Error getting " + Def.FORM, e);
        }

        if (log.isLoggable(Level.FINEST)) {
            log.finest("Form: " + form);
        }

        // calendar
        if (form.equalsIgnoreCase(Def.CALENDAR_FORM_NAME)) {
            if (log.isLoggable(Level.FINEST)) {
                log.finest("create EVENT from Calendar");
            }
            return true;
        }
        // task
        else if (form.equalsIgnoreCase(Def.TODO_FORM_NAME)) {
            if (log.isLoggable(Level.FINEST)) {
                log.finest("create TASK from TodoCalendar");
            }
            return false;
        }
        // otherwise error
        else {
            throw new EntityException("Error switching Calendar/Task");
        }

    }

//----------------------------------------------------------- PRIVATE METHODS


    /**
     * @param ctmp           ItemCalendar
     * @param itemFormat     String
     * @param dominoFormName String
     * @return Document
     * @throws EntityException
     */
    private Document createDominoCalendar(ItemCalendar ctmp,
                                          String itemFormat,
                                          String dominoFormName)
            throws EntityException {


        if (log.isLoggable(Level.FINEST)) {
            log.finest("CALENDAR: CalendarManager content " +
                    "\n CALENDAR: CalendarManager format " + itemFormat);
        }

        Document docToReturn;
        String dominoCalendarType;

        // convert client calendar type into domino calendar type
        dominoCalendarType = UtilityCalendar.setDominoCalendarType(ctmp, itemFormat);

        switch (Integer.parseInt(dominoCalendarType)) {
            case 1: {
                // anniversary
                docToReturn = createDominoAnniversary(ctmp, dominoFormName, itemFormat);
                break;
            }
            case 2: {
                // all day
                docToReturn = createDominoAllDayEvent(ctmp, dominoFormName, itemFormat);
                break;
            }
            case 3: {
                if (log.isLoggable(Level.FINEST)) {
                    log.finest("DOMINO MEETING NOT IMPLEMENTED YET");
                }
                //docToReturn = createDominoMeeting(ctmp, dominoFormName,itemFormat);
                docToReturn = createDominoAppointment(ctmp, dominoFormName, itemFormat);
                break;
            }
            case 4: {
                // reminder
                docToReturn = createDominoReminder(ctmp, dominoFormName, itemFormat);
                break;
            }
            default: {
                // appointment
                docToReturn = createDominoAppointment(ctmp, dominoFormName, itemFormat);
            }
        }

        // confidential
        // sensitity tag for SIF-XML e CLASS for ICAL
        String clientConfidential = ctmp.getEvent().getClassEvent().getPropertyValueAsString();
        String dominoConfidential = Utility.setDominoConfidential(
                clientConfidential,
                itemFormat);
        if (dominoConfidential != null) {
            try {
                docToReturn.appendItemValue(Def.ORG_CONFIDENTIAL, dominoConfidential);
            } catch (NotesException ex) {
                log.finest(ex.getMessage());
            }
        }

        //alarm
        Reminder clientAlarmRem = ctmp.getEvent().getReminder();
        if (clientAlarmRem != null) {

            if (clientAlarmRem.isActive()) {
                try {
                    int min = ctmp.getEvent().getReminder().getMinutes();
                    docToReturn.appendItemValue("Alarms", "1");
                    docToReturn.appendItemValue("$Alarm", "1");
                    docToReturn.appendItemValue("$AlarmUnit",
                            Utility.calcAlarmOffsetUnit(min));
                    docToReturn.appendItemValue("$AlarmOffset",
                            new Integer(-min));
                } catch (NotesException ex) {
                    log.finest(ex.getMessage());
                }
            }
        }

        //recurrence
        if (ctmp.getEvent().isRecurrent()) {

            try {
                RecurrentDocument recDoc = new RecurrentDocument(docToReturn, timezone);
                RecurrencePattern rp = ctmp.getEvent().getRecurrencePattern();
                String start = ctmp.getEvent().getDtStart().getPropertyValueAsString();
                String end = ctmp.getEvent().getDtEnd().getPropertyValueAsString();
                recDoc.setRecurrencePattern(rp, start, end);

            } catch (Exception ex) {
                log.finest(ex.getMessage());
            }

        }


        if (log.isLoggable(Level.FINEST)) {
            log.finest("createDominoCalendar end ");
        }

        return docToReturn;
    }

    /**
     * @param docToUptdate Document
     * @param calendar     ItemCalendar
     * @param itemFormat   String
     * @return Document
     * @throws EntityException
     */
    private Document updateDominoCalendar(Document docToUptdate,
                                          ItemCalendar calendar,
                                          String itemFormat)
            throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("update Calendar\n: ... ");
        }

        Document docToReturn = null;
        String dominoCalendarType;

        dominoCalendarType = UtilityCalendar.setDominoCalendarType(calendar, itemFormat);

        if (dominoCalendarType.equals("0")) {
            docToReturn = updateDominoAppointment(docToUptdate, calendar, itemFormat);
        } else if (dominoCalendarType.equals("1")) {
            docToReturn = updateDominoAnniversary(docToUptdate, calendar, itemFormat);
        } else if (dominoCalendarType.equals("2")) {
            docToReturn = updateDominoAllDayEvent(docToUptdate, calendar, itemFormat);
        } else if (dominoCalendarType.equals("3")) {
            if (log.isLoggable(Level.FINEST)) {
                log.finest("NOT IMPLEMENTED YET");
            }
            //docToReturn = updateDominoMeeting(docToUptdate,ctmp,itemFormat);
            docToReturn = updateDominoAppointment(docToUptdate, calendar, itemFormat);
        } else if (dominoCalendarType.equals("4")) {
            docToReturn = updateDominoReminder(docToUptdate, calendar, itemFormat);
        }


        try {

            // confidential
            // sensitity tag for SIF-XML e CLASS for ICAL

            String clientConfidential = calendar.getEvent().getClassEvent().getPropertyValueAsString();
            String dominoConfidential = Utility.setDominoConfidential(
                    clientConfidential,
                    itemFormat);
            if (dominoConfidential != null) {
                setItemValue(docToReturn, Def.ORG_CONFIDENTIAL, dominoConfidential);
            } else {
                setItemValue(docToReturn, Def.ORG_CONFIDENTIAL, null);
                docToReturn.removeItem(Def.PUBLIC_ACCESS);
            }
        } catch (NotesException ex) {
            log.finest(ex.getMessage());
        }

        //alarm
        Reminder clientAlarmRem = calendar.getEvent().getReminder();
        if (clientAlarmRem != null) {
            try {

                if (clientAlarmRem.isActive()) {
                    int min = calendar.getEvent().getReminder().getMinutes();
                    setItemValue(docToReturn, "Alarms", "1");
                    setItemValue(docToReturn, "$Alarm", "1");
                    setItemValue(docToReturn, "$AlarmUnit",
                            Utility.calcAlarmOffsetUnit(min));
                    setItemValue(docToReturn, "$AlarmOffset",
                            new Integer(-min));
                } else {
                    setItemValue(docToReturn, "Alarms", "");
                    docToReturn.removeItem("$Alarm");
                    docToReturn.removeItem("$AlarmUnit");
                    docToReturn.removeItem("$AlarmOffset");
                }
            } catch (NotesException ex) {
                log.finest(ex.getMessage());
            }

        }
        try {

            RecurrentDocument recDoc = new RecurrentDocument(docToReturn, timezone);
            if (calendar.getEvent().isRecurrent()) {
                RecurrencePattern rp = calendar.getEvent().getRecurrencePattern();
                String start = calendar.getEvent().getDtStart().getPropertyValueAsString();
                String end = calendar.getEvent().getDtEnd().getPropertyValueAsString();
                recDoc.setRecurrencePattern(rp, start, end);
            } else {
                recDoc.clearRecurrence();
            }
        } catch (Exception ex) {

            log.finest(ex.getMessage());
        }


        if (log.isLoggable(Level.FINEST)) {
            log.finest("updateDominoCalendar end ");
        }

        return docToReturn;
    }

    /**
     * @param calendar       ItemCalendar
     * @param dominoFormName String
     * @param itemFormat     String
     * @return Document
     * @throws EntityException
     */
    private Document createDominoAllDayEvent(
            ItemCalendar calendar,
            String dominoFormName,
            String itemFormat)
            throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("createDominoAnniversary Start ");
        }

        Document doc;
        Database db;

        String subject = calendar.getEvent().getSummary().getPropertyValueAsString();
        String body = calendar.getEvent().getDescription().getPropertyValueAsString();
        String location = calendar.getEvent().getLocation().getPropertyValueAsString();

        // for all day the date format is yyyy-mm-dd
        String start = calendar.getEvent().getDtStart().getPropertyValueAsString();
        DateTime startDateTime = Utility.setAllDayDomino_StartDateTime(this.dominoWrapper, start);
        DateTime startDate = Utility.setAllDayDomino_Date(this.dominoWrapper, start);
        DateTime startTime = Utility.setAllDayDomino_StartTime(this.dominoWrapper);
        DateTime endDateTime = Utility.setAllDayDomino_EndDateTime(this.dominoWrapper, start);
        DateTime endDate = Utility.setAllDayDomino_Date(this.dominoWrapper, start);
        DateTime endTime = Utility.setAllDayDomino_EndTime(this.dominoWrapper);

        String categories = calendar.getEvent().getCategories().getPropertyValueAsString();

        // print client value
        if (log.isLoggable(Level.FINEST)) {
            log.finest("\n subject   : " + subject
                    + "\n body      : " + body
                    + "\n location  : " + location
                    + "\n startDT   : " + startDateTime
                    + "\n startDate : " + startDate
                    + "\n startTime : " + startTime
                    + "\n endDT     : " + endDateTime
                    + "\n endDate   : " + endDate
                    + "\n endTime   : " + endTime);
        }

        // create a Domino alldayenvet
        try {
            db = dominoWrapper.getDatabase();
            doc = db.createDocument();

            doc.appendItemValue(Def.FORM, dominoFormName);
            doc.appendItemValue(Def.APPOINTMENT_TYPE, "2");

            doc.appendItemValue(Def.PRINCIPAL, dominoWrapper.getRemoteUserName());
            doc.appendItemValue(Def.FROM, dominoWrapper.getRemoteUserName());
            doc.appendItemValue(Def.CHAIR, dominoWrapper.getRemoteUserName());

            doc.appendItemValue(Def.VIEW_ICON, new Integer(9));

            doc.appendItemValue(Def.PUBLIC_ACCESS, "1");
            doc.appendItemValue(Def.ORG_TABLE, "P0");
            doc.appendItemValue(Def.EXCLUDE_FROM_VIEW, "D");
            doc.appendItemValue(Def.SEQUENCE_NUM, new Integer(1));

            doc.appendItemValue(Def.SUBJECT, subject);
            doc.appendItemValue(Def.LOCATION, location);
            RichTextItem rti = doc.createRichTextItem(Def.BODY);
            rti.appendText(body);

            // date fields
            doc.appendItemValue(Def.CALENDAR_DATE_TIME, startDateTime);
            doc.appendItemValue(Def.NO_PURGE, endDateTime);

            doc.appendItemValue(Def.START_DATE, startDate);
            doc.appendItemValue(Def.START_TIME, startTime);
            doc.appendItemValue(Def.START_DATE_TIME, startDateTime);

            doc.appendItemValue(Def.END_DATE, endDate);
            doc.appendItemValue(Def.END_TIME, endTime);
            doc.appendItemValue(Def.END_DATE_TIME, endDateTime);

            doc.appendItemValue("AppendStartTime", startTime);
            doc.appendItemValue("AppendEndTime", endTime);

            // reuired in iNotes6
            doc.appendItemValue("ApptUNID", doc.getUniversalID());

            //categories
            if (categories != null)
                doc.appendItemValue(Def.CATEGORIES, categories);

        } catch (Exception e) {
            throw new EntityException(e.toString());
        }

        return doc;
    }

    /**
     * creates a  anniversary form Domino
     *
     * @param calendar       ItemCalendar
     * @param dominoFormName String
     * @param itemFormat     String
     * @return Document
     * @throws EntityException
     */
    private Document createDominoAnniversary(
            ItemCalendar calendar,
            String dominoFormName,
            String itemFormat)
            throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("createDominoAnniversary Start ");
        }

        Document doc;
        Database db;

        String subject = calendar.getEvent().getSummary().getPropertyValueAsString();
        String body = calendar.getEvent().getDescription().getPropertyValueAsString();
        String location = calendar.getEvent().getLocation().getPropertyValueAsString();

        String start = calendar.getEvent().getDtStart().getPropertyValueAsString();
        DateTime startDate = Utility.setDominoDateTime(dominoWrapper, start);

        // no end date

        String categories = calendar.getEvent().getCategories().getPropertyValueAsString();

        // print client value
        if (log.isLoggable(Level.FINEST)) {
            log.finest("subject   : " + subject
                    + "\n body      : " + body
                    + "\n location  : " + location
                    + "\n startS    : " + startDate);
        }

        // create a Domino contact
        try {
            db = dominoWrapper.getDatabase();
            doc = db.createDocument();


            doc.appendItemValue(Def.FORM, dominoFormName);
            doc.appendItemValue(Def.APPOINTMENT_TYPE, "1");
            doc.appendItemValue(Def.PRINCIPAL, dominoWrapper.getRemoteUserName());
            doc.appendItemValue(Def.FROM, dominoWrapper.getRemoteUserName());
            doc.appendItemValue(Def.CHAIR, dominoWrapper.getRemoteUserName());

            doc.appendItemValue(Def.VIEW_ICON, new Integer(177));

            doc.appendItemValue(Def.PUBLIC_ACCESS, "1");
            doc.appendItemValue(Def.ORG_TABLE, "A0");
            doc.appendItemValue(Def.EXCLUDE_FROM_VIEW, "D");
            doc.appendItemValue(Def.SEQUENCE_NUM, new Integer(1));

            doc.appendItemValue(Def.SUBJECT, subject);
            doc.appendItemValue(Def.LOCATION, location);
            RichTextItem rti = doc.createRichTextItem(Def.BODY);
            rti.appendText(body);

            doc.appendItemValue(Def.CALENDAR_DATE_TIME, startDate);

            doc.appendItemValue(Def.START_DATE, startDate);
            doc.appendItemValue(Def.START_TIME, startDate);
            doc.appendItemValue(Def.START_DATE_TIME, startDate);

            // questi li devo settare anche se non servono altrimenti
            // lo stdR6Mail non ha il bottone Edit
            doc.appendItemValue(Def.END_DATE, startDate);
            doc.appendItemValue(Def.END_TIME, startDate);
            doc.appendItemValue(Def.END_DATE_TIME, startDate);

            // necessario in iNotes6
            doc.appendItemValue("ApptUNID", doc.getUniversalID());

            //doc.appendItemValue("$NoPurge",startDate) ;
            //doc.appendItemValue("StartTimeZone","Z=-1$DO=1$DL=3 -1 1 10 -1 1$ZN=Western/Central Europe") ;
            //doc.appendItemValue("EndTimeZone","Z=-1$DO=1$DL=3 -1 1 10 -1 1$ZN=Western/Central Europe") ;

            //categories
            if (categories != null)
                doc.appendItemValue(Def.CATEGORIES, categories);

        } catch (Exception e) {
            throw new EntityException(e.toString());
        }

        return doc;

    }


    /**
     * create meeting for Domino
     *
     * @param calendar       ItemCalendar
     * @param dominoFormName String
     * @param itemFormat     String
     * @return Document
     * @throws EntityException
     */
    private Document createDominoMeeting(
            ItemCalendar calendar,
            String dominoFormName,
            String itemFormat)
            throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("createDominoMeeting - start ");
        }

        Document doc;
        Database db;

        // get foundation information
        String subject = calendar.getEvent().getSummary().getPropertyValueAsString();
        String body = calendar.getEvent().getDescription().getPropertyValueAsString();
        String location = calendar.getEvent().getLocation().getPropertyValueAsString();

        String start = calendar.getEvent().getDtStart().getPropertyValueAsString();
        DateTime startDate = Utility.setDominoDateTime(dominoWrapper, start);

        String end = calendar.getEvent().getDtEnd().getPropertyValueAsString();
        DateTime endDate = Utility.setDominoDateTime(dominoWrapper, end);

        String categories = calendar.getEvent().getCategories().getPropertyValueAsString();

        // print client value
        if (log.isLoggable(Level.FINEST)) {
            log.finest("subject   : " + subject
                    + "\n body      : " + body
                    + "\n location  : " + location
                    + "\n start    : " + startDate
                    + "\n end      : " + endDate
                    + "\n FORM      : " + dominoFormName);
        }

        // create a Domino contact
        try {
            db = dominoWrapper.getDatabase();
            doc = db.createDocument();


            doc.appendItemValue(Def.FORM, dominoFormName);
            doc.appendItemValue(Def.APPOINTMENT_TYPE, "3");
            doc.appendItemValue(Def.PRINCIPAL, dominoWrapper.getRemoteUserName());
            doc.appendItemValue(Def.FROM, dominoWrapper.getRemoteUserName());
            doc.appendItemValue(Def.CHAIR, dominoWrapper.getRemoteUserName());

            doc.appendItemValue(Def.VIEW_ICON, new Integer(158));

            doc.appendItemValue(Def.PUBLIC_ACCESS, "1");
            doc.appendItemValue(Def.ORG_TABLE, "C0");
            doc.appendItemValue(Def.EXCLUDE_FROM_VIEW, "D");
            doc.appendItemValue(Def.SEQUENCE_NUM, new Integer(1));

            doc.appendItemValue(Def.SUBJECT, subject);
            doc.appendItemValue(Def.LOCATION, location);
            RichTextItem rti = doc.createRichTextItem(Def.BODY);
            rti.appendText(body);

            doc.appendItemValue(Def.CALENDAR_DATE_TIME, startDate);

            doc.appendItemValue(Def.START_DATE_TIME, startDate);

            // necessario per client MailStd6
            doc.appendItemValue(Def.END_DATE_TIME, endDate);

            // necessario in iNotes6
            doc.appendItemValue(Def.APPT_UNID, doc.getUniversalID());

            //categories
            if (categories != null)
                doc.appendItemValue(Def.CATEGORIES, categories);

            //doc.appendItemValue("$NoPurge",endDate) ;
            //doc.appendItemValue("StartTimeZone","Z=-1$DO=1$DL=3 -1 1 10 -1 1$ZN=Western/Central Europe") ;
            //doc.appendItemValue("EndTimeZone","Z=-1$DO=1$DL=3 -1 1 10 -1 1$ZN=Western/Central Europe") ;

        } catch (Exception e) {
            throw new EntityException(e.toString());
        }

        return doc;
    }


    /**
     * creates an appointment for Domino
     *
     * @param calendar       ItemCalendar
     * @param dominoFormName String
     * @param itemFormat     String
     * @return Document
     * @throws EntityException
     */
    private Document createDominoAppointment(ItemCalendar calendar,
                                             String dominoFormName,
                                             String itemFormat)
            throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("createDominoAppointment Start ");
        }

        Document doc;
        Database db;

        String subject = calendar.getEvent().getSummary().getPropertyValueAsString();
        if (log.isLoggable(Level.FINEST)) {
            log.finest("Appointment check for subject |" + subject + "|");
        }

        String body = calendar.getEvent().getDescription().getPropertyValueAsString();
        String location = calendar.getEvent().getLocation().getPropertyValueAsString();

        String start = calendar.getEvent().getDtStart().getPropertyValueAsString();
        DateTime startDate = Utility.setDominoDateTime(dominoWrapper, start);

        String end = calendar.getEvent().getDtEnd().getPropertyValueAsString();
        DateTime endDate = Utility.setDominoDateTime(dominoWrapper, end);

        String categories = calendar.getEvent().getCategories().getPropertyValueAsString();

        // print client value
        if (log.isLoggable(Level.FINEST)) {
            log.finest("\n subject   : " + subject
                    + "\n body      : " + body
                    + "\n location  : " + location
                    + "\n start    : " + startDate
                    + "\n end      : " + endDate);
        }

        // create a Domino Appointment
        try {
            db = dominoWrapper.getDatabase();
            doc = db.createDocument();

            doc.appendItemValue(Def.FORM, dominoFormName);
            doc.appendItemValue(Def.APPOINTMENT_TYPE, "0");
            doc.appendItemValue(Def.PRINCIPAL, dominoWrapper.getRemoteUserName());
            doc.appendItemValue(Def.FROM, dominoWrapper.getRemoteUserName());
            doc.appendItemValue(Def.CHAIR, dominoWrapper.getRemoteUserName());

            doc.appendItemValue(Def.VIEW_ICON, new Integer(160));

            doc.appendItemValue(Def.PUBLIC_ACCESS, "1");
            doc.appendItemValue(Def.ORG_TABLE, "C0");
            doc.appendItemValue(Def.EXCLUDE_FROM_VIEW, "D");
            doc.appendItemValue(Def.SEQUENCE_NUM, new Integer(1));

            doc.appendItemValue(Def.SUBJECT, subject);
            doc.appendItemValue(Def.LOCATION, location);
            RichTextItem rti = doc.createRichTextItem(Def.BODY);
            rti.appendText(body);

            doc.appendItemValue(Def.CALENDAR_DATE_TIME, startDate);
            doc.appendItemValue(Def.START_DATE_TIME, startDate);
            // necessario per client MailStd6
            doc.appendItemValue(Def.END_DATE_TIME, endDate);

            // necessario in iNotes6
            doc.appendItemValue(Def.APPT_UNID, doc.getUniversalID());

            //categories
            if (categories != null)
                doc.appendItemValue(Def.CATEGORIES, categories);

            //doc.appendItemValue("$NoPurge",endDate) ;
            //doc.appendItemValue("StartTimeZone","Z=-1$DO=1$DL=3 -1 1 10 -1 1$ZN=Western/Central Europe") ;
            //doc.appendItemValue("EndTimeZone","Z=-1$DO=1$DL=3 -1 1 10 -1 1$ZN=Western/Central Europe") ;


        } catch (Exception e) {
            throw new EntityException(e.toString());
        }

        return doc;
    }


    /**
     * creates a reminder for domino
     *
     * @param calendar       ItemCalendar
     * @param dominoFormName String
     * @param itemFormat     String
     * @return Document
     * @throws EntityException
     */
    private Document createDominoReminder(
            ItemCalendar calendar,
            String dominoFormName,
            String itemFormat)
            throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("createDominoReminder - start ");
        }

        Document doc;
        Database db;

        String subject = calendar.getEvent().getSummary().getPropertyValueAsString();
        if (log.isLoggable(Level.FINEST)) {
            log.finest("Reminder: check for subject |" + subject + "|");
        }


        String body = calendar.getEvent().getDescription().getPropertyValueAsString();
        String location = calendar.getEvent().getLocation().getPropertyValueAsString();

        String start = calendar.getEvent().getDtStart().getPropertyValueAsString();
        DateTime startDate = Utility.setDominoDateTime(dominoWrapper, start);

        String categories = calendar.getEvent().getCategories().getPropertyValueAsString();
        // print client value
        if (log.isLoggable(Level.FINEST)) {
            log.finest("subject   : " + subject
                    + "\n body      : " + body
                    + "\n location  : " + location
                    + "\n startS    : " + startDate);
        }

        // create a Domino reminder
        try {
            db = dominoWrapper.getDatabase();
            doc = db.createDocument();


            doc.appendItemValue(Def.FORM, dominoFormName);
            doc.appendItemValue(Def.APPOINTMENT_TYPE, "4");

            doc.appendItemValue(Def.PRINCIPAL, dominoWrapper.getRemoteUserName());
            doc.appendItemValue(Def.FROM, dominoWrapper.getRemoteUserName());
            doc.appendItemValue(Def.CHAIR, dominoWrapper.getRemoteUserName());

            doc.appendItemValue(Def.VIEW_ICON, new Integer(10));

            doc.appendItemValue(Def.PUBLIC_ACCESS, "1");
            doc.appendItemValue(Def.ORG_TABLE, "C0");
            doc.appendItemValue(Def.EXCLUDE_FROM_VIEW, "D");
            doc.appendItemValue(Def.SEQUENCE_NUM, new Integer(1));

            doc.appendItemValue(Def.SUBJECT, subject);
            doc.appendItemValue(Def.LOCATION, location);
            RichTextItem rti = doc.createRichTextItem(Def.BODY);
            rti.appendText(body);

            doc.appendItemValue(Def.CALENDAR_DATE_TIME, startDate);

            doc.appendItemValue(Def.START_DATE, startDate);
            doc.appendItemValue(Def.START_TIME, startDate);
            doc.appendItemValue(Def.START_DATE_TIME, startDate);

            // questi li devo settare anche se non servono altrimenti
            // lo stdR6Mail non ha il bottone Edit
            doc.appendItemValue(Def.END_DATE, startDate);
            doc.appendItemValue(Def.END_TIME, startDate);
            doc.appendItemValue(Def.END_DATE_TIME, startDate);

            // necessario in iNotes6
            doc.appendItemValue("ApptUNID", doc.getUniversalID());

            //categories
            if (categories != null)
                doc.appendItemValue(Def.CATEGORIES, categories);

            //doc.appendItemValue("$NoPurge",startDate) ;
            //doc.appendItemValue("StartTimeZone","Z=-1$DO=1$DL=3 -1 1 10 -1 1$ZN=Western/Central Europe") ;
            //doc.appendItemValue("EndTimeZone","Z=-1$DO=1$DL=3 -1 1 10 -1 1$ZN=Western/Central Europe") ;

        } catch (Exception e) {
            throw new EntityException(e.toString());
        }

        return doc;
    }

    /**
     * @param doc        Document
     * @param calendar   ItemCalendar
     * @param itemFormat String
     * @return Document
     * @throws EntityException
     */
    private Document updateDominoAllDayEvent(Document doc,
                                             ItemCalendar calendar,
                                             String itemFormat)
            throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("updateDominoAllDayEvent - start ");
        }

        // get values from foundation
        String subject = calendar.getEvent().getSummary().getPropertyValueAsString();
        String body = calendar.getEvent().getDescription().getPropertyValueAsString();
        String location = calendar.getEvent().getLocation().getPropertyValueAsString();

        // yyyy-MM-dd format
        String start = calendar.getEvent().getDtStart().getPropertyValueAsString();
        DateTime startDateTime = Utility.setAllDayDomino_StartDateTime(this.dominoWrapper, start);
        DateTime startDate = Utility.setAllDayDomino_Date(this.dominoWrapper, start);
        DateTime startTime = Utility.setAllDayDomino_StartTime(this.dominoWrapper);
        DateTime endDateTime = Utility.setAllDayDomino_EndDateTime(this.dominoWrapper, start);
        DateTime endDate = Utility.setAllDayDomino_Date(this.dominoWrapper, start);
        DateTime endTime = Utility.setAllDayDomino_EndTime(this.dominoWrapper);

        String categories = calendar.getEvent().getCategories().getPropertyValueAsString();
        try {

            setItemValue(doc, Def.APPOINTMENT_TYPE, "2");
            setItemValue(doc, Def.VIEW_ICON, new Integer(9));
            setItemValue(doc, Def.ORG_TABLE, "P0");
            setItemValue(doc, Def.SUBJECT, subject);
            setItemValue(doc, Def.BODY, body);
            setItemValue(doc, Def.LOCATION, location);

            // date fields
            setItemValue(doc, Def.CALENDAR_DATE_TIME, startDateTime);
            setItemValue(doc, Def.NO_PURGE, endDateTime);

            setItemValue(doc, Def.START_DATE, startDate);
            setItemValue(doc, Def.START_TIME, startTime);
            setItemValue(doc, Def.START_DATE_TIME, startDateTime);

            setItemValue(doc, Def.END_DATE, endDate);
            setItemValue(doc, Def.END_TIME, endTime);
            setItemValue(doc, Def.END_DATE_TIME, endDateTime);

            setItemValue(doc, "AppendStartTime", startTime);
            setItemValue(doc, "AppendEndTime", endTime);

            //categories
            if (categories != null)
                setItemValue(doc, Def.CATEGORIES, categories);

        } catch (Exception e) {
            throw new EntityException(e);
        }

        return doc;
    }

    /**
     * @param doc        Document
     * @param calendar   ItemCalendar
     * @param itemFormat String
     * @return Document
     * @throws EntityException
     */
    private Document updateDominoAnniversary(Document doc,
                                             ItemCalendar calendar,
                                             String itemFormat)
            throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("updateDominoAnniversary - start ");
        }

        // get values from foundation
        String subject = calendar.getEvent().getSummary().getPropertyValueAsString();
        String body = calendar.getEvent().getDescription().getPropertyValueAsString();
        String location = calendar.getEvent().getLocation().getPropertyValueAsString();

        String start = calendar.getEvent().getDtStart().getPropertyValueAsString();
        DateTime startDate = Utility.setDominoDateTime(dominoWrapper, start);

        String categories = calendar.getEvent().getCategories().getPropertyValueAsString();
        // no end date


        try {

            setItemValue(doc, Def.APPOINTMENT_TYPE, "1");
            setItemValue(doc, Def.SUBJECT, subject);
            setItemValue(doc, Def.BODY, body);
            setItemValue(doc, Def.LOCATION, location);

            setItemValue(doc, Def.CALENDAR_DATE_TIME, startDate);

            setItemValue(doc, Def.START_DATE, startDate);
            setItemValue(doc, Def.START_TIME, startDate);
            setItemValue(doc, Def.START_DATE_TIME, startDate);

            setItemValue(doc, Def.END_DATE, startDate);
            setItemValue(doc, Def.END_TIME, startDate);
            setItemValue(doc, Def.END_DATE_TIME, startDate);

            //categories
            if (categories != null)
                setItemValue(doc, Def.CATEGORIES, categories);

        } catch (Exception e) {
            throw new EntityException(e);
        }

        return doc;
    }

    /**
     * @param doc        Document
     * @param calendar   ItemCalendar
     * @param itemFormat String
     * @return Document
     * @throws EntityException
     */
    private Document updateDominoMeeting(Document doc,
                                         ItemCalendar calendar,
                                         String itemFormat)
            throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("updateDominoMeeting - start ");
        }

        // get values from foundation
        String subject = calendar.getEvent().getSummary().getPropertyValueAsString();
        String body = calendar.getEvent().getDescription().getPropertyValueAsString();
        String location = calendar.getEvent().getLocation().getPropertyValueAsString();

        String start = calendar.getEvent().getDtStart().getPropertyValueAsString();
        DateTime startDate = Utility.setDominoDateTime(dominoWrapper, start);

        String end = calendar.getEvent().getDtEnd().getPropertyValueAsString();
        DateTime endDate = Utility.setDominoDateTime(dominoWrapper, end);

        String categories = calendar.getEvent().getCategories().getPropertyValueAsString();

        // print client value
        if (log.isLoggable(Level.FINEST)) {
            log.finest("subject   : " + subject
                    + "\n body      : " + body
                    + "\n location  : " + location
                    + "\n startS    : " + startDate
                    + "\n endS      : " + endDate);
        }

        try {

            setItemValue(doc, Def.APPOINTMENT_TYPE, "3");
            setItemValue(doc, Def.SUBJECT, subject);
            setItemValue(doc, Def.BODY, body);
            setItemValue(doc, Def.LOCATION, location);

            setItemValue(doc, Def.CALENDAR_DATE_TIME, startDate);

            setItemValue(doc, Def.START_DATE, startDate);
            setItemValue(doc, Def.START_TIME, startDate);
            setItemValue(doc, Def.START_DATE_TIME, startDate);

            setItemValue(doc, Def.END_DATE, endDate);
            setItemValue(doc, Def.END_TIME, endDate);
            setItemValue(doc, Def.END_DATE_TIME, endDate);

            //categories
            if (categories != null)
                setItemValue(doc, Def.CATEGORIES, categories);

        } catch (Exception e) {
            throw new EntityException(e);
        }

        return doc;
    }


    /**
     * @param doc        Document
     * @param calendar   ItemCalendar
     * @param itemFormat String
     * @return Document
     * @throws EntityException
     */
    private Document updateDominoAppointment(Document doc,
                                             ItemCalendar calendar,
                                             String itemFormat)
            throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("updateDominoAppointment - start ");
        }

        // get values from foundation
        String subject = calendar.getEvent().getSummary().getPropertyValueAsString();
        if (log.isLoggable(Level.FINEST)) {
            log.finest("Appointment check for subject |" + subject + "|");
        }

        String body = calendar.getEvent().getDescription().getPropertyValueAsString();
        String location = calendar.getEvent().getLocation().getPropertyValueAsString();

        String start = calendar.getEvent().getDtStart().getPropertyValueAsString();
        DateTime startDate = Utility.setDominoDateTime(dominoWrapper, start);

        String end = calendar.getEvent().getDtEnd().getPropertyValueAsString();
        DateTime endDate = Utility.setDominoDateTime(dominoWrapper, end);

        String categories = calendar.getEvent().getCategories().getPropertyValueAsString();

        // print client value
        if (log.isLoggable(Level.FINEST)) {
            log.finest("subject   : " + subject
                    + "\n body      : " + body
                    + "\n location  : " + location
                    + "\n startS    : " + startDate
                    + "\n endS      : " + endDate);
        }

        try {
            setItemValue(doc, Def.APPOINTMENT_TYPE, "0");
            setItemValue(doc, Def.SUBJECT, subject);
            setItemValue(doc, Def.BODY, body);
            doc.removeItem(Def.BODY);
            RichTextItem rti = doc.createRichTextItem(Def.BODY);
            rti.appendText(body);
            setItemValue(doc, Def.LOCATION, location);

            setItemValue(doc, Def.CALENDAR_DATE_TIME, startDate);

            setItemValue(doc, Def.START_DATE, startDate);
            setItemValue(doc, Def.START_TIME, startDate);
            setItemValue(doc, Def.START_DATE_TIME, startDate);

            setItemValue(doc, Def.END_DATE, endDate);
            setItemValue(doc, Def.END_TIME, endDate);
            setItemValue(doc, Def.END_DATE_TIME, endDate);

            //categories
            if (categories != null)
                setItemValue(doc, Def.CATEGORIES, categories);

        } catch (Exception e) {
            throw new EntityException(e);
        }

        return doc;
    }

    /**
     * @param doc        Document
     * @param calendar   ItemCalendar
     * @param itemFormat String
     * @return Document
     * @throws EntityException
     */
    private Document updateDominoReminder(Document doc,
                                          ItemCalendar calendar,
                                          String itemFormat)
            throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("updateDominoReminder - start ");
        }

        // get values from foundation
        String subject = calendar.getEvent().getSummary().getPropertyValueAsString();
        if (log.isLoggable(Level.FINEST)) {
            log.finest("Appointment check for subject |" + subject + "|");
        }


        String body = calendar.getEvent().getDescription().getPropertyValueAsString();
        String location = calendar.getEvent().getLocation().getPropertyValueAsString();

        String start = calendar.getEvent().getDtStart().getPropertyValueAsString();
        DateTime startDate = Utility.setDominoDateTime(dominoWrapper, start);

        // non end date

        String categories = calendar.getEvent().getCategories().getPropertyValueAsString();

        // print client value
        if (log.isLoggable(Level.FINEST)) {
            log.finest("subject   : " + subject
                    + "\n body      : " + body
                    + "\n location  : " + location
                    + "\n startS    : " + startDate);
        }

        try {

            setItemValue(doc, Def.APPOINTMENT_TYPE, "2");
            setItemValue(doc, Def.SUBJECT, subject);
            setItemValue(doc, Def.BODY, body);
            setItemValue(doc, Def.LOCATION, location);

            setItemValue(doc, Def.CALENDAR_DATE_TIME, startDate);

            setItemValue(doc, Def.START_DATE, startDate);
            setItemValue(doc, Def.START_TIME, startDate);
            setItemValue(doc, Def.START_DATE_TIME, startDate);

            setItemValue(doc, Def.END_DATE, startDate);
            setItemValue(doc, Def.END_TIME, startDate);
            setItemValue(doc, Def.END_DATE_TIME, startDate);

            //categories
            if (categories != null)
                setItemValue(doc, Def.CATEGORIES, categories);

        } catch (Exception e) {
            throw new EntityException(e);
        }

        return doc;
    }


    /**
     * converts Domino calendar in Foundation Calendar
     *
     * @param dominoDoc  Document
     * @param itemFormat String
     * @return ItemCalendar
     */
    private ItemCalendar createFoundationEvent(Document dominoDoc,
                                               String itemFormat)
            throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("EVENT: createSyncItem start");
        }

        // get values from Domino Document
        String dominoCalendarType;
        String clientCalendarType;
        String subject;
        String body;
        String location;
        String start;
        DateTime startDomino;
        String end;
        DateTime endDomino;
        String dominoAlarm;
        int dominoOffset;
        String dominoConfidential;
        String clientConfidential;
        String categories;

        // get info from Domino Document
        try {

            dominoCalendarType = dominoDoc.getItemValueString(Def.APPOINTMENT_TYPE);

            clientCalendarType = UtilityCalendar.setClientCalendarType(
                    dominoCalendarType,
                    itemFormat);

            subject = dominoDoc.getItemValueString(Def.SUBJECT);
            body = dominoDoc.getItemValueString(Def.BODY);
            location = dominoDoc.getItemValueString(Def.LOCATION);


            if (dominoCalendarType.equalsIgnoreCase("2")) {
                // if all day event

                startDomino = Utility.getDominoDateTime(dominoDoc, Def.START_DATE_TIME);
                start = Utility.setAllDayClient_StartDate(startDomino);
                end = start;

            } else {
                //otherwise
                startDomino = Utility.getDominoDateTime(dominoDoc, Def.START_DATE_TIME);
                start = Utility.setClientDateTime(startDomino);

                endDomino = Utility.getDominoDateTime(dominoDoc, Def.END_DATE_TIME);
                end = Utility.setClientDateTime(endDomino);
            }

            // alarm
            dominoAlarm = dominoDoc.getItemValueString(Def.ALARMS);

            dominoOffset = dominoDoc.getItemValueInteger(Def.ALARM_OFFSET);

            // confidential
            dominoConfidential = dominoDoc.getItemValueString(Def.ORG_CONFIDENTIAL);
            clientConfidential = Utility.setClientConfidential(dominoConfidential,
                    itemFormat);

            categories = getDocumentCategories(dominoDoc);


        } catch (Exception e) {
            throw new EntityException(e);
        }

        // create a new foundation calendar
        ItemCalendar tmpCalendar = new ItemCalendar();

        // set UID
        String uid;
        try {
            uid = dominoDoc.getUniversalID();
        } catch (Exception e) {
            throw new EntityException(e);
        }
        tmpCalendar.setId(uid);


        tmpCalendar.getEvent().getSummary().setPropertyValue(subject);
        tmpCalendar.getEvent().getDescription().setPropertyValue(body);
        tmpCalendar.getEvent().getLocation().setPropertyValue(location);

        tmpCalendar.getEvent().getDtStart().setPropertyValue(start);
        tmpCalendar.getEvent().getDtEnd().setPropertyValue(end);

        if (dominoCalendarType.equalsIgnoreCase("2"))
            tmpCalendar.getEvent().setAllDay(Boolean.TRUE);
        else
            tmpCalendar.getEvent().setAllDay(Boolean.FALSE);

        //reminder
        if (dominoOffset <= 0 && dominoAlarm != null) {

            int min = -1 * dominoOffset;
            if (dominoAlarm.equals("1")) {
                tmpCalendar.getEvent().setReminder(new Reminder());
                tmpCalendar.getEvent().getReminder().setMinutes(min);
                tmpCalendar.getEvent().getReminder().setActive(true);
            } else {
                tmpCalendar.getEvent().setReminder(new Reminder());
                tmpCalendar.getEvent().getReminder().setMinutes(min);
                tmpCalendar.getEvent().getReminder().setActive(false);
            }
        }

        // confidential
        if (clientConfidential != null) {
            tmpCalendar.getEvent().getClassEvent().setPropertyValue(clientConfidential);
        }

        // categories

        if (categories != null) {
            tmpCalendar.getEvent().getCategories().setPropertyValue(categories);
        }

        // calendar type
        if (itemFormat.equals(Def.ICAL_TYPE)) {
            tmpCalendar.addXTag(UtilityCalendar.setXTagForCalendar(clientCalendarType));
        } else {
            if (clientCalendarType.equals("ALLDAY")) {
                tmpCalendar.getEvent().setAllDay(Boolean.TRUE);
            } else {
                /**
                 * todo calendar type
                 */
            }
        }

        //recurrence
        try {

            RecurrentDocument rec = new RecurrentDocument(dominoDoc, timezone);
            if (rec.isRecurrent()) {
                tmpCalendar.getEvent().setRecurrencePattern(
                        rec.getRecurrencePattern());
            }
        } catch (Exception ex) {
            log.finest(ex.getMessage());
        }

        if (log.isLoggable(Level.FINEST)) {
            log.finest("end setting ");
        }

        return tmpCalendar;

    }

}