/**
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.domino.items.manager ;

import com.funambol.domino.engine.cache.SyncItemInfo;
import com.funambol.domino.exception.DominoAccessException;
import com.funambol.domino.exception.EntityException;
import com.funambol.domino.items.dao.ContactDAO;
import com.funambol.domino.items.model.ItemContact;
import com.funambol.domino.util.Def;
import com.funambol.domino.util.DominoWrapper;
import com.funambol.domino.util.Utility;
import com.funambol.domino.util.UtilityContact;
import com.funambol.foundation.pdi.common.Property;
import com.funambol.foundation.pdi.contact.*;
import com.funambol.foundation.pdi.contact.Name;
import lotus.domino.*;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Vector;
import java.util.logging.Level;


/**
 * @author Gilberto Migliavacca
 * @version $Id: ContactManager.java,v 1.4 2006/09/06 11:23:25 diogo Exp $
 */
public class ContactManager extends EntityManager {
    
    
    //-------------------------------------------------------------- Costructors
    
    /**
     *
     */
    public ContactManager() {
    }
    
    /**
     * @param dominoWrapper_e DominoWrapper
     * @throws DominoAccessException
     */
    public ContactManager(DominoWrapper dominoWrapper_e)
        throws DominoAccessException {
        dominoWrapper = dominoWrapper_e;
        dao = new ContactDAO(dominoWrapper_e);
    }
    
    //----------------------------------------------------------- Public methods
    
    
    /**
     * updates a document defined by UID
     * In the parameter content there are all the
     * value from the client
     *
     * @param oldItem ItemContact
     * @param newItem ItemContact
     * @param itemFormat  String
     * @throws EntityException
     */
    public void updateItem(ItemContact oldItem,
            ItemContact newItem,
            String itemFormat)
            throws EntityException {
        if (log.isLoggable(Level.FINEST)) {
            log.finest("Start - Contact updateItem ");
        }
        String uid = oldItem.getId();
        Document docToUptdate = dao.getItemByUID(uid, this.dominoWrapper);
        Document doc = updateDominoContact(docToUptdate, newItem, itemFormat);
        try {
            dao.updateItem(doc);
        } catch (EntityException e) {
            throw new EntityException(e);
        }
        
        
    }
    
    
    /**
     * @param contact    Contact
     * @param itemFormat String
     * @return Document
     * @throws EntityException
     */
    public ItemContact insertItem(ItemContact contact, String itemFormat)
    throws EntityException {
        
        if (log.isLoggable(Level.FINEST)) {
            log.finest("Start - ItemContact insertItem ");
        }
        Document doc = createDominoContact(contact,
                itemFormat,
                Def.CONTACT_FORM_NAME);
        Document docNew;
        try {
            docNew = dao.insertItem(doc);
        } catch (EntityException e) {
            throw new EntityException(e);
        }
        
        // convert Document in contact
        ItemContact ctmpout;
        try {
            ctmpout = createFoundationContact(docNew, itemFormat);
        } catch (Exception e) {
            throw new EntityException(e);
        }
        
        return ctmpout;
        
    }
    
    
    /**
     * @return SyncItemInfo[]
     * @throws EntityException
     */
    public SyncItemInfo[] getAllSyncItemInfo(String viewname)
    throws EntityException {
        
        if (log.isLoggable(Level.FINEST)) {
            log.finest("getAllSyncItemInfo Start");
        }
        
        SyncItemInfo[] syncItemInfos;
        try {
            View view = this.dominoWrapper.getView(viewname);
            syncItemInfos = dao.getAllSyncItemInfo(view);
        } catch (Exception e) {
            throw new EntityException(e);
        }
        
        if (log.isLoggable(Level.FINEST)) {
            log.finest("Domino List with " + syncItemInfos.length + " Document");
        }
        
        return syncItemInfos;
        
    }
    
    /**
     * @param UID String
     * @return Document
     * @throws EntityException
     */
    public ItemContact getItemByUID(String UID, String itemFormat)
    throws EntityException {
        
        if (log.isLoggable(Level.FINEST)) {
            log.finest("getItemByUID start " + UID);
        }
        
        Document doc;
        try {
            doc = dao.getItemByUID(UID, this.dominoWrapper);
        } catch (Exception e) {
            throw new EntityException(e);
        }
        
        // convert Document in contact
        ItemContact ctmpout = null;
        if (doc != null) {
            try {
                ctmpout = createFoundationContact(doc, itemFormat);
            } catch (Exception e) {
                throw new EntityException(e);
            }
        }
        
        return ctmpout;
    }
    
    
    /**
     * checks if a contact is already inserted in the db
     * and returns the Document in the DB
     * we use firstname, lastname as search criteria
     *
     * @param contact    ItemContact
     * @param itemFormat String
     * @return found ItemContact already present
     */
    public ItemContact[] getItemsByWhereClause(ItemContact contact,
            String itemFormat)
            throws EntityException {
        
        if (log.isLoggable(Level.FINEST)) {
            log.finest("getItemByWhereClause start ");
        }
        
        List foundDocs = new LinkedList();
        List foundItemContacts = new LinkedList();
        Iterator iter;
        
        // first name
        String fn = (String) contact.getName().getFirstName().getPropertyValue();
        // last name
        String ln = (String) contact.getName().getLastName().getPropertyValue();
        // MAIL OTHER
        String em;
        if (itemFormat.equals(Def.VCARD_TYPE)) {
            em = UtilityContact.getPersonalEmailFromFoundation(contact, 1);
        } else {
            em = UtilityContact.getPersonalEmailFromFoundation(contact, 2);
        }
        
        if (log.isLoggable(Level.FINEST)) {
            log.finest("baseline parameters: " +
                    "\nfirst name: " + fn +
                    "\nlast name:  " + ln +
                    "\nemail:      " + em);
        }
        
        try {
            String formula = "Form =\"" + Def.CONTACT_FORM_NAME + "\" & " +
                    Def.FIRST_NAME + " = \"" + fn + "\" & " +
                    Def.LAST_NAME + " = \"" + ln + "\"";
            if (log.isLoggable(Level.FINEST)) {
                log.finest("Domino Formula " + formula);
            }
            DocumentCollection dc = dominoWrapper.getDatabase().search(formula);
            
            if (log.isLoggable(Level.FINEST)) {
                log.finest("Found contacts number " + dc.getCount());
            }
            if (dc.getCount() > 0) {
                if (log.isLoggable(Level.FINEST)) {
                    log.finest("ItemContact already Exist");
                }
                for (int i = 1; i <= dc.getCount(); i++) {
                    foundDocs.add(dc.getNthDocument(i));
                }
                
            }
        } catch (Exception e) {
            throw new EntityException(e);
        }
        
        // convert Document in a ItemContact
        ItemContact ctmpout;
        if (foundDocs.size() > 0) {
            try {
                iter = foundDocs.iterator();
                while (iter.hasNext()) {
                    ctmpout = createFoundationContact((Document) iter.next(), itemFormat);
                    foundItemContacts.add(ctmpout);
                }
            } catch (Exception e) {
                throw new EntityException(e);
            }
        }
        
        return (ItemContact[]) foundItemContacts.toArray(new ItemContact[0]);
    }
    
    //---------------------------------------------------------- Private methods
    
    
    /**
     * create a Domino ItemContact from a sync4j
     * foundation ItemContact
     *
     * @param ctmp           ItemContact
     * @param itemformat     String
     * @param dominoFormName String
     * @return Document
     * @throws EntityException
     */
    private Document createDominoContact(
            ItemContact ctmp,
            String itemformat,
            String dominoFormName) throws EntityException {
        
        if (log.isLoggable(Level.FINEST)) {
            log.finest("create ItemContact\n: ... " +
                    "\n itemformat " + itemformat);
        }
        
        Document doc;
        Database db;
        String tmp = null;
        
        // retrive sync4j foundation ItemContact detail
        Name name = ctmp.getName();
        BusinessDetail bd = ctmp.getBusinessDetail();
        Address badd = bd.getAddress();
        PersonalDetail pd = ctmp.getPersonalDetail();
        Address padd = pd.getAddress();
        Property categories = ctmp.getCategories();
        
        
        // create a Domino contact
        try {
            db = dominoWrapper.getDatabase();
            
            doc = db.createDocument();
            
            //system fields
            doc.appendItemValue(Def.FORM, dominoFormName);
            doc.appendItemValue(Def.PATH, dominoWrapper.getDbName());
            doc.appendItemValue(Def.PRINCIPAL, dominoWrapper.getRemoteUserName());
            doc.appendItemValue(Def.FROM, dominoWrapper.getRemoteUserName());
            
            doc.appendItemValue(Def.EXCLUDE_FROM_VIEW, "A");
            doc.appendItemValue(Def.CATEGORY, "");
            doc.appendItemValue(Def.PROTECTFROMARCHIVE, "1");
            
            // contact fileds
            // name
            if(name != null) {
                
                if(name.getSalutation() != null) {
                    tmp = name.getSalutation().getPropertyValueAsString();
                    if ("0".equals(tmp)) {
                        tmp = "[0]";
                    }
                    doc.appendItemValue(Def.TITLE, tmp);
                }
                
                if(name.getFirstName() != null)
                    doc.appendItemValue(Def.FIRST_NAME, name.getFirstName().getPropertyValueAsString());
                
                if(name.getMiddleName() != null)
                    doc.appendItemValue(Def.MIDDLE_INITIAL, name.getMiddleName().getPropertyValueAsString());
                
                if(name.getLastName() != null)
                    doc.appendItemValue(Def.LAST_NAME, name.getLastName().getPropertyValueAsString());
                
                if(name.getSuffix() != null) {
                    tmp = name.getSuffix().getPropertyValueAsString();
                    if ("0".equals(tmp)) {
                        tmp = "[0]";
                    }
                    doc.appendItemValue(Def.SUFFIX, tmp);
                }
                
                if(name.getNickname() != null)
                    doc.appendItemValue(Def.SHORT_NAME, name.getNickname().getPropertyValueAsString());
                
                
                try {
                    tmp = name.getDisplayName().getPropertyValueAsString();
                    
                }catch(NullPointerException ex) {
                    tmp = "";
                }
                if("".equals(tmp)) {
                    try {
                        tmp = bd.getCompany().getPropertyValueAsString();
                    } catch(NullPointerException ex) {
                        tmp = "";
                    }
                    
                    if("".equals(tmp)) {
                        
                        try {
                            tmp = name.getSuffix().getPropertyValueAsString();
                        } catch(NullPointerException ex) {
                            tmp = "";
                        }
                        
                    }
                    
                }
                doc.appendItemValue(Def.FULL_NAME, tmp);
                
            }
            
            if(pd != null) {
                
                if(pd.getBirthday() != null)
                    doc.appendItemValue(Def.BIRTHDAY, pd.getBirthday());
                
                if(pd.getAnniversary() != null)
                    doc.appendItemValue(Def.ANNIVERSARY, pd.getAnniversary());
                
                if(pd.getSpouse() != null)
                    doc.appendItemValue(Def.SPOUSE, pd.getSpouse());
                
                if(pd.getChildren() != null)
                    doc.appendItemValue(Def.CHILDREN,  pd.getChildren());
                
            }
            
            if(bd != null) {
                
                if(bd.getManager() != null)
                    doc.appendItemValue(Def.MANAGER, bd.getManager());
                if(bd.getAssistant() != null)
                    doc.appendItemValue(Def.ASSISTANT,  bd.getAssistant());
                
                if(bd.getDepartment() != null)
                    doc.appendItemValue(Def.DEPARTMENT, bd.getDepartment().getPropertyValueAsString());
                
                if(bd.getCompany() != null) {
                    tmp =   "" +
                            bd.getCompany().getPropertyValueAsString() +
                            name.getFirstName().getPropertyValueAsString() +
                            name.getLastName().getPropertyValueAsString();
                    if ("".equals(tmp)) {
                        tmp = "[UNKNOWN]";
                    } else {
                        tmp = bd.getCompany().getPropertyValueAsString();
                    }
                    doc.appendItemValue(Def.COMPANY_NAME, tmp);
                    
                }
                
                
                tmp = UtilityContact.getBusinessTitleFromFoundation(ctmp, 1);
                if(tmp != null)
                    doc.appendItemValue(Def.JOB_TITLE, tmp);
            }
            
            
            if(badd != null) {
                
                if(badd.getStreet() != null)
                    doc.appendItemValue(Def.OFFICE_STREET_ADDRESS, badd.getStreet().getPropertyValueAsString());
                
                if(badd.getCity() != null)
                    doc.appendItemValue(Def.OFFICE_CITY, badd.getCity().getPropertyValueAsString());
                
                if(badd.getState() != null)
                    doc.appendItemValue(Def.OFFICE_STATE, badd.getState().getPropertyValueAsString());
                
                if(badd.getPostalCode() != null)
                    doc.appendItemValue(Def.OFFICE_ZIP, badd.getPostalCode().getPropertyValueAsString());
                
                if(badd.getCountry() != null)
                    doc.appendItemValue(Def.OFFICE_COUNTRY, badd.getCountry().getPropertyValueAsString());
                
            }
            
            if(padd != null) {
                
                if(padd.getStreet() != null)
                    doc.appendItemValue(Def.STREET_ADDRESS, padd.getStreet().getPropertyValueAsString());
                
                if(padd.getCity() != null)
                    doc.appendItemValue(Def.CITY, padd.getCity().getPropertyValueAsString());
                
                if(padd.getState() != null)
                    doc.appendItemValue(Def.STATE, padd.getState().getPropertyValueAsString());
                
                if(padd.getPostalCode() != null)
                    doc.appendItemValue(Def.ZIP, padd.getPostalCode().getPropertyValueAsString());
                
                if(padd.getCountry() != null)
                    doc.appendItemValue(Def.COUNTRY, padd.getCountry().getPropertyValueAsString());
                
            }
            
            if(categories != null)
                doc.appendItemValue(Def.CATEGORIES, categories.getPropertyValueAsString());
            
            String[] webPages;
            
            webPages = UtilityContact.getWebPagesFromFoundation(ctmp, new String[]{Def.BUSINESS_WEBPAGE});
            if (webPages.length>0) {
                setItemValue(doc, Def.WEB_SITE, webPages[0]);
            } else {
                setItemValue(doc, Def.WEB_SITE, "");
            }
            
            webPages = UtilityContact.getWebPagesFromFoundation(ctmp, new String[]{Def.BUSINESS_2_WEBPAGE});
            if (webPages.length>0) {
                setItemValue(doc, Def.WORK_2_URL, webPages[0]);
            } else {
                setItemValue(doc, Def.WORK_2_URL, "");
            }
            
            webPages = UtilityContact.getWebPagesFromFoundation(ctmp, new String[]{Def.BUSINESS_3_WEBPAGE});
            if (webPages.length>0) {
                setItemValue(doc, Def.WORK_3_URL, webPages[0]);
            } else {
                setItemValue(doc, Def.WORK_3_URL, "");
            }
            
            setItemValue(doc, Def.HOME_URL, UtilityContact.getPersonalWebPageFromFoundation(ctmp));
            setItemValue(doc, Def.OTHER_URL, UtilityContact.getOtherWebPageFromFoundation(ctmp));
            
            List notes = ctmp.getNotes();
            if (notes != null) {
                tmp = "";
                for (Iterator it = notes.iterator(); it.hasNext();) {
                    Note note = (Note) it.next();
                    tmp += note.getPropertyValueAsString();
                }
                
            }
            doc.appendItemValue(Def.COMMENT, tmp);
            
            doc.appendItemValue(Def.ASSISTANT_PHONE,
                    UtilityContact.getBusinessPhoneFromFoundation(ctmp,
                    Def.ASSISTANT_PHONE_NUM));
            
            doc.appendItemValue(Def.PAGER_PHONE_NUMBER,
                    UtilityContact.getBusinessPhoneFromFoundation(ctmp,
                    Def.PAGER_NUMBER));
            
            doc.appendItemValue(Def.WORK2_PHONE,
                    UtilityContact.getBusinessPhoneFromFoundation(ctmp,
                    Def.BUSINESS_2_PHONE_NUM));
            
            doc.appendItemValue(Def.HOME2_PHONE,
                    UtilityContact.getPersonalPhoneFromFoundation(ctmp,
                    Def.PERSONAL_HOME_2_PHONE_NUM));
            
            // different in VCARD and XML
            if (itemformat.equals(Def.VCARD_TYPE)) {
                doc.appendItemValue(Def.HOME_PHONE_NUMBER,
                        UtilityContact.getPersonalPhoneFromFoundation(ctmp, 1));
            } else {
                doc.appendItemValue(Def.HOME_PHONE_NUMBER,
                        UtilityContact.getPersonalPhoneFromFoundation(ctmp, 3));
            }
            
            doc.appendItemValue(Def.HOME_FAX_PHONE_NUMBER,
                    UtilityContact.getPersonalPhoneFromFoundation(ctmp, 5));
            
            doc.appendItemValue(Def.OFFICE_PHONE_NUMBER,
                    UtilityContact.getBusinessPhoneFromFoundation(ctmp, 1));
            doc.appendItemValue(Def.OFFICE_FAX_PHONE_NUMBER,
                    UtilityContact.getBusinessPhoneFromFoundation(ctmp, 3));
            
            // same in VCARD and XML
            doc.appendItemValue(Def.PERSONAL_CELL_PHONE,
                    UtilityContact.getPersonalPhoneFromFoundation(ctmp, 2));
            
            doc.appendItemValue(Def.BUSINESS_CELL_PHONE,
                    UtilityContact.getBusinessPhoneFromFoundation(ctmp, 2));
            
            // MAIL OTHER
            // different in VCARD and XML
            if (itemformat.equals(Def.VCARD_TYPE)) {
                doc.appendItemValue(Def.MAIL_OTHER,
                        UtilityContact.getPersonalEmailFromFoundation(ctmp, 1));
            } else {
                doc.appendItemValue(Def.MAIL_OTHER,
                        UtilityContact.getPersonalEmailFromFoundation(ctmp, 2));
            }
            
            // MAIL HOME
            // different in VCARD and XML
            if (itemformat.equals(Def.VCARD_TYPE)) {
                doc.appendItemValue(Def.MAIL_HOME,
                        UtilityContact.getPersonalEmailFromFoundation(ctmp, 2));
            } else {
                doc.appendItemValue(Def.MAIL_HOME,
                        UtilityContact.getPersonalEmailFromFoundation(ctmp, 1));
            }
            
            // MAIL WORK
            String[] businessEmails;
            businessEmails = UtilityContact.getEmailsOfTypeFromFoundation(ctmp, Def.EMAIL_3_ADDRESS);
            if (businessEmails.length > 0) {
                doc.appendItemValue(Def.MAIL_WORK, businessEmails[0]);
            }
            businessEmails = UtilityContact.getEmailsOfTypeFromFoundation(ctmp, Def.BUSINESS_EMAIL_2_ADDRESS);
            if (businessEmails.length > 0) {
                doc.appendItemValue(Def.MAIL_WORK2, businessEmails[0]);
            }
            businessEmails = UtilityContact.getEmailsOfTypeFromFoundation(ctmp, Def.BUSINESS_EMAIL_3_ADDRESS);
            if (businessEmails.length > 0) {
                doc.appendItemValue(Def.MAIL_WORK3, businessEmails[0]);
            }
        } catch (Exception e) {
            throw new EntityException(e.toString());
        }
        
        return doc;
    }
    
    
    /**
     * @param doc        Document
     * @param contact    ItemContact
     * @param itemformat String
     * @return Document
     * @throws EntityException
     */
    private Document updateDominoContact(Document doc,
            ItemContact contact,
            String itemformat)
            throws EntityException {
        
        if (log.isLoggable(Level.FINEST)) {
            log.finest("update ItemContact\n: ... ");
        }
        
        Name name = contact.getName();
        PersonalDetail pd = contact.getPersonalDetail();
        BusinessDetail bd = contact.getBusinessDetail();
        Address badd = bd.getAddress();
        Address padd = pd.getAddress();
        Property categories = contact.getCategories();
        
        try {
            
            // name
            if(name != null) {
                
                if(name.getSalutation() != null) {
                    String title = name.getSalutation().getPropertyValueAsString();
                    if ("0".equals(title)) {
                        title = "[0]";
                    }
                    setItemValue(doc, Def.TITLE, title);
                }
                
                if(name.getFirstName() != null)
                    setItemValue(doc, Def.FIRST_NAME, name.getFirstName().getPropertyValueAsString());
                
                if(name.getMiddleName() != null)
                    setItemValue(doc, Def.MIDDLE_INITIAL, name.getMiddleName().getPropertyValueAsString());
                
                if(name.getLastName() != null)
                    setItemValue(doc, Def.LAST_NAME, name.getLastName().getPropertyValueAsString());
                
                if(name.getSuffix() != null) {
                    String sufix = name.getSuffix().getPropertyValueAsString();
                    if ("0".equals(sufix)) {
                        sufix = "[0]";
                    }
                    setItemValue(doc, Def.SUFFIX, sufix);
                }
                
                if(name.getNickname() != null)
                    setItemValue(doc, Def.SHORT_NAME, name.getNickname().getPropertyValueAsString());
                
                String displayName;
                try {
                    displayName = name.getDisplayName().getPropertyValueAsString();
                    
                }catch(NullPointerException ex) {
                    displayName = "";
                }
                if("".equals(displayName)) {
                    try {
                        displayName = bd.getCompany().getPropertyValueAsString();
                    } catch(NullPointerException ex) {
                        displayName = "";
                    }
                    
                    if("".equals(displayName)) {
                        
                        try {
                            displayName = name.getSuffix().getPropertyValueAsString();
                        } catch(NullPointerException ex) {
                            displayName = "";
                        }
                        
                    }
                    
                }
                setItemValue(doc, Def.FULL_NAME, displayName);
                
            }
            
            if(pd != null) {
                
                if(pd.getBirthday() != null)
                    setItemValue(doc, Def.BIRTHDAY, pd.getBirthday());
                
                if(pd.getAnniversary() != null)
                    setItemValue(doc, Def.ANNIVERSARY, pd.getAnniversary());
                
                if(pd.getSpouse() != null)
                    setItemValue(doc, Def.SPOUSE, pd.getSpouse());
                
                if(pd.getChildren() != null)
                    setItemValue(doc, Def.CHILDREN,  pd.getChildren());
                
            }
            
            if(bd != null) {
                
                if(bd.getManager() != null)
                    setItemValue(doc, Def.MANAGER, bd.getManager());
                if(bd.getAssistant() != null)
                    setItemValue(doc, Def.ASSISTANT,  bd.getAssistant());
                
                if(bd.getDepartment() != null)
                    setItemValue(doc, Def.DEPARTMENT, bd.getDepartment().getPropertyValueAsString());
                
                if(bd.getCompany() != null) {
                    String tmp =   "" +
                            bd.getCompany().getPropertyValueAsString() +
                            name.getFirstName().getPropertyValueAsString() +
                            name.getLastName().getPropertyValueAsString();
                    if ("".equals(tmp)) {
                        tmp = "[UNKNOWN]";
                    } else {
                        tmp = bd.getCompany().getPropertyValueAsString();
                    }
                    setItemValue(doc, Def.COMPANY_NAME, tmp);
                    
                }
                
                String jobTitle = UtilityContact.getBusinessTitleFromFoundation(contact, 1);
                if(jobTitle != null)
                    setItemValue(doc, Def.JOB_TITLE, jobTitle);
            }
            
            
            if(badd != null) {
                
                if(badd.getStreet() != null)
                    setItemValue(doc, Def.OFFICE_STREET_ADDRESS, badd.getStreet().getPropertyValueAsString());
                
                if(badd.getCity() != null)
                    setItemValue(doc, Def.OFFICE_CITY, badd.getCity().getPropertyValueAsString());
                
                if(badd.getState() != null)
                    setItemValue(doc, Def.OFFICE_STATE, badd.getState().getPropertyValueAsString());
                
                if(badd.getPostalCode() != null)
                    setItemValue(doc, Def.OFFICE_ZIP, badd.getPostalCode().getPropertyValueAsString());
                
                if(badd.getCountry() != null)
                    setItemValue(doc, Def.OFFICE_COUNTRY, badd.getCountry().getPropertyValueAsString());
                
            }
            
            if(padd != null) {
                
                if(padd.getStreet() != null)
                    setItemValue(doc, Def.STREET_ADDRESS, padd.getStreet().getPropertyValueAsString());
                
                if(padd.getCity() != null)
                    setItemValue(doc, Def.CITY, padd.getCity().getPropertyValueAsString());
                
                if(padd.getState() != null)
                    setItemValue(doc, Def.STATE, padd.getState().getPropertyValueAsString());
                
                if(padd.getPostalCode() != null)
                    setItemValue(doc, Def.ZIP, padd.getPostalCode().getPropertyValueAsString());
                
                if(padd.getCountry() != null)
                    setItemValue(doc, Def.COUNTRY, padd.getCountry().getPropertyValueAsString());
                
            }
            
            if(categories != null)
                setItemValue(doc, Def.CATEGORIES, categories.getPropertyValueAsString());
            
            
            String[] webPages;
            
            webPages = UtilityContact.getWebPagesFromFoundation(contact, new String[]{Def.BUSINESS_WEBPAGE});
            if (webPages.length>0) {
                setItemValue(doc, Def.WEB_SITE, webPages[0]);
            } else {
                setItemValue(doc, Def.WEB_SITE, "");
            }
            
            webPages = UtilityContact.getWebPagesFromFoundation(contact, new String[]{Def.BUSINESS_2_WEBPAGE});
            if (webPages.length>0) {
                setItemValue(doc, Def.WORK_2_URL, webPages[0]);
            } else {
                setItemValue(doc, Def.WORK_2_URL, "");
            }
            
            webPages = UtilityContact.getWebPagesFromFoundation(contact, new String[]{Def.BUSINESS_3_WEBPAGE});
            if (webPages.length>0) {
                setItemValue(doc, Def.WORK_3_URL, webPages[0]);
            } else {
                setItemValue(doc, Def.WORK_3_URL, "");
            }
            
            setItemValue(doc, Def.HOME_URL, UtilityContact.getPersonalWebPageFromFoundation(contact));
            setItemValue(doc, Def.OTHER_URL, UtilityContact.getOtherWebPageFromFoundation(contact));
            
            
            
            List notes = contact.getNotes();
            if (notes != null) {
                String comment = "";
                for (Iterator it = notes.iterator(); it.hasNext();) {
                    Note note = (Note) it.next();
                    comment += note.getPropertyValueAsString();
                }
                setItemValue(doc, Def.COMMENT, comment);
                
            }
            setItemValue(doc, Def.ASSISTANT_PHONE,
                    UtilityContact.getBusinessPhoneFromFoundation(contact,
                    Def.ASSISTANT_PHONE_NUM));
            
            setItemValue(doc, Def.PAGER_PHONE_NUMBER,
                    UtilityContact.getBusinessPhoneFromFoundation(contact,
                    Def.PAGER_NUMBER));
            
            setItemValue(doc, Def.WORK2_PHONE,
                    UtilityContact.getBusinessPhoneFromFoundation(contact,
                    Def.BUSINESS_2_PHONE_NUM));
            
            setItemValue(doc, Def.HOME2_PHONE,
                    UtilityContact.getPersonalPhoneFromFoundation(contact,
                    Def.PERSONAL_HOME_2_PHONE_NUM));
            
            // different in VCARD and XML
            if (itemformat.equals(Def.VCARD_TYPE)) {
                setItemValue(doc, Def.HOME_PHONE_NUMBER,
                        UtilityContact.getPersonalPhoneFromFoundation(contact, 1));
            } else {
                setItemValue(doc, Def.HOME_PHONE_NUMBER,
                        UtilityContact.getPersonalPhoneFromFoundation(contact, 3));
            }
            
            setItemValue(doc, Def.HOME_FAX_PHONE_NUMBER,
                    UtilityContact.getPersonalPhoneFromFoundation(contact, 5));
            setItemValue(doc, Def.OFFICE_PHONE_NUMBER,
                    UtilityContact.getBusinessPhoneFromFoundation(contact, 1));
            setItemValue(doc, Def.OFFICE_FAX_PHONE_NUMBER,
                    UtilityContact.getBusinessPhoneFromFoundation(contact, 3));
            
            // same in VCARD and XML
            setItemValue(doc, Def.PERSONAL_CELL_PHONE,
                    UtilityContact.getPersonalPhoneFromFoundation(contact, 2));
            
            setItemValue(doc, Def.BUSINESS_CELL_PHONE,
                    UtilityContact.getBusinessPhoneFromFoundation(contact, 2));
            
            // MAIL OTHER
            // different in VCARD and XML
            if (itemformat.equals(Def.VCARD_TYPE)) {
                setItemValue(doc, Def.MAIL_OTHER,
                        UtilityContact.getPersonalEmailFromFoundation(contact, 1));
            } else {
                setItemValue(doc, Def.MAIL_OTHER,
                        UtilityContact.getPersonalEmailFromFoundation(contact, 2));
            }
            
            // MAIL HOME
            // different in VCARD and XML
            if (itemformat.equals(Def.VCARD_TYPE)) {
                setItemValue(doc, Def.MAIL_HOME,
                        UtilityContact.getPersonalEmailFromFoundation(contact, 2));
            } else {
                setItemValue(doc, Def.MAIL_HOME,
                        UtilityContact.getPersonalEmailFromFoundation(contact, 1));
            }
            
            // MAIL WORK
            String[] businessEmails;
            businessEmails = UtilityContact.getEmailsOfTypeFromFoundation(contact, Def.EMAIL_3_ADDRESS);
            if (businessEmails.length > 0) {
                setItemValue(doc, Def.MAIL_WORK, businessEmails[0]);
            }
            businessEmails = UtilityContact.getEmailsOfTypeFromFoundation(contact, Def.BUSINESS_EMAIL_2_ADDRESS);
            if (businessEmails.length > 0) {
                setItemValue(doc, Def.MAIL_WORK2, businessEmails[0]);
            }
            businessEmails = UtilityContact.getEmailsOfTypeFromFoundation(contact, Def.BUSINESS_EMAIL_3_ADDRESS);
            if (businessEmails.length > 0) {
                setItemValue(doc, Def.MAIL_WORK3, businessEmails[0]);
            }
            
            
        } catch (Exception e) {
            throw new EntityException(e);
        }
        
        return doc;
    }
    
    /**
     * @param doc Document
     * @return ItemContact
     */
    private ItemContact createFoundationContact(Document doc,
            String itemFormat)
            throws EntityException {
        
        WebPage webPage;
        ItemContact ctmp = new ItemContact();
        
        String uid;
        
        try {
            uid = doc.getUniversalID();
        } catch (Exception e) {
            throw new EntityException(e);
        }
        ctmp.setId(uid);
        
        Name name = ctmp.getName();
        PersonalDetail pd = ctmp.getPersonalDetail();
        Address padd = ctmp.getPersonalDetail().getAddress();
        BusinessDetail bd = ctmp.getBusinessDetail();
        Address badd = ctmp.getBusinessDetail().getAddress();
        
        try {
            
            String title = doc.getItemValueString(Def.TITLE);
            if ("0".equals(title))
                title = "";
            if ("[0]".equals(title)) {
                title = "0";
            }
            name.getSalutation().setPropertyValue(title);
            
            String firstName = "";
            String lastName = "";
            if(doc.getItemValueString(Def.FIRST_NAME) != null) {
                firstName = doc.getItemValueString(Def.FIRST_NAME);
            }
            
            if(doc.getItemValueString(Def.LAST_NAME) != null) {
                lastName = doc.getItemValueString(Def.LAST_NAME);
            }
            if ("".equals(firstName) || "".equals(firstName)) {
                name.getDisplayName().setPropertyValue(firstName + lastName);
            } else {
                name.getDisplayName().setPropertyValue(firstName + ", " + lastName);
            }
            
            name.getFirstName().setPropertyValue(
                    doc.getItemValueString(Def.FIRST_NAME));
            name.getMiddleName().setPropertyValue(
                    doc.getItemValueString(Def.MIDDLE_INITIAL));
            name.getLastName().setPropertyValue(
                    doc.getItemValueString(Def.LAST_NAME));
            
            String suffix = doc.getItemValueString(Def.SUFFIX);
            if ("0".equals(suffix)) {
                suffix = "";
            }
            if ("[0]".equals(suffix)) {
                suffix = "0";
            }
            name.getSuffix().setPropertyValue(suffix);
            
            name.getNickname().setPropertyValue(
                    doc.getItemValueString(Def.SHORT_NAME));

            DateTime birthday = Utility.getDominoDateTime(doc, Def.BIRTHDAY);
            if (birthday != null) {
                pd.setBirthday(Utility.setClientDateTime(birthday));
            } else {
                String strBirthday = doc.getItemValueString(Def.BIRTHDAY);
                if (Utility.isAllDayFormat(strBirthday)) {
                    pd.setBirthday(strBirthday);
                } else {
                    pd.setBirthday("");
                }
            }
            
            DateTime anniversary = Utility.getDominoDateTime(doc, Def.ANNIVERSARY);
            if (anniversary != null) {
                pd.setAnniversary(Utility.setClientDateTime(anniversary));
            } else {
                 String strAnniversary = doc.getItemValueString(Def.ANNIVERSARY);
                if (Utility.isAllDayFormat(strAnniversary)) {
                    pd.setAnniversary(strAnniversary);
                } else {
                    pd.setAnniversary("");
                }
            }
            
            pd.setSpouse(doc.getItemValueString(Def.SPOUSE));
            pd.setChildren(doc.getItemValueString(Def.CHILDREN));
            
            bd.setManager(doc.getItemValueString(Def.MANAGER));
            bd.setAssistant(doc.getItemValueString(Def.ASSISTANT));
            bd.getDepartment().setPropertyValue(
                    doc.getItemValueString(Def.DEPARTMENT));
            
            String company = doc.getItemValueString(Def.COMPANY_NAME);
            
            if ("[UNKNOWN]".equals(company)) {
                company = "";
            }
            
            bd.getCompany().setPropertyValue(company);
            UtilityContact.setBusinessTitleInFoundation(ctmp,
                    doc.getItemValueString(Def.JOB_TITLE), 1);
            
            UtilityContact.addWebPageInFoundation(ctmp, Def.BUSINESS_WEBPAGE, doc.getItemValueString(Def.WEB_SITE));
            
            padd.getStreet().setPropertyValue(
                    doc.getItemValueString(Def.STREET_ADDRESS));
            padd.getCity().setPropertyValue(
                    doc.getItemValueString(Def.CITY));
            padd.getState().setPropertyValue(
                    doc.getItemValueString(Def.STATE));
            padd.getPostalCode().setPropertyValue(
                    doc.getItemValueString(Def.ZIP));
            padd.getCountry().setPropertyValue(
                    doc.getItemValueString(Def.COUNTRY));
            
            badd.getStreet().setPropertyValue(
                    doc.getItemValueString(Def.OFFICE_STREET_ADDRESS));
            badd.getCity().setPropertyValue(
                    doc.getItemValueString(Def.OFFICE_CITY));
            badd.getState().setPropertyValue(
                    doc.getItemValueString(Def.OFFICE_STATE));
            badd.getPostalCode().setPropertyValue(
                    doc.getItemValueString(Def.OFFICE_ZIP));
            badd.getCountry().setPropertyValue(
                    doc.getItemValueString(Def.OFFICE_COUNTRY));
            
            // categories
            Vector dominoCategories = doc.getItemValue(Def.CATEGORIES);
            String formatedCategories = "";
            for (Iterator it = dominoCategories.iterator(); it.hasNext();) {
                formatedCategories += (String) it.next();
                if (it.hasNext())
                    formatedCategories += ", ";
            }
            ctmp.getCategories().setPropertyValue(formatedCategories);
            
            
            UtilityContact.setPersonalWebPageInFoundation(ctmp,
                    doc.getItemValueString(Def.HOME_URL));
            
            webPage = new WebPage(doc.getItemValueString(Def.OTHER_URL));
            webPage.setPropertyType(Def.OTHER_WEBPAGE);
            
            pd.addWebPage(webPage);
            
            Note note = new Note();
            note.setNoteType(Def.BODY);
            note.setPropertyValue(doc.getItemValueString(Def.COMMENT));
            ctmp.addNote(note);
            
            
            UtilityContact.setBusinessPhoneInFoundation(ctmp,
                    doc.getItemValueString(Def.ASSISTANT_PHONE),
                    Def.ASSISTANT_PHONE_NUM);
            
            UtilityContact.setBusinessPhoneInFoundation(ctmp,
                    doc.getItemValueString(Def.PAGER_PHONE_NUMBER),
                    Def.PAGER_NUMBER);
            
            UtilityContact.setBusinessPhoneInFoundation(ctmp,
                    doc.getItemValueString(Def.WORK2_PHONE),
                    Def.BUSINESS_2_PHONE_NUM);
            
            UtilityContact.setPersonalPhoneInFoundation(ctmp,
                    doc.getItemValueString(Def.HOME2_PHONE),
                    Def.PERSONAL_HOME_2_PHONE_NUM);
            
            // different in VCARD and XML
            if (itemFormat.equals(Def.VCARD_TYPE)) {
                UtilityContact.setPersonalPhoneInFoundation(ctmp,
                        doc.getItemValueString(Def.HOME_PHONE_NUMBER), 1);
            } else {
                UtilityContact.setPersonalPhoneInFoundation(ctmp,
                        doc.getItemValueString(Def.HOME_PHONE_NUMBER), 3);
            }
            
            UtilityContact.setPersonalPhoneInFoundation(ctmp,
                    doc.getItemValueString(Def.HOME_FAX_PHONE_NUMBER), 5);
            UtilityContact.setBusinessPhoneInFoundation(ctmp,
                    doc.getItemValueString(Def.OFFICE_PHONE_NUMBER), 1);
            UtilityContact.setBusinessPhoneInFoundation(ctmp,
                    doc.getItemValueString(Def.OFFICE_FAX_PHONE_NUMBER), 3);
            
            // same in VCARD and XML
            UtilityContact.setPersonalPhoneInFoundation(ctmp,
                    doc.getItemValueString(Def.PERSONAL_CELL_PHONE), 2);
            
            UtilityContact.setBusinessPhoneInFoundation(ctmp,
                    doc.getItemValueString(Def.BUSINESS_CELL_PHONE), 2);
            
            // MAIL OTHER
            // different in VCARD and XML
            if (itemFormat.equals(Def.VCARD_TYPE)) {
                // ICAL
                UtilityContact.setEmail1InFoundation(ctmp,
                        doc.getItemValueString(Def.MAIL_OTHER));
            } else {
                // XML
                UtilityContact.setEmail2InFoundation(ctmp,
                        doc.getItemValueString(Def.MAIL_OTHER));
            }
            
            // MAIL HOME
            // different in VCARD and XML
            if (itemFormat.equals(Def.VCARD_TYPE)) {
                // ICAL
                UtilityContact.setEmail2InFoundation(ctmp,
                        doc.getItemValueString(Def.MAIL_HOME));
            } else {
                // XML
                UtilityContact.setEmail1InFoundation(ctmp,
                        doc.getItemValueString(Def.MAIL_HOME));
            }
            
            // MAILS WORK
            UtilityContact.addEmailOfTypeInFoundation(ctmp, Def.EMAIL_3_ADDRESS,
                    doc.getItemValueString(Def.MAIL_WORK));
        } catch (Exception e) {
            throw new EntityException(e);
        }
        
        return ctmp;
    }
}
