/**
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.domino.items.manager ;

import com.funambol.domino.xml.XmlParseException;

import java.util.logging.Level;

import com.funambol.domino.engine.cache.SyncItemInfo;


import lotus.domino.Document;
import lotus.domino.DocumentCollection;
import lotus.domino.DateTime;
import lotus.domino.View;
import lotus.domino.Database;
import lotus.domino.RichTextItem;


import com.funambol.domino.exception.*;
import com.funambol.domino.items.dao.TaskDAO;
import com.funambol.domino.items.model.ItemTask;
import com.funambol.domino.util.*;

import java.util.Date;
import java.util.List;
import java.util.LinkedList;

/**
 * @author Gilberto Migliavacca
 */
public class TaskManager extends EntityManager {
    
    
    /**
     *
     */
    public TaskManager() {
    }
    
    /**
     * @param dominoWrapper_e DominoWrapper
     * @throws DominoAccessException
     */
    public TaskManager(DominoWrapper dominoWrapper_e)
    throws DominoAccessException {
        dominoWrapper = dominoWrapper_e;
        dao = new TaskDAO(dominoWrapper_e);
    }
    
    // ---------------------------------------------------------- Public Methods
    
    
    /**
     * @param UID String
     * @return ItemTask
     * @throws EntityException
     */
    public ItemTask getItemByUID(String UID,
            String itemFormat)
            throws EntityException {
        
        if (log.isLoggable(Level.FINEST)) {
            log.finest("getItemByUID start " + UID);
        }
        Document doc;
        try {
            doc = dao.getItemByUID(UID, this.dominoWrapper);
        } catch (Exception e) {
            throw new EntityException(e);
        }
        
        // converts domino doc in event
        ItemTask ctmpout = null;
        if (doc != null) {
            try {
                ctmpout = this.createFoundationTask(doc, itemFormat);
            } catch (Exception e) {
                throw new EntityException(e);
            }
        }
        
        return ctmpout;
    }
    
    
    /**
     * @return SyncItemInfo[]
     * @throws EntityException
     */
    public SyncItemInfo[] getAllSyncItemInfo(String viewname)
    throws EntityException {
        
        if (log.isLoggable(Level.FINEST)) {
            log.finest("getAllSyncItemInfo Start");
        }
        
        SyncItemInfo[] syncItemInfos;
        try {
            View view = this.dominoWrapper.getView(viewname);
            syncItemInfos = dao.getAllSyncItemInfo(view);
        } catch (Exception e) {
            throw new EntityException(e);
        }
        
        if (log.isLoggable(Level.FINEST)) {
            log.finest("Domino List with " + syncItemInfos.length + " Document");
        }
        
        return syncItemInfos;
        
    }
    
    
    /**
     * updates a document defined by UID
     * In the parameter content there are all the
     * value from the client
     *
     * @param oldTask    ItemTask
     * @param newTask    ItemTask
     * @param itemFormat String
     * @param iscal      boolean
     * @throws EntityException
     */
    public void updateItem(ItemTask oldTask,
            ItemTask newTask,
            String itemFormat,
            boolean iscal)
            throws EntityException {
        
        if (log.isLoggable(Level.FINEST)) {
            log.finest("Start - Task updateItem " +
                    "itemFormat (" + itemFormat + ") " +
                    "is task type (" + iscal + ")");
        }
        
        Document doc = null;
        String uid = oldTask.getId();
        Document docToUptdate = dao.getItemByUID(uid, this.dominoWrapper);
        
        if (itemFormat.equals(Def.ICAL_TYPE)) {
            // check if the content contains VEVENT or VTODO
            if (iscal) {
                doc = updateDominoTask(docToUptdate,
                        newTask,
                        itemFormat);
            }
        } else { // @todo check XML
            doc = updateDominoTask(docToUptdate,
                    newTask,
                    itemFormat);
        }
        
        try {
            dao.updateItem(doc);
        } catch (EntityException e) {
            throw new EntityException(e);
        }
        
    }
    
    
    /**
     * @param task       ItemTask
     * @param itemFormat String
     * @param istask     boolean
     * @return ItemTask
     * @throws EntityException
     */
    public ItemTask insertItem(ItemTask task,
            String itemFormat,
            boolean istask)
            throws EntityException {
        
        if (log.isLoggable(Level.FINEST)) {
            log.finest("Start - Task insertItem " +
                    "format (" + itemFormat + ")" +
                    "is task type (" + istask + ")");
        }
        
        Document docNew;
        Document doc = null;
        
        if (itemFormat.equals(Def.ICAL_TYPE)) {
            // check if the content contains VEVENT or VTODO
            if (istask) {
                doc = createDominoTask(task,
                        itemFormat,
                        Def.TODO_FORM_NAME);
            }
        } else { // @todo check XML
            doc = createDominoTask(task,
                    itemFormat,
                    Def.TODO_FORM_NAME);
        }
        
        try {
            docNew = dao.insertItem(doc);
        } catch (EntityException e) {
            throw new EntityException(e);
        }
        
        // converts domino doc in event
        ItemTask ctmpout;
        try {
            ctmpout = this.createFoundationTask(docNew, itemFormat);
        } catch (Exception e) {
            throw new EntityException(e);
        }
        
        return ctmpout;
    }
    
    /**
     * checks if a task is already inserted in the db
     * and returns the Document in the DB
     * we use: startdate, duedate, subject as search criteria
     *
     * @param task       ItemTask
     * @param itemFormat String
     * @return found ItemTask already present
     */
    public ItemTask[] getItemsByWhereClause(ItemTask task,
            String itemFormat)
            throws EntityException {
        List foundItems;
        String subject, startDate, dueDate, formula;
        DateTime startDateTime, dueDateTime;
        
        if (log.isLoggable(Level.FINEST)) {
            log.finest("getItemByWhereClause start ");
        }
        
        foundItems = new LinkedList();
        
        try {
            startDate = DateTools.toClientDate(task.getDate());
            dueDate = DateTools.toClientDate(task.getDueDate());
        } catch (XmlParseException ex) {
            throw new EntityException(ex);
        }
        subject = task.getSubject();
        
        if (log.isLoggable(Level.FINEST)) {
            log.finest("baseline parameters: StartDate: " + startDate + "DueDate: " + dueDate + "Subject: " + subject);
        }
        
        startDateTime = Utility.setDominoDateTime(dominoWrapper, startDate);
        dueDateTime = Utility.setDominoDateTime(dominoWrapper, dueDate);
        
        //lotus domino understands LEI as null
        String sdds = "LEI";
        if (startDateTime != null)
            sdds = "@Date([" + startDateTime + "])";
        
        String edds = "LEI";
        if (dueDateTime != null)
            edds = "@Date([" + dueDateTime + "])";
        
        formula = Def.SUBJECT + " = \"" + subject + "\" & " +
                Def.START_DATE_TIME + " = " + sdds + " & " +
                Def.DUE_DATE_TIME + " = " + edds;
        
        try {
            
            if (log.isLoggable(Level.FINEST)) {
                log.finest("Domino Formula " + formula);
            }
            
            DocumentCollection dc = dominoWrapper.getDatabase().search(formula);
            
            if (log.isLoggable(Level.FINEST)) {
                log.finest("Found task number " + dc.getCount());
            }
            
            if (dc.getCount() > 0) {
                if (log.isLoggable(Level.FINEST)) {
                    log.finest("Task already Exist");
                }
                for (int i=0; i<dc.getCount(); i++) {
                    foundItems.add(createFoundationTask(dc.getNthDocument(i+1), itemFormat));
                }
            }
        } catch (Exception e) {
            throw new EntityException(e);
        }
        return (ItemTask[]) foundItems.toArray(new ItemTask[0]);
    }
    
    // ---------------------------------------------------------- Public Methods
    
    /**
     * get Documento from Domino using UID
     * check if the Docoment.FORM property is: Appointment or Task
     *
     * @param dominoWrapper DominoWrapper
     * @param UID           String
     * @throws EntityException
     */
    public boolean isTask(DominoWrapper dominoWrapper, String UID)
    throws EntityException {
        
        
        Document doc;
        try {
            doc = dao.getItemByUID(UID, dominoWrapper);
        } catch (Exception e) {
            throw new EntityException(e);
        }
        
        String form;
        try {
            form = doc.getItemValueString(Def.FORM);
        } catch (Exception e) {
            throw new EntityException("Error getting " + Def.FORM, e);
        }
        
        if (log.isLoggable(Level.FINEST)) {
            log.finest("Form: " + form);
        }
        
        // task
        if (form.equalsIgnoreCase(Def.TODO_FORM_NAME)) {
            if (log.isLoggable(Level.FINEST)) {
                log.finest("create TASK from Task");
            }
            return true;
        }
        // calendar
        else if (form.equalsIgnoreCase(Def.CALENDAR_FORM_NAME)) {
            if (log.isLoggable(Level.FINEST)) {
                log.finest("create EVENT from Calendar");
            }
            return false;
        }
        // otherwise error
        else {
            throw new EntityException("Error checking domino document Type");
        }
        
    }
    
    //----------------------------------------------------------- PRIVATE METHODS
    
    
    private Document createDominoTask(ItemTask task,
            String itemFormat,
            String dominoFormName)
            throws EntityException {
        
        
        if (log.isLoggable(Level.FINEST)) {
            log.finest("TASK: TaskManager content " +
                    "\n TASK: TaskManager format " + itemFormat);
        }
        
        Document docToReturn = null;
        String dominoTaskType;
        
        // convert client task type into domino task type
        dominoTaskType = task.isTeamTask() ? "2" : "1";
        
        // switch
        if (dominoTaskType.equals("1")) {
            // personal ToDo
            docToReturn = createDominoPersonalTask(task, dominoFormName, itemFormat);
        } else if (dominoTaskType.equals("2")) {
            
            if (log.isLoggable(Level.FINEST)) {
                log.finest("NOT IMPLEMENTED YET");
            }
            
            // group ToDo
            docToReturn = createDominoGroupTask(task, dominoFormName, itemFormat);
        }
        
        if (log.isLoggable(Level.FINEST)) {
            log.finest("createDominoTask end ");
        }
        
        return docToReturn;
    }
    
    /**
     * @param docToUptdate
     * @param newTask
     * @param itemFormat
     * @return
     * @throws EntityException
     */
    private Document updateDominoTask(Document docToUptdate,
            ItemTask newTask,
            String itemFormat)
            throws EntityException {
        
        if (log.isLoggable(Level.FINEST)) {
            log.finest("update Task\n: ... ");
        }
        
        Document docToReturn = null;
        String dominoTaskType;
        
        dominoTaskType = newTask.isTeamTask() ? "2" : "1";
        
        if (dominoTaskType.equals("1")) {
            docToReturn = updateDominoPersonalTask(docToUptdate, newTask, itemFormat);
            
        } else if (dominoTaskType.equals("2")) {
            
            if (log.isLoggable(Level.FINEST)) {
                log.finest("NOT IMPLEMENTED YET");
            }
            docToReturn = updateDominoGroupTask(docToUptdate, newTask, itemFormat);
        }
        
        if (log.isLoggable(Level.FINEST)) {
            log.finest("updateDominoTask end ");
        }
        
        return docToReturn;
    }
    
    /**
     * @noinspection UNUSED_SYMBOL,UNUSED_THROWS
     *///TODO
    private Document createDominoGroupTask(
            ItemTask task,
            String dominoFormName,
            String itemFormat)
            throws EntityException {
        
        
        return null;
    }
    
    
    /**
     * creates an appointment for Domino
     *
     * @param task           ItemTask
     * @param dominoFormName String
     * @param itemFormat     String
     * @return Document
     * @throws EntityException
     */
    private Document createDominoPersonalTask(ItemTask task,
            String dominoFormName,
            String itemFormat)
            throws EntityException {
        
        if (log.isLoggable(Level.FINEST)) {
            log.finest("createDominoPersonalToDo - Start ");
        }
        
        Document doc;
        Database db;
        
        String subject = task.getSubject();
        
        String body = task.getTextDescription();
        
        String start = DateTools.dateToPDIDate(task.getDate());
        DateTime startDate = Utility.setDominoDateTime(dominoWrapper, start);
        
        String end = DateTools.dateToPDIDate(task.getDueDate());
        DateTime dueDate = Utility.setDominoDateTime(dominoWrapper, end);
        
        int reminderOffset = 0;
        String reminderTime = null;
        if (task.getReminderSet()) {
            reminderTime = DateTools.dateToPDIDate(task.getReminderTime());
            reminderOffset = Utility.calcAlarmOffsetInMinutes(task.getDate(),
                    task.getDueDate(),
                    task.getReminderTime());
        }
        
        // confidential
        // sensitity tag for SIF-XML e CLASS for ICAL
        String clientConfidential = task.getSensitivity();
        String dominoConfidential = Utility.setDominoConfidential(clientConfidential,
                itemFormat);
        
        String importance = task.getImportance();
        
        String status = task.getStatus();
        
        String categories = task.getCategories();
        
        // print client value
        if (log.isLoggable(Level.FINEST)) {
            log.finest("\n subject   : " + subject
                    + "\n body      : " + body
                    + "\n reminder offset     : " + reminderOffset
                    + "\n reminder time : " + reminderTime
                    + "\n start date    : " + startDate
                    + "\n due date      : " + dueDate
                    + "\n confidential      : " + dominoConfidential);
        }
        
        // create a Domino Appointment
        try {
            db = dominoWrapper.getDatabase();
            doc = db.createDocument();
            
            doc.appendItemValue(Def.FORM, dominoFormName);
            
            doc.appendItemValue(Def.PRINCIPAL, dominoWrapper.getRemoteUserName());
            doc.appendItemValue(Def.FROM, dominoWrapper.getRemoteUserName());
            doc.appendItemValue(Def.CHAIR, dominoWrapper.getRemoteUserName());
            
            doc.appendItemValue(Def.VIEW_ICON, new Integer(168));
            
            doc.appendItemValue(Def.TASK_TYPE, "1");
            doc.appendItemValue(Def.PUBLIC_ACCESS, "1");
            doc.appendItemValue(Def.ORG_TABLE, "T0");
            doc.appendItemValue(Def.EXCLUDE_FROM_VIEW, "D");
            doc.appendItemValue(Def.SEQUENCE_NUM, new Integer(1));
            
            doc.appendItemValue(Def.SUBJECT, subject);
            RichTextItem rti = doc.createRichTextItem(Def.BODY);
            rti.appendText(body);
            
            doc.appendItemValue(Def.CALENDAR_DATE_TIME, startDate);
            doc.appendItemValue(Def.START_DATE, startDate);
            doc.appendItemValue(Def.START_TIME, startDate);
            doc.appendItemValue(Def.START_DATE_TIME, startDate);
            
            // needed for client MailStd6
            doc.appendItemValue(Def.END_DATE, dueDate);
            doc.appendItemValue(Def.END_TIME, dueDate);
            doc.appendItemValue(Def.END_DATE_TIME, dueDate);
            
            doc.appendItemValue(Def.DUE_DATE, dueDate);
            doc.appendItemValue(Def.DUE_DATE_TIME, dueDate);
            
            // needed for client iNotes6
            doc.appendItemValue(Def.APPT_UNID, doc.getUniversalID());
            
            // alarm
            if (task.getReminderSet() && reminderOffset != 0) {
                doc.appendItemValue("Alarms", "1");
                doc.appendItemValue("$Alarm", "1");
                doc.appendItemValue("$AlarmUnit", Utility.calcAlarmOffsetUnit(reminderOffset));
                doc.appendItemValue("$AlarmOffset", new Integer(reminderOffset));
            }
            
            // confidential
            if (dominoConfidential != null) {
                doc.appendItemValue(Def.ORG_CONFIDENTIAL, dominoConfidential);
            }
            
            // importance
            if (importance != null) {
                if (importance.equals("2"))      // High
                    doc.appendItemValue(Def.IMPORTANCE, "1");
                else if (importance.equals("1")) // Normal
                    doc.appendItemValue(Def.IMPORTANCE, "2");
                else if (importance.equals("0")) // Low
                    doc.appendItemValue(Def.IMPORTANCE, "3");
            }
            // status
            if (status != null) {
                if (status.equals("0"))      // Future
                    doc.appendItemValue(Def.DUE_STATE, new Integer(2));
                else if (status.equals("1")) // Current
                    doc.appendItemValue(Def.DUE_STATE, new Integer(1));
                else if (status.equals("2")) // Complete
                    doc.appendItemValue(Def.DUE_STATE, new Integer(9));
                else if (status.equals("3")) // Overdue
                    doc.appendItemValue(Def.DUE_STATE, new Integer(0));
                else if (status.equals("4")) // Rejected
                    doc.appendItemValue(Def.DUE_STATE, new Integer(8));
            }
            
            
            
            // completed date time
            if(task.isComplete()) {
                String completed = DateTools.dateToPDIDate(task.getDateCompleted());
                DateTime dateCompleted = Utility.setDominoDateTime(dominoWrapper, completed);
                doc.appendItemValue(Def.DUE_STATE, new Integer(9));
                doc.appendItemValue(Def.COMPLETED_DATE_TIME, dateCompleted);
            }
            
            //categories
            if (categories != null)
                doc.appendItemValue(Def.CATEGORIES, categories);
            
            //doc.appendItemValue("$NoPurge",endDate) ;
            //doc.appendItemValue("StartTimeZone","Z=-1$DO=1$DL=3 -1 1 10 -1 1$ZN=Western/Central Europe") ;
            //doc.appendItemValue("EndTimeZone","Z=-1$DO=1$DL=3 -1 1 10 -1 1$ZN=Western/Central Europe") ;
            
            
        } catch (Exception e) {
            throw new EntityException(e.toString());
        }
        
        return doc;
    }
    
    /**
     * @noinspection UNUSED_THROWS,UNUSED_SYMBOL
     *///TODO
    private Document updateDominoGroupTask(Document docToUptdate,
            ItemTask task,
            String itemFormat)
            throws EntityException {
        
        
        return null;
    }
    
    
    /**
     * @param doc
     * @param task
     * @param itemFormat
     * @return
     * @throws EntityException
     */
    private Document updateDominoPersonalTask(Document doc,
            ItemTask task,
            String itemFormat)
            throws EntityException {
        
        if (log.isLoggable(Level.FINEST)) {
            log.finest("updateDominoPersonalToDo - Start ");
        }
        
        // get values from foundation
        String subject = task.getSubject();
        
        if (log.isLoggable(Level.FINEST)) {
            log.finest("Task check for subject |" + subject + "|");
        }
        
        String body = task.getTextDescription();
        String start = DateTools.dateToPDIDate(task.getDate());
        DateTime startDate = Utility.setDominoDateTime(dominoWrapper, start);
        
        String due = DateTools.dateToPDIDate(task.getDueDate());
        DateTime dueDate = Utility.setDominoDateTime(dominoWrapper, due);
        
        int reminderOffset = 0;
        String reminderTime = null;
        if (task.getReminderSet()) {
            reminderTime = DateTools.dateToPDIDate(task.getReminderTime());
            reminderOffset = Utility.calcAlarmOffsetInMinutes(task.getDate(),
                    task.getDueDate(),
                    task.getReminderTime());
        }
        
        // confidential
        // sensitity tag for SIF-XML e CLASS for ICAL
        String clientConfidential = task.getSensitivity();
        String dominoConfidential = Utility.setDominoConfidential(clientConfidential,
                itemFormat);
        
        String importance = task.getImportance();
        
        String status     = task.getStatus();
        
        String categories = task.getCategories();
        
        // print client value
        if (log.isLoggable(Level.FINEST)) {
            log.finest("\n subject   : " + subject
                    + "\n body      : " + body
                    + "\n reminder offset     : " + reminderOffset
                    + "\n reminder time : " + reminderTime
                    + "\n start date    : " + startDate
                    + "\n due date      : " + dueDate
                    + "\n confidential      : " + dominoConfidential);
        }
        
        try {
            setItemValue(doc, Def.SUBJECT, subject);
            setItemValue(doc, Def.BODY, body);
            
            setItemValue(doc, Def.CALENDAR_DATE_TIME, startDate);
            
            setItemValue(doc, Def.START_DATE, startDate);
            setItemValue(doc, Def.START_TIME, startDate);
            setItemValue(doc, Def.START_DATE_TIME, startDate);
            
            setItemValue(doc, Def.END_DATE, dueDate);
            setItemValue(doc, Def.END_TIME, dueDate);
            setItemValue(doc, Def.END_DATE_TIME, dueDate);
            
            setItemValue(doc, Def.DUE_DATE, dueDate);
            setItemValue(doc, Def.DUE_DATE_TIME, dueDate);
            
            // alarm
            if (task.getReminderSet() && reminderOffset != 0) {
                setItemValue(doc, "Alarms", "1");
                setItemValue(doc, "$Alarm", "1");
                setItemValue(doc, "$AlarmUnit",
                        Utility.calcAlarmOffsetUnit(reminderOffset));
                setItemValue(doc, "$AlarmOffset", new Integer(reminderOffset));
            } else {
                setItemValue(doc, "Alarms", "");
                doc.removeItem("$Alarm");
                doc.removeItem("$AlarmUnit");
                doc.removeItem("$AlarmOffset");
            }
            
            // confidential
            if (dominoConfidential != null) {
                setItemValue(doc, Def.ORG_CONFIDENTIAL, dominoConfidential);
            }
            
            // importance
            if (importance != null) {
                if (importance.equals("2"))      // High
                    setItemValue(doc, Def.IMPORTANCE, "1");
                else if (importance.equals("1")) // Normal
                    setItemValue(doc, Def.IMPORTANCE, "2");
                else if (importance.equals("0")) // Low
                    setItemValue(doc, Def.IMPORTANCE, "3");
            }
            
            // status
            if (status != null) {
                if (status.equals("0"))      // Future
                    setItemValue(doc, Def.DUE_STATE, new Integer(2));
                else if (status.equals("1")) // Current
                    setItemValue(doc, Def.DUE_STATE, new Integer(1));
                else if (status.equals("2")) // Complete
                    setItemValue(doc, Def.DUE_STATE, new Integer(9));
                else if (status.equals("3")) // Overdue
                    setItemValue(doc, Def.DUE_STATE, new Integer(0));
                else if (status.equals("4")) // Rejected
                    setItemValue(doc, Def.DUE_STATE, new Integer(8));
            }
            
            // completed date time
            if(task.isComplete()) {
                String completed = DateTools.dateToPDIDate(task.getDateCompleted());
                DateTime dateCompleted = Utility.setDominoDateTime(dominoWrapper, completed);
                setItemValue(doc, Def.DUE_STATE, new Integer(9));
                setItemValue(doc, Def.COMPLETED_DATE_TIME, dateCompleted);
            } else {
                
                doc.removeItem(Def.COMPLETED_DATE_TIME);
                doc.removeItem("ASSIGNSTATE");
                doc.removeItem("NoticeType");
                doc.removeItem("$CSTrack");
        
            }
            
            //categories
            if (categories != null)
                setItemValue(doc, Def.CATEGORIES, categories);
            
        } catch (Exception e) {
            throw new EntityException(e);
        }
        
        return doc;
    }
    
    
    /**
     * converts Domino task in Foundation Task
     *
     * @param dominoDoc  Document
     * @param itemFormat String
     * @return ItemTask
     */
    private ItemTask createFoundationTask(Document dominoDoc,
            String itemFormat)
            throws EntityException {
        
        if (log.isLoggable(Level.FINEST)) {
            log.finest("TASK: createSyncItem start");
        }
        
        // get values from Domino Document
        String subject;
        String body;
        String start;
        DateTime startDomino;
        String due;
        DateTime dueDomino;
        String dominoAlarm;
        long dominoOffset;
        String dominoConfidential;
        String clientConfidential;
        String dominoImportance;
        int dominoDueState;
        String categories;
        // get info from Domino Document
        try {
            
            
            subject = dominoDoc.getItemValueString(Def.SUBJECT);
            body = dominoDoc.getItemValueString(Def.BODY);
            
            
            startDomino = Utility.getDominoDateTime(dominoDoc, Def.START_DATE_TIME);
            start = Utility.setClientDateTime(startDomino);
            
            dueDomino = Utility.getDominoDateTime(dominoDoc, Def.DUE_DATE_TIME);
            due = Utility.setClientDateTime(dueDomino);
            
            // alarm
            dominoAlarm = dominoDoc.getItemValueString(Def.ALARMS);
            dominoOffset = (long) dominoDoc.getItemValueDouble(Def.ALARM_OFFSET);
            
            // confidential
            dominoConfidential = dominoDoc.getItemValueString(Def.ORG_CONFIDENTIAL);
            clientConfidential = Utility.setClientConfidential(dominoConfidential,
                    itemFormat);
            
            dominoImportance = dominoDoc.getItemValueString(Def.IMPORTANCE);
            dominoDueState = dominoDoc.getItemValueInteger(Def.DUE_STATE);
            
            //categories
            categories = getDocumentCategories(dominoDoc);
            
        } catch (Exception e) {
            throw new EntityException(e);
        }
        
        // create a new foundation task
        ItemTask task = new ItemTask();
        
        // set UID
        String uid;
        try {
            uid = dominoDoc.getUniversalID();
        } catch (Exception e) {
            throw new EntityException(e);
        }
        task.setId(uid);
        
        
        task.setSubject(subject);
        task.setTextDescription(body);
        try {
            
            task.setDate(DateTools.pdiToDate(start));
            
            task.setDueDate(DateTools.pdiToDate(due));
            
        } catch (XmlParseException ex) {
            throw new EntityException(ex);
        }
        
        
        if (dominoAlarm != null) {
            
            if (dominoAlarm.equalsIgnoreCase("1") && dominoOffset != 0) {
                
                task.setReminderSet(true);
                
                Date alarmDate;
                alarmDate = Utility.calcAlarmDate(task.getDate(),
                        task.getDueDate(),
                        dominoOffset);
                
                task.setReminderTime(alarmDate);
                
            } else {
                task.setReminderSet(false);
            }
            
        }
        
        // confidential
        if (clientConfidential != null) {
            task.setSensitivity(clientConfidential);
        }
        
        // importance
        if (dominoImportance.equals("1"))      // High
            task.setImportance("2");
        else if (dominoImportance.equals("2")) // Normal
            task.setImportance("1");
        else if (dominoImportance.equals("3")) // Low
            task.setImportance("0");
        
        task.setComplete(false);
        
        // status
        switch (dominoDueState) {
            case 0 :
                task.setStatus("3");  // Overdue
                break;
            case 1 :
                task.setStatus("1");  // Current
                break;
            case 2 :
                task.setStatus("0");  // Future
                break;
            case 8 :
                task.setStatus("4");  // Rejected
                break;
            case 9 :
                task.setStatus("2");  // Complete
                DateTime completedDomino = Utility.getDominoDateTime(dominoDoc, Def.COMPLETED_DATE_TIME);
                String dateCompleted = Utility.setClientDateTime(completedDomino);
                try {
                    task.setDateCompleted(DateTools.pdiToDate(dateCompleted));
                } catch (XmlParseException ex) {
                    log.finest("There was an error while trying to set \"DateCompleted\" foundation task property!");
                }
                task.setComplete(true);
                break;
        }
        
        // categories
        if (categories != null)
            task.setCategories(categories);
        
        if (log.isLoggable(Level.FINEST)) {
            log.finest("end setting ");
        }
        
        return task;
        
    }
    
}