/**
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */

package com.funambol.domino.items.model;

import com.funambol.foundation.pdi.event.Calendar;
import com.funambol.foundation.pdi.event.Event;
import java.util.Date;

/**
 * Represent an Exchange Contact
 *
 * @version $Id: ItemCalendar.java,v 1.1 2006/08/08 08:33:33 gibi Exp $
 **/
public class ItemCalendar extends com.funambol.foundation.pdi.event.Calendar {

    //------------------------------------------------------------- Private data

    private String     id                  = null  ;

    private String     href                = null  ;

    private Date       lastUpdate          = null  ;
    private char       modificationStatus  = ' '   ;

    //------------------------------------------------------------- Constructors

    /**
     * Creates a Calendar
     *
     */
    public ItemCalendar () {

    }

    /**
     * Creates a Calendar
     *
     *
     * @param id client item id
     *
     */
    public ItemCalendar (String id) {

        this.id = id ;

    }

    /**
     * Creates a Calendar
     *
     *
     * @param id client item id
     * @param t client item timestamp
     *
     */
    public ItemCalendar (String id, Date t) {

        this.id           = id ;
        this.lastUpdate   = t  ;

    }

    /**
     * Creates a Calendar
     *
     *
     * @param id
     * @param status
     *
     */
    public ItemCalendar (String id, char status) {

        this.id                   = id                 ;
        this.modificationStatus   = modificationStatus ;

    }

    /**
     * Creates a Calendar
     *
     *
     * @param id
     * @param href
     * @param status
     * @param lastUpdate
     *
     */
    public ItemCalendar (String     id        ,
                     String     href      ,
                     char       status    ,
                     Date       lastUpdate) {

        this.id                        = id          ;
        this.href                      = href        ;
        this.modificationStatus        = status      ;
        this.lastUpdate                = lastUpdate  ;

    }

    public ItemCalendar(String id, Calendar calendar) {

        this.setId(id);
        this.setCalScale(calendar.getCalScale());
        this.setEvent(calendar.getEvent());
        this.setMethod(calendar.getMethod());
        this.setProdId(calendar.getProdId());
        this.setVersion(calendar.getVersion());
        this.setXTags(calendar.getXTags());

    }

    //----------------------------------------------------------- Public methods

    public String getId                ()  {
        return id;
    }

    public Date getLastUpdate          ()  {
        return lastUpdate;
    }


    public char getModificationStatus  ()  {
        return modificationStatus;
    }

    public String getHref              ()  {
        return href;
    }

    public void setId(String id)                    {
        this.id = id;
    }

    public void setLastUpdate(Date lastUpdate)      {
        this.lastUpdate = lastUpdate;
    }

    public void setModificationStatus(char status)  {
        this.modificationStatus = status;
    }

    public void setHref(String href)                {
        this.href = href;
    }

    public void copyCommonProperties(ItemCalendar otherCalendar) {

        Event thisEvent = this.getEvent();
        Event otherEvent = otherCalendar.getEvent();

        thisEvent.setAllDay(otherEvent.getAllDay());

        if(otherEvent.getBusyStatus() != null)
            thisEvent.setBusyStatus(otherEvent.getBusyStatus());

        if(otherEvent.getCategories() != null)
            thisEvent.setCategories(otherEvent.getCategories());

        if(otherEvent.getClassEvent() != null)
            thisEvent.setClassEvent(otherEvent.getClassEvent());

        if(otherEvent.getCreated() != null)
            thisEvent.setCreated(otherEvent.getCreated());

        if(otherEvent.getDAlarm() != null)
            thisEvent.setDAlarm(otherEvent.getDAlarm());

        if(otherEvent.getDescription() != null)
            thisEvent.setDescription(otherEvent.getDescription());

        if(otherEvent.getDtEnd() != null)
            thisEvent.setDtEnd(otherEvent.getDtEnd());

        if(otherEvent.getDtStamp() != null)
            thisEvent.setDtStamp(otherEvent.getDtStamp());

        if(otherEvent.getDtStart() != null)
            thisEvent.setDtStart(otherEvent.getDtStart());

        if(otherEvent.getDuration() != null)
            thisEvent.setDuration(otherEvent.getDuration());

        if(otherEvent.getLastModified() != null)
            thisEvent.setLastModified(otherEvent.getLastModified());

        if(otherEvent.getLatitude() != null)
            thisEvent.setLatitude(otherEvent.getLatitude());

        if(otherEvent.getLocation() != null)
            thisEvent.setLocation(otherEvent.getLocation());

        if(otherEvent.getLongitude() != null)
            thisEvent.setLongitude(otherEvent.getLongitude());

        if(otherEvent.getMeetingStatus() != null)
            thisEvent.setMeetingStatus(otherEvent.getMeetingStatus());

        if(otherEvent.getMileage() != null)
            thisEvent.setMileage(otherEvent.getMileage());

        if(otherEvent.getOrganizer() != null)
            thisEvent.setOrganizer(otherEvent.getOrganizer());

        if(otherEvent.getPAlarm() != null)
            thisEvent.setPAlarm(otherEvent.getPAlarm());

        if(otherEvent.getPriority() != null)
            thisEvent.setPriority(otherEvent.getPriority());

        if(otherEvent.getRecurrencePattern() != null)
            thisEvent.setRecurrencePattern(otherEvent.getRecurrencePattern());

        if(otherEvent.getReminder() != null)
            thisEvent.setReminder(otherEvent.getReminder());

        if(otherEvent.getReplyTime() != null)
            thisEvent.setReplyTime(otherEvent.getReplyTime());

//        if(otherEvent.getRrule() != null)
//            thisEvent.setRrule(otherEvent.getRrule());

        if(otherEvent.getSequence() != null)
            thisEvent.setSequence(otherEvent.getSequence());

        if(otherEvent.getStatus() != null)
            thisEvent.setStatus(otherEvent.getStatus());

        if(otherEvent.getSummary() != null)
            thisEvent.setSummary(otherEvent.getSummary());

        if(otherEvent.getTransp() != null)
            thisEvent.setTransp(otherEvent.getTransp());

        if(otherEvent.getUid() != null)
            thisEvent.setUid(otherEvent.getUid());

        if(otherEvent.getUrl() != null)
            thisEvent.setUrl(otherEvent.getUrl());

        if(otherEvent.getXTags() != null)
            thisEvent.setXTag(otherEvent.getXTags());

    }
}
