/**
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */

package com.funambol.domino.items.model;


import com.funambol.foundation.pdi.contact.BusinessDetail;
import com.funambol.foundation.pdi.contact.Contact;
import com.funambol.foundation.pdi.contact.PersonalDetail;
import java.util.Date;

/**
 * Represent an Exchange Contact
 *
 * @version $Id: ItemContact.java,v 1.1 2006/08/08 08:33:33 gibi Exp $
 **/
public class ItemContact extends com.funambol.foundation.pdi.contact.Contact {

    //------------------------------------------------------------- Private data

    private String     id                = null ;
    private String     userId            = null ;

    private String     href              = null ;

    private Date       lastUpdate        = null ;
    private char       status            = ' '  ;

    //------------------------------------------------------------- Constructors

    /**
     * Creates a Contact
     *
     */
    public ItemContact() {

    }

    /**
     * Creates a Contact
     *
     *
     * @param id client item id
     *
     */
    public ItemContact(String id) {

        this.id = id   ;

    }

    /**
     * Creates a Contact
     *
     *
     * @param id client item id
     * @param t client item timestamp
     *
     */
    public ItemContact(String id, Date t) {

        this.id         = id   ;
        this.lastUpdate = t    ;

    }

    /**
     * Creates a Contact
     *
     *
     * @param id client item id
     * @param status client item status
     *
     */
    public ItemContact(String id, char status) {

        this.id      = id     ;
        this.status  = status ;

    }

    /**
     * Creates a Contact
     *
     *
     * @param id client item id
     * @param href reference to the item
     * @param status client item state
     * @param lastUpdate last update of the item
     *
     */
    public ItemContact(String   id         ,
            String   href       ,
            char     status     ,
            Date     lastUpdate ) {

        this.id             = id          ;
        this.href           = href        ;
        this.status         = status      ;
        this.lastUpdate     = lastUpdate  ;

    }

    public ItemContact(String id, Contact contact) {

        this.setId(id);
        this.setBusinessDetail(contact.getBusinessDetail());
        this.setCategories(contact.getCategories());
        this.setFolder(contact.getFolder());
        this.setImportance(contact.getImportance());
        this.setLanguages(contact.getLanguages());
        this.setMileage(contact.getMileage());
        this.setName(contact.getName());
        this.setNotes(contact.getNotes());
        this.setPersonalDetail(contact.getPersonalDetail());
        this.setRevision(contact.getRevision());
        this.setSensitivity(contact.getSensitivity());
        this.setSubject(contact.getSubject());
        this.setTimezone(contact.getTimezone());
        this.setUid(contact.getUid());
        this.setXTags(contact.getXTags());

    }
    //----------------------------------------------------------- Public methods


    public String getId()                      {
        return this.id;
    }

    public void setId(String id)               {
        this.id = id;
    }

    public String getUserId()                  {
        return this.userId;
    }

    public Date getLastUpdate()                {
        return this.lastUpdate;
    }

    public char getStatus()                    {
        return this.status;
    }

    public void setLastUpdate(Date lastUpdate) {
        this.lastUpdate = lastUpdate;
    }

    public void setStatus(char status)         {
        this.status = status;
    }

    public String getHref()                    {
        return this.href;
    }

    public void setHref(String href)           {
        this.href = href;
    }

//    public void copyCommonProperties(ItemContact otherContact) {
//
//        BusinessDetail businessDetail = this.getBusinessDetail();
//        BusinessDetail otherBusinessDetail = otherContact.getBusinessDetail();
//
//        if(otherBusinessDetail != null) {
//
//            if(businessDetail == null)
//                businessDetail = new BusinessDetail();
//
//            if(otherBusinessDetail.getAssistant() != null)
//                businessDetail.setAssistant(otherBusinessDetail.getAssistant());
//
//            if(otherBusinessDetail.getCompanies() != null)
//                businessDetail.setCompanies(otherBusinessDetail.getCompanies());
//
//
//            if(otherBusinessDetail.getEmails() != null)
//                businessDetail.setEmails(otherBusinessDetail.getEmails());
//
//            if(otherBusinessDetail.getManager() != null)
//                businessDetail.setManager(otherBusinessDetail.getManager());
//
//            if(otherBusinessDetail.getOfficeLocation() != null)
//                businessDetail.setOfficeLocation(otherBusinessDetail.getOfficeLocation());
//
//            if(otherBusinessDetail.getPhones() != null)
//                businessDetail.setPhones(otherBusinessDetail.getPhones());
//
//            if(otherBusinessDetail.getTitles() != null)
//                businessDetail.setTitles(otherBusinessDetail.getTitles());
//
//            if(otherBusinessDetail.getWebPages() != null)
//                businessDetail.setWebPages(otherBusinessDetail.getWebPages());
//        }
//        if(otherContact.getCategories() != null)
//            this.setCategories(otherContact.getCategories());
//
//        if(otherContact.getFolder() != null)
//            this.setFolder(otherContact.getFolder());
//
//        if(otherContact.getImportance() != null)
//            this.setImportance(otherContact.getImportance());
//
//        if(otherContact.getLanguages() != null)
//            this.setLanguages(otherContact.getLanguages());
//
//        if(otherContact.getMileage() != null)
//            this.setMileage(otherContact.getMileage());
//
//        if(otherContact.getName() != null)
//            this.setName(otherContact.getName());
//
//        if(otherContact.getNotes() != null)
//            this.setNotes(otherContact.getNotes());
//
//        PersonalDetail personalDetail = this.getPersonalDetail();
//        PersonalDetail otherPersonalDetail = otherContact.getPersonalDetail();
//
//        if(otherPersonalDetail != null) {
//
//            if(personalDetail == null)
//                personalDetail = new PersonalDetail();
//
//            if(otherPersonalDetail.getAnniversary() != null)
//                personalDetail.setAnniversary(otherPersonalDetail.getAnniversary());
//
//            if(otherPersonalDetail.getBirthday() != null)
//                personalDetail.setBirthday(otherPersonalDetail.getBirthday());
//
//            if(otherPersonalDetail.getChildren() != null)
//                personalDetail.setChildren(otherPersonalDetail.getChildren());
//
//            if(otherPersonalDetail.getEmails() != null)
//                personalDetail.setEmails(otherPersonalDetail.getEmails());
//
//            if(otherPersonalDetail.getGender() != null)
//                personalDetail.setGender(otherPersonalDetail.getGender());
//
//            if(otherPersonalDetail.getHobbies() != null)
//                personalDetail.setHobbies(otherPersonalDetail.getHobbies());
//
//            if(otherPersonalDetail.getPhones() != null)
//                personalDetail.setPhones(otherPersonalDetail.getPhones());
//
//            if(otherPersonalDetail.getSpouse() != null)
//                personalDetail.setSpouse(otherPersonalDetail.getSpouse());
//
//            if(otherPersonalDetail.getWebPages() != null)
//                personalDetail.setWebPages(otherPersonalDetail.getWebPages());
//        }
//
//        if(otherContact.getRevision() != null)
//            this.setRevision(otherContact.getRevision());
//
//        if(otherContact.getSensitivity() != null)
//            this.setSensitivity(otherContact.getSensitivity());
//
//        this.setStatus(otherContact.getStatus());
//
//        if(otherContact.getSubject() != null)
//            this.setSubject(otherContact.getSubject());
//
//        if(otherContact.getTimezone() != null)
//            this.setTimezone(otherContact.getTimezone());
//
//        if(otherContact.getUid() != null)
//            this.setUid(otherContact.getUid());
//
//        if(otherContact.getXTags() != null)
//            this.setXTags(otherContact.getXTags());
//
//    }

}
