/**
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.domino.util  ;

import lotus.domino.NotesFactory;
import lotus.domino.Database;
import lotus.domino.View;
import lotus.domino.Session;
import lotus.domino.DbDirectory;

import com.funambol.domino.exception.DominoAccessException;
import com.funambol.framework.logging.Sync4jLogger;

import java.util.logging.Logger;
import java.util.logging.Level;

/**
 *
 * <p>Create a connection with Lotus Dominio Server 6.5</p>
 *
 * @version $Id: DominoWrapper.java,v 1.1 2006/08/08 08:35:14 gibi Exp $
 */
public class DominoWrapper {

    // --------------------------------------------------------------- Constants


    // ------------------------------------------------------------ Private data

    protected Logger  log = Sync4jLogger.getLogger(Def.LOGGER_NAME);

    // -------------------------------------------------------------- Properties

    /**
     *
     */
    private Session  session         = null;
    /**
     *
     */
    private Database db              = null;
    /**
     * e.i. mail\snichele.nsf
     */
    private String   dbname          = null;
    /**
     * e.i. CN=stefano nichele/O=sainstudio
     */
    private String   remoteUserName  = null;
    /**
     * e.i. stefano nichele
     */
    private String   user            = null;


   /**
    *
    * gets session
    *
    *
    * @param server String
    * @param port int
    * @param login String
    * @param password String
    * @return boolean
    * @throws DominoAccessException
    */
   public boolean setSession(String server,
                              int port,
                              String login,
                              String password,
                              boolean ssl)
    throws DominoAccessException {

        boolean oksession = false;
        try {
            String ior = NotesFactory.getIOR(server+":"+port);
            if (log.isLoggable(Level.FINEST)) {
              log.finest("Domino " + ior);
            }
            if (log.isLoggable(Level.INFO)) {
              log.info("Domino ior got");
            }

            if(ssl) {
              String args[] = {"-ORBEnableSSLSecurity"};
              session = NotesFactory.createSessionWithIOR(ior, args, login,password);
            } else {
              session = NotesFactory.createSessionWithIOR(ior, login, password);
            }

            if (log.isLoggable(Level.INFO)) {
              log.info("Domino session started");
            }

            oksession = true;
        }catch (Exception e){
           e.printStackTrace();
           throw new DominoAccessException(e.toString());
        }
        return oksession;
    }

    /**
     *
     *
     * @param server String
     * @throws DominoAccessException
     */
    public void setProperties(String server) throws DominoAccessException {

       // get remote user and user
       try {
           remoteUserName = session.getUserName();
           user = session.getCommonUserName();
       }catch (Exception e){
           throw new DominoAccessException(e.toString());
       }

       // get DB
       try {
           DbDirectory dbd = session.getDbDirectory(server);
           db = dbd.openMailDatabase();
       }catch (Exception e){
           throw new DominoAccessException(e.toString());
       }

       // get DB path
       try {
           dbname = db.getFilePath();
       }catch (Exception e){
           throw new DominoAccessException(e.toString());
       }

   }


    /**
     * closes Domino connection
     *
     * @throws DominoAccessException
     */
    public void releaseProperties() throws DominoAccessException{
        try {
            db.recycle();
        } catch (Exception e) {
            throw new DominoAccessException("Error Closing connection to Domino ",
                                          e);
        }
    }

    /**
     * closes Domino connection
     *
     * @throws DominoAccessException
     */
    public void releaseSession() throws DominoAccessException{
        try {
            session.recycle();
        } catch (Exception e) {
            throw new DominoAccessException("Error Closing connection to Domino ",
                                          e);
        }
    }


    /**
     * gets a View
     *
     * @param name String
     * @return View
     * @throws DominoAccessException
     */
    public View getView(String name) throws DominoAccessException {
       View view    = null;
       try {
           view = db.getView(name);

       } catch (Exception e){
           throw new DominoAccessException(e.toString());
       }
       return view;
    }


    /**
     * gets the databse
     *
     * @return Database object
     */
    public Database getDatabase() {
        return this.db;
    }

    /**
     *
     * @return db name String
     */
    public String getDbName() {
        return this.dbname ;
    }

    /**
     *
     * @return remote User Name String
     */
    public String getRemoteUserName(){
        return this.remoteUserName ;
    }

    /**
     *
     * @return User String
     */
    public String getUser(){
        return this.user ;
    }

}
