/**
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.domino.util;

import com.funambol.domino.engine.cache.SyncItemInfo;
import com.funambol.domino.exception.EntityException;
import com.funambol.framework.engine.SyncItem;
import com.funambol.framework.logging.Sync4jLogger;
import com.funambol.framework.tools.Base64;
import lotus.domino.*;

import java.sql.Timestamp;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.TimeZone;
import java.util.logging.Level;
import java.util.logging.Logger;


/**
 * <p>Utility class</p>
 *
 * @version $Id: Utility.java,v 1.3 2006/09/07 09:49:31 diogo Exp $
 */

public class Utility {

    // -------------------------------------------------------------- Private data

    /**     */
    private static Logger log = Sync4jLogger.getLogger(Def.LOGGER_NAME);

    // ------------------------------------------------------------ Public Methods


    /**
     * converts the lotus DateTime to java.sql.Timestamp
     * used by getAllSyncItemInfo()
     *
     * @param dt DateTime
     * @return Timestamp
     */
    public static Timestamp dominoDateTimeToTimestamp(DateTime dt) {
        long dtlong = 0;
        try {
            dtlong = dt.toJavaDate().getTime();
        } catch (Exception e) {
            e.printStackTrace();
        }
        return new Timestamp(dtlong);
    }


    public static int calcAlarmOffsetInMinutes(Date start, Date end, Date alarm) {

        int offset = 0;

        if (alarm.getTime() < start.getTime()) {
            offset = (int) (alarm.getTime() - start.getTime()) / 1000 / 60;
        } else if (alarm.getTime() > end.getTime()) {
            offset = (int) (alarm.getTime() - end.getTime()) / 1000 / 60;
        }

        return offset;
    }


    public static Date calcAlarmDate(Date start, Date end, long offset) {

        offset *= 60000;

        if (offset > 0) {
            offset += end.getTime();
        } else if (offset < 0) {
            offset += start.getTime();
        } else
            return null;

        return new Date(offset);

    }

    public static String calcAlarmOffsetUnit(int offset) {

        if (offset < 0)
            offset *= -1;

        if (offset >= 1440)
            return "D";
        else if (offset >= 60 && offset < 1440)
            return "H";

        return "M";
    }

    /**
     * used by create and update Todo
     *
     * @param dominoWrapper DominoWrapper
     * @return DateTime
     */
    public static DateTime getDominoNow(DominoWrapper dominoWrapper) {

        Database db;
        Session stmp;
        DateTime currTime = null;

        try {
            db = dominoWrapper.getDatabase();
            stmp = db.getParent();
            currTime = stmp.createDateTime(new Date(System.currentTimeMillis()));
            currTime.setNow();
        } catch (Exception e) {
            e.printStackTrace();
        }

        return currTime;
    }


    /**
     * credantials format:  "username:passowrd"
     *
     * @param username    String
     * @param credentials String
     * @return password String
     */
    public static String getPassword(String username, String credentials) {
        String up = new String(Base64.decode(credentials));
        int l = username.length();
        return up.substring(l + 1, up.length());
    }


    /**
     * get Date from Domino Document
     * <p/>
     * if there are no related value to the input label
     * the method getItemValueDateTimeArray returns error
     *
     * @param doc   Document
     * @param label String
     * @return DateTime
     */
    public static DateTime getDominoDateTime(Document doc, String label) {

        DateTime sd = null;

        try {
            sd = (DateTime) doc.getItemValueDateTimeArray(label).elementAt(0);
        } catch (Exception e) {
            if (log.isLoggable(Level.SEVERE)) {
                log.severe("MANAGED ERROR: Error getting " + label);
            }
        }
        if (sd == null) {
            if (log.isLoggable(Level.SEVERE)) {
                log.severe("MANAGED ERROR: Error getting " + label);
            }
        }

        if (log.isLoggable(Level.FINEST)) {
            log.finest("getDominoDateTime item: " + label + " value: " + sd);
        }

        return sd;
    }


    /**
     * converts a Domino DateTime
     * in a client format:
     *
     * @param d DateTime
     * @return String
     */
    public static String setClientDateTime(DateTime d) {

        return setClientDateTime(d, TimeZone.getTimeZone("UTC"));
    }

    public static String setClientDateTime(DateTime d, TimeZone timeZone) {

        String dt = null;

        if (d == null) {
            return dt;
        }

        if (log.isLoggable(Level.FINEST)) {
            String zone = null;
            try {
                zone = d.getZoneTime();
            } catch (NotesException ex) {
                log.severe("Error getting timezone from Domino DateTime.");
            }
            log.finest("setClientDateTime DateTime " + d + ", TimeZone " + zone);
        }

        // we need dd/mm/yyyy HH:MM:SS format
        java.util.Date ud;
        try {
            ud = d.toJavaDate();
        } catch (Exception e) {
            log.severe("Error converting Domino DateTime.");
            return dt;
        }

        if (log.isLoggable(Level.FINEST)) {
            log.finest("setClientDateTime Util Date " + ud);
        }

        if (ud != null) {
            String pattern = "yyyyMMdd'T'HHmmss'Z'";
            SimpleDateFormat formatter = new SimpleDateFormat(pattern);
            formatter.setTimeZone(timeZone);
            dt = formatter.format(ud);
        }

        if (log.isLoggable(Level.FINEST)) {
            log.finest("setClientDateTime String " + dt);
        }

        return dt;
    }
//
//
//    /**
//     *
//     * @param o String
//     * @return String
//     */
//    public static String nullToVoid(String o){
//        String s = "";
//        if ((o==null) || (o.equalsIgnoreCase("null")) ){
//            s = "";
//        } else {
//            s = o;
//        }
//        return s;
//    }

    /**
     * @param conf       String
     * @param itemFormat String
     * @return String
     */
    public static String setDominoConfidential(String conf,
                                               String itemFormat) {
        String dominoConfidential = "0";

        if (conf != null) {
            if (itemFormat.equals(Def.ICAL_TYPE)) {
                // ICAL
                if (conf.equalsIgnoreCase("PRIVATE")) {
                    dominoConfidential = "1";
                }
            } else {
                // XML
                if (conf.equalsIgnoreCase("0")) {
                    dominoConfidential = "0";
                } else if (conf.equalsIgnoreCase("1") ||
                        conf.equalsIgnoreCase("2") ||
                        conf.equalsIgnoreCase("3")) {
                    dominoConfidential = "1";
                }
            }
        }
        return dominoConfidential;
    }

    /**
     * "in" parameter is a String with a date in UTC format:
     * some phones could send a date in UTC format
     * without 'Z' at the end (i.e. "20050329T110000")
     * <p/>
     * The method parses the Date using "yyyyMMdd'T'HHmmss'Z'" mask.
     * The method returns a Domino DateTime
     *
     * @param dw DominoWrapper
     * @return DateTime
     */
    public static DateTime setDominoDateTime(DominoWrapper dw, String in) {

        DateTime dominoDate = null;
        Date date;

        if (in == null) {
            return dominoDate;
        }

        if (log.isLoggable(Level.FINEST)) {
            log.finest("setDominoDateTime  input " + in);
        }

        try {
            SimpleDateFormat formatter = new SimpleDateFormat("yyyyMMdd'T'HHmmss'Z'");
            formatter.setTimeZone(TimeZone.getTimeZone("UTC"));
            date = formatter.parse(in);
            Database db = dw.getDatabase();
            Session stmp = db.getParent();
            dominoDate = stmp.createDateTime(date);
        } catch (Exception e) {
            e.printStackTrace();
        }


        if (log.isLoggable(Level.FINEST)) {
            log.finest("setDominoDateTime  input Domino Date " + dominoDate);
        }

        return dominoDate;
    }

    /**
     * "in" parameter is a String with a date in DAYFORMAT:
     * yyyy-mm-dd format (for all day event).
     * The method parses the Date using "yyyy-MM-dd" mask.
     * The method returns a Domino DateTime
     *
     * @param dw DominoWrapper
     * @return DateTime
     */
    public static DateTime setAllDayDomino_StartDateTime(DominoWrapper dw, String in) {

        DateTime dominoDate = null;

        if (in == null) {
            return dominoDate;
        }

        if (log.isLoggable(Level.FINEST)) {
            log.finest("setDominoAllDay_StartDateTime  input " + in);
        }

        try {
            String tmp = in + " 04:00";
            SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd HH:mm");
            Date dtmp = formatter.parse(tmp);
            dominoDate = dw.getDatabase().getParent().createDateTime(dtmp);
        } catch (Exception ne) {
            log.finest("Exception has occurred while formatting start date time for all day event");
        }

        if (log.isLoggable(Level.FINEST)) {
            log.finest("setDominoAllDay_StartDateTime  output " + dominoDate);
        }

        return dominoDate;
    }


    /**
     * "in" parameter is a String with a date in DAYFORMAT:
     * yyyy-mm-dd format (for all day event).
     * The method parses the Date using "yyyy-MM-dd" mask.
     * The method returns a Domino DateTime
     *
     * @param dw DominoWrapper
     * @return DateTime
     */
    public static DateTime setAllDayDomino_Date(DominoWrapper dw, String in) {


        DateTime dominoDate = null;

        if (in == null) {
            return dominoDate;
        }

        if (log.isLoggable(Level.FINEST)) {
            log.finest("setDominoAllDay_StartDate  input " + in);
        }

        try {
            SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd");
            Date dtmp = formatter.parse(in);
            dominoDate = dw.getDatabase().getParent().createDateTime(dtmp);
        } catch (Exception ne) {
            log.finest("Exception has occurred while formatting date for all day event");
        }

        if (log.isLoggable(Level.FINEST)) {
            log.finest("setDominoAllDay_StartDate  output " + dominoDate);
        }

        return dominoDate;
    }

    /**
     * we have to create a 04:00:00 AM DateTime
     *
     * @param dw DominoWrapper
     * @return DateTime
     */
    public static DateTime setAllDayDomino_StartTime(DominoWrapper dw) {

        DateTime dominoDate = null;

        try {
            dominoDate = dw.getDatabase().getParent().createDateTime("04:00:00 AM");
        } catch (Exception e) {
            log.finest("Exception has occurred while formatting start time for all day event");
        }

        if (log.isLoggable(Level.FINEST)) {
            log.finest("setDominoAllDay_StartTime  output " + dominoDate);
        }

        return dominoDate;
    }

    /**
     * returns a yyyy-MM-dd format date
     *
     * @param dateTime DateTime
     * @return String formated like this yyyy-MM-dd
     */
    public static String setAllDayClient_StartDate(DateTime dateTime) {

        String clientDate = null;

        try {
            java.util.Date d = dateTime.toJavaDate();
            SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd");
            clientDate = formatter.format(d);
        } catch (Exception e) {
            e.printStackTrace();
        }


        if (log.isLoggable(Level.FINEST)) {
            log.finest("setAllDayClient_StartDate  output " + clientDate);
        }

        return clientDate;
    }


    /**
     * "in" parameter is a String with a date in DAYFORMAT:
     * yyyy-mm-dd format (for all day event).
     * The method parses the Date using "yyyy-MM-dd" mask.
     * The method returns a Domino DateTime
     *
     * @param dw DominoWrapper
     * @return DateTime
     */
    public static DateTime setAllDayDomino_EndDateTime(DominoWrapper dw, String in) {

        DateTime dominoDate = null;

        if (in == null) {
            return dominoDate;
        }

        if (log.isLoggable(Level.FINEST)) {
            log.finest("setDominoAllDay_EndDateTime  input " + in);
        }

        try {
            String tmp = in + " 20:00";
            SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd HH:mm");
            Date dtmp = formatter.parse(tmp);
            dominoDate = dw.getDatabase().getParent().createDateTime(dtmp);
        } catch (Exception ne) {
            log.finest("Exception has occurred while formatting end date time for all day event");
        }

        if (log.isLoggable(Level.FINEST)) {
            log.finest("setDominoAllDay_EndDateTime  output " + dominoDate);
        }

        return dominoDate;
    }


    /**
     * we have to create a DateTime with 08:00 PM
     *
     * @param dw DominoWrapper
     * @return DateTime
     */
    public static DateTime setAllDayDomino_EndTime(DominoWrapper dw) {

        DateTime dominoDate = null;

        try {
            dominoDate = dw.getDatabase().getParent().createDateTime("20:00:00 PM");
        } catch (Exception e) {
            log.finest("Exception has occurred while formatting end time for all day event");
        }

        if (log.isLoggable(Level.FINEST)) {
            log.finest("setDominoAllDay_EndTime  output " + dominoDate);
        }

        return dominoDate;
    }


    /**
     * @param conf       String
     * @param itemFormat String
     * @return String
     */
    public static String setClientConfidential(String conf,
                                               String itemFormat) {
        String clientConfidential = null;

        if (conf != null) {
            if (itemFormat.equals(Def.ICAL_TYPE)) {
                // ICAL
                if (conf.equals("")) {
                    clientConfidential = "PUBLIC";
                } else if (conf.equalsIgnoreCase("1")) {
                    clientConfidential = "PRIVATE";
                }
            } else {
                // XML - private
                if (conf.equals("")) {
                    clientConfidential = "0";
                } else if (conf.equalsIgnoreCase("1")) {
                    clientConfidential = "2";
                }
            }
        }
        return clientConfidential;
    }

    /**
     * check if a Document is already present in the Domino Server
     *
     * flag 1=contact; 2=calendar; 3=task; 4=note;
     *
     * @param flag int
     * @param entitymanager Object
     * @param syncItem SyncItem
     *
     * @return operation String
     */
    /*
   public static String checkDocumentPresence(int flag,
                                              Object entitymanager,
                                              SyncItem syncItem){
       String operation = "";

       if      (flag ==1){
          entitymanager = (ContactManager)entitymanager;
       }
       else if (flag == 2){
          entitymanager = (CalendarManager)entitymanager;
       }
       else if (flag == 3){
          entitymanager = (TodoManager)entitymanager;
       }
       else if (flag == 4){
          entitymanager = (NoteManager)entitymanager;
       }


       if (syncItem.getMappedKey() != null) {
           try {
               docToUptdate = entitymanager.getItemByUID(id);
               operation = Def.OPERATION_UPDATE;
           } catch (Exception ex) {
               // if the ID is not in the correct format
               // the Domino server throws an invalid uid exception
               operation = Def.OPERATION_INSERT;
               log.severe("MANAGED ERROR: " + ex.getMessage());
           }
       } else {
           operation = Def.OPERATION_INSERT;
       }

       return operation;

   }
    */

    /**
     * get content from a SyncItem
     */
    public static String getContentFromSyncItem(SyncItem syncItem,
                                                boolean isEncode)
            throws EntityException {

        String content = getContentFromSyncItem(syncItem);
        if (isEncode &&
                content != null &&
                content.length() > 0) {
            content = new String(Base64.decode(content));
        }
        return content;
    }

    /**
     * get content from a SyncItem
     */
    public static String getContentFromSyncItem(SyncItem syncItem)
            throws EntityException {
        String content;

        // get content of the item
        try {
            byte[] itemContent = syncItem.getContent();
            if (itemContent == null) {
                itemContent = new byte[0];
            }
            content = new String(itemContent);

        } catch (Exception e) {
            throw new EntityException("Error getting the item " + syncItem, e);
        }

        return content;
    }

    /**
     * @param dominoWrapper DominoWrapper
     * @param formula       String
     * @return found domino document Document
     */
    public static Document searchDominoDocument(DominoWrapper dominoWrapper,
                                                String formula)
            throws EntityException {
        Document doc = null;
        if (log.isLoggable(Level.FINEST)) {
            log.finest("Formula " + formula);
        }
        try {
            DocumentCollection dc = dominoWrapper.getDatabase().search(formula);

            if (log.isLoggable(Level.FINEST)) {
                log.finest("Found contacts number " + dc.getCount());
            }

            if (dc.getCount() > 0) {
                if (log.isLoggable(Level.FINEST)) {
                    log.finest("Contact already Exist");
                }
                doc = dc.getFirstDocument();
            }
        } catch (Exception e) {
            throw new EntityException(e);
        }

        return doc;

    }

    //------------------------------------------------------------ PRIVATE METHODS


    /**
     * Convert the given date following this roles:
     * <ui>
     * <li>if the given date is in UTC no conversion is required</li>
     * <li>if the given date is not in UTC and the given timezone isn't null,
     * the date is localizated with the timezone and then it's converted in UTC.</li>
     * <li>the returned string is always in UTC</li>
     * <li>if the given timezone is null and the date isn't in UTC, the date is localizated
     * with the default timezone</li>
     * <li>the given date can be in one beetwen the following formats:
     * <code>yyyyMMdd'T'HHmmss'Z',  yyyyMMdd'T'HHmmss</code></li>
     * </ui>
     *
     * @param sDate    the given string date to convert
     * @param timezone TimeZone
     * @return String
     * @throws Exception
     */
    public static String convertLocalDateToUTC(String sDate,
                                               TimeZone timezone)
            throws Exception {

        if (sDate == null || sDate.equals("")) {
            return sDate;
        }

        String utcPattern = "yyyyMMdd'T'HHmmss'Z'";
        DateFormat utcFormatter = new SimpleDateFormat(utcPattern);
        utcFormatter.setTimeZone(TimeZone.getTimeZone("UTC"));

        String noUtcPattern = "yyyyMMdd'T'HHmmss";
        DateFormat noUtcFormatter = new SimpleDateFormat(noUtcPattern);

        if (sDate.indexOf('Z') != -1) {
            //
            // No conversion is required
            //
            return sDate;
        }

        Date date;
        if (timezone != null) {
            noUtcFormatter.setTimeZone(timezone);
        }
        date = noUtcFormatter.parse(sDate);

        return utcFormatter.format(date);
    }

    /**
     * Convert the given date following this roles:
     * <ui>
     * <li>if the given timezone isn't null,
     * the date is localizated with the timezone and then reformatted in
     * <code>yyyyMMdd'T'HHmmss'Z'</code>.</li>
     * <li>the returned string is always in this format <code>yyyyMMdd'T'HHmmss'Z'</code></li>
     * <li>if the given timezone is null the date is not changed</li>
     * </ui>
     *
     * @param sDate    the given string date to convert
     * @param timezone TimeZone
     * @return String
     * @throws Exception
     */
    public static String convertUTCDateToLocal(String sDate,
                                               TimeZone timezone)
            throws Exception {

        if (sDate == null || sDate.equals("")) {
            return sDate;
        }

        if (timezone == null) {
            return sDate;
        }

        String utcPattern = "yyyyMMdd'T'HHmmss'Z'";
        DateFormat utcFormatter = new SimpleDateFormat(utcPattern);
        utcFormatter.setTimeZone(TimeZone.getTimeZone("UTC"));

        Date date;
        date = utcFormatter.parse(sDate);

        utcFormatter.setTimeZone(timezone);

        return utcFormatter.format(date);
    }

    /**
     * Convert the given sDate in iso 8601 format.>>
     * <P>The format accepted are:
     * <ui>
     * <li>yyyyMMdd'T'HHmmss'Z' (if the date is in this format and the given
     * timezone isn't null and useTimeZone is true, the date is localized with
     * the given timezone)</li>
     * <li>yyyy-MM-dd</li>
     * <li>yyyy/MM/dd</li>
     * <li>all formats starts with the previous format (i.e. yyyy-MM-ddTHH:mm:ss.sss)</li>
     * </ui>
     *
     * @param sDate String
     * @return the sDate in iso 8601 format
     */
    public static String normalizeToISO8601(String sDate,
                                            TimeZone tz,
                                            boolean useTimeZone)
            throws Exception {

        if (sDate == null || sDate.equals("")) {
            return sDate;
        }

        if (useTimeZone && tz != null) {
            //
            // Try to apply the timezone
            //
            String utcPattern = "yyyyMMdd'T'HHmmss'Z'";
            SimpleDateFormat utcFormatter = new
                    SimpleDateFormat(utcPattern);
            utcFormatter.setTimeZone(tz);
            Date date;
            try {
                date = utcFormatter.parse(sDate);
                utcFormatter.setTimeZone(TimeZone.getTimeZone("UTC"));
                sDate = utcFormatter.format(date);
            } catch (ParseException ex) {
                //
                // Ignore this error. The date isn't in this format.
                //
            }
        }

        int year;
        int month;
        int day;
        String tmp;
        int last;

        //
        // The first four digits are the year
        //
        tmp = sDate.substring(0, 4);
        year = Integer.parseInt(tmp);

        //
        // Read the month
        //
        char c = sDate.charAt(4);
        if (c == '/' || c == '-') {
            tmp = sDate.substring(5, 7);
            last = 7;
        } else {
            tmp = sDate.substring(4, 6);
            last = 6;
        }
        month = Integer.parseInt(tmp);

        //
        // Read the day
        //
        c = sDate.charAt(last);
        if (c == '/' || c == '-') {
            tmp = sDate.substring(last + 1, last + 3);
        } else {
            tmp = sDate.substring(last, last + 2);
        }
        day = Integer.parseInt(tmp);

        StringBuffer isoDate = new StringBuffer(10);
        isoDate.append(year).append("-");

        if (month < 10) {
            isoDate.append("0");
        }
        isoDate.append(month).append("-");

        if (day < 10) {
            isoDate.append("0");
        }
        isoDate.append(day);
        return isoDate.toString();
    }


    /**
     * if ICAL --> check if the content contains VEVENT or VTODO
     * if XML  --> check if the content contains <calendar> or <task>
     *
     * @param content    String
     * @param itemFormat String
     * @return boolean
     * @throws EntityException
     */
    public static boolean isCalendar(String content, String itemFormat)
            throws EntityException {

        int isevent;
        int istodo;

        // ICAL format
        if (itemFormat.equals(Def.ICAL_TYPE)) {
            isevent = content.indexOf("VEVENT");
            istodo = content.indexOf("VTODO");
        }
        // XML format (in the syncsource set SIF-E)
        else if (itemFormat.equals(Def.SIFE_TYPE)) {
            isevent = content.indexOf("<appointment>"); // SIF-E
            istodo = content.indexOf("<task>"); // SIF-T
        }
        // otherwise error
        else {
            throw new EntityException("Error switching Calendar/Task");
        }

        if (log.isLoggable(Level.FINEST)) {
            log.finest("calendar type " + isevent + " - " + istodo);
        }

        // case ( >0 is 1)
        //      ( -1 is 0)
        //  1 0 calendar
        //  0 1 task
        //  0 0 ERROR
        //  1 1 ERROR

        if (isevent >= 0 && istodo == -1) {
            if (log.isLoggable(Level.FINEST)) {
                log.finest("converting event");
            }
            return true;
        } else if (isevent == -1 && istodo >= 0) {
            if (log.isLoggable(Level.FINEST)) {
                log.finest("converting todo");
            }
            return false;
        } else {
            throw new EntityException("Error switching Calendar/Task");
        }

    }


    /**
     * if ICAL --> check if the content contains VEVENT or VTODO
     * if XML  --> check if the content contains <calendar> or <task>
     *
     * @param content    String
     * @param itemFormat String
     * @return boolean
     * @throws EntityException
     */
    public static boolean isTask(String content, String itemFormat)
            throws EntityException {

        int istodo;

        // ICAL format
        if (itemFormat.equals(Def.ICAL_TYPE)) {
            istodo = content.indexOf("VTODO");
        }
        // XML format (in the syncsource set SIF-E)
        else if (itemFormat.equals(Def.SIFT_TYPE)) {
            istodo = content.indexOf("<task>"); // SIF-T
        }
        // otherwise error
        else {
            throw new EntityException("Error checking if item is a Task");
        }

        if (log.isLoggable(Level.FINEST)) {
            log.finest("item type " + istodo);
        }

        return istodo >= 0;

    }

    /**
     * check the Object: Calendar or TodoCalendar
     *
     * @param obj Object
     * @return boolean
     * @throws EntityException
     */
    /*
   public static boolean isCalendar(ItemCalendar obj)
    throws EntityException {

       CalendarContent objcontent = obj.getCalendarContent();

       String type = objcontent.getFamily();

       if (log.isLoggable(Level.FINEST)) {
           log.finest(" isCalendar(obj) calendar type : " + type);
       }


       // calendar
       if      (type.equalsIgnoreCase("appointment")){
           return true;
       }
       // task
       else if (type.equalsIgnoreCase("task")){
           return false;
       }
       // otherwise error
       else {
           throw new EntityException ("Error switching Calendar/Task");
       }
   }
    */

    /**
     * merge two array
     *
     * @param a SyncItemInfo[]
     * @param b SyncItemInfo[]
     * @return SyncItemInfo[]
     */
    public static SyncItemInfo[] mergeArray(SyncItemInfo[] a,
                                            SyncItemInfo[] b) {
        SyncItemInfo[] syncItemInfo;

        int alength = 0;
        int blength = 0;
        if (a != null) {
            alength = a.length;
        }
        if (b != null) {
            blength = b.length;
        }

        int len = alength + blength;

        syncItemInfo = new SyncItemInfo[len];
        int i;
        for (i = 0; i < alength; i++) {
            syncItemInfo[i] = a[i];
        }
        for (int y = 0; y < blength; y++) {
            syncItemInfo[i + y] = b[y];
        }
        return syncItemInfo;
    }

    /**
     * Checks if the input String is a representation of a date respecting "yyyy-MM-dd" format.
     *
     * @param date
     * @return boolean
     */
    public static boolean isAllDayFormat(String date) {
        SimpleDateFormat format;
        if (date == null) {
            return false;
        }
        format = new SimpleDateFormat("yyyy-MM-dd");
        try {
            format.parse(date);
        } catch (ParseException e) {
            return false;
        }
        return true;
    }

}