/**
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.domino.util;


import com.funambol.domino.items.model.ItemCalendar;
import java.util.logging.Logger;
import java.util.logging.Level;
import java.util.List;

import com.funambol.foundation.pdi.common.XTag;
import com.funambol.framework.logging.Sync4jLogger;

/**
 *
 * <p>Utility class for Calendar Manager</p>
 *
 * @version $Id: UtilityCalendar.java,v 1.1 2006/08/08 08:35:14 gibi Exp $
 */

public class UtilityCalendar {

    // ------------------------------------------------------------ Private data

    /**     */
    private static Logger  log = Sync4jLogger.getLogger(Def.LOGGER_NAME);

    /**
     *
     * in the:
     *  - s60 emulator
     *  - nokia 7650
     * there is a X-TAG ("X-EPOCAGENDAENTRYTYPE")
     * that defines the client calendar type
     */

    private static String calendarTypeXtag = "X-EPOCAGENDAENTRYTYPE";



    // ---------------------------------------------------------- Public Methods


    /**
     *
     * converts client offset alarm in domino offset alarm
     *
     * if client send XML-SIF format
     * clientOffset =
     *
     * if client send ical format
     * clientOffset = 20050204T130000;;0;CalenAlarmSound / null
     *
     * startTime param is start date in yyyymmddTHHMMSS format
     *
     *
     * @param clientOffset String
     * @param startTime String
     * @param itemFormat String
     * @return int
     */
    public static int setDominoAlarmOffset(String clientOffset,
                                           String startTime,
                                           String itemFormat){

        int dominoOffset = 0;
        if (itemFormat.equals(Def.ICAL_TYPE)) {

            //to do ical format

        } else { // XML
            try {

                dominoOffset = Integer.parseInt(clientOffset);
            } catch (NumberFormatException ex) {
                dominoOffset = 0;
            }

        }

        return dominoOffset;
    }



    /**
     *
     * converts client alarm in Domino alarm
     *
     * if XML-SIF format:  clientAlarm = true/false
     * if ical format:     clientAlarm = !null or null
     *
     * @param clientAlarm String
     * @param itemFormat String
     * @return String
     */
    public static String setDominoAlarm(String clientAlarm,
                                        String itemFormat){

        String dominoAlarm = null;

        if (log.isLoggable(Level.FINEST)) {
            log.finest("client alarm value --> " + clientAlarm);
        }


        if (itemFormat.equals(Def.ICAL_TYPE)) {
            if  (clientAlarm != null){
                dominoAlarm = "1";
            }
        } else { // XML
            if  (clientAlarm != null){
                if (clientAlarm.equalsIgnoreCase("true")) {
                    dominoAlarm = "1";
                }
            }
        }

        if (log.isLoggable(Level.FINEST)) {
            log.finest("domino alarm value --> " + dominoAlarm);
        }

        return dominoAlarm;
    }


    /**
     * converts client calendar type into domino calendar type
     *
     * @param c Calendar
     * @param itemFormat String
     *
     * @return String
     */
    public static String setDominoCalendarType(ItemCalendar c,
                                               String itemFormat) {

        String dominoCalendarType = "0";

        if (itemFormat.equals(Def.ICAL_TYPE)) {
            // ICAL
            dominoCalendarType = dominoCalTypeFromICal(c);
        } else {
            // SIF XML
            dominoCalendarType = dominoCalTypeFromXML(c);
        }

        return dominoCalendarType;
    }



    /**
     * converts domino calendar type into client calendar type
     *
     * @param dominoType String
     * @param itemFormat String
     *
     * @return String
     */
    public static String setClientCalendarType(String dominoType,
                                               String itemFormat) {

        String clientCalendarType = "0";

        if (itemFormat.equals(Def.ICAL_TYPE)) {
            clientCalendarType = clientCalTypeFromICal(dominoType);
        } else {
            clientCalendarType = clientCalTypeFromXML(dominoType);
        }

        return clientCalendarType;
    }

    /**
     *
     * @param value String
     * @return XTag
     */
    public static XTag setXTagForCalendar(String value) {
        XTag xtag = new XTag();
        xtag.setXTagValue(calendarTypeXtag);
        xtag.getXTag().setPropertyValue(value);
        return xtag;
    }


    //-------------------------------------------------------- PRIVATE METHODS

    /**
     * converts client calendar type into domino
     * calendar type. Start form ICAL format
     *
     * in the:
     *  - s60 emulator
     *  - nokia 7650
     *  - ..
     *  - @todo: check with other phone mobile
     * there is a X-TAG ("X-EPOCAGENDAENTRYTYPE")
     * that defines the client calendar type
     *
     *
     * client       domino  description
     * ------------------------------------
     * APPOINTMENT  0       Appointment (s60: "meeting" [no attendees])
     * ANNIVERSARY  1       Anniversary (s60: "anniversary")
     * ?            2       AllDayEvent
     * ?            3       Meeting
     * EVENT        4       Reminder    (s60: "day note" [no hours values])
     *
     *
     * @param c Calendar
     * @return String
     */
    private static String dominoCalTypeFromICal(
            ItemCalendar c) {

        String clientCalendarType = "";
        String dominoCalendarType = "0";

        // get calendar type
        XTag xtag = null;
        String label = "";

        // check for all day
        boolean isAllDay = (c.getEvent().getAllDay()).booleanValue();

        if (isAllDay) {
            // all day
            // (the phone doesn't send xtag = ALLDAY
            // but the sync server understands if a
            // vcal is an "all day event" and sets the
            // allDay property
            clientCalendarType = "ALLDAY";
        } else {
            // other cases
            List v = c.getEvent().getXTags();
            int size = 0;
            if (v != null) {
                size = v.size();
            }
            for (int i=0; i<size; i++){
                xtag = (XTag)v.get(i);
                label = xtag.getXTagValue();
                // the Xtag we use is  X-EPOCAGENDAENTRYTYPE
                // @todo check the other phone
                if (label.equalsIgnoreCase(calendarTypeXtag)){
                    clientCalendarType = (String)xtag.getXTag().getPropertyValue();
                }
            }
        }


        if (log.isLoggable(Level.FINEST)) {
            log.finest("client calendar type --> " + clientCalendarType);
        }

        // convert client type to server type
        if (clientCalendarType.equalsIgnoreCase("APPOINTMENT")){
            dominoCalendarType = "0";
        } else if (clientCalendarType.equalsIgnoreCase("ANNIVERSARY")){
            dominoCalendarType = "1";
        } else if (clientCalendarType.equalsIgnoreCase("EVENT")){
            dominoCalendarType = "4";
        } else if (clientCalendarType.equalsIgnoreCase("ALLDAY")){
            dominoCalendarType = "2";
        }

        if (log.isLoggable(Level.FINEST)) {
            log.finest("domino calendar type --> " + dominoCalendarType);
        }

        return dominoCalendarType;
    }

    /**
     *
     * @param dominoType String
     * @return String
     */
    private static String clientCalTypeFromICal(String dominoType) {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("domino calendar type --> " + dominoType);
        }

        String clientCalendarType = "";

        if        (dominoType.equalsIgnoreCase("0")){
            clientCalendarType = "APPOINTMENT";
        } else if (dominoType.equalsIgnoreCase("1")){
            clientCalendarType = "ANNIVERSARY";
        } else if (dominoType.equalsIgnoreCase("4")){
            clientCalendarType = "EVENT";
        } else if (dominoType.equalsIgnoreCase("2")){
            // @todo
            //clientCalendarType = "ALLDAY";
            clientCalendarType = "APPOINTMENT";
        }

        if (log.isLoggable(Level.FINEST)) {
            log.finest("client calendar type --> " + clientCalendarType);
        }
        return clientCalendarType;
    }


    /**
     * converts client calendar type into domino calendar type.
     * Starts from XML-SIF
     *
     * The XML SIF for event (SIF-E) contains only
     * - appointment
     * - metting
     * the difference is the number of attendees.
     *
     * number attendees = 0 --> appointment
     * numner attendees > 0 --> meeting
     *
     *
     * @param c Calendar
     * @return String
     */
    private static String dominoCalTypeFromXML(
            ItemCalendar c) {

        String dominoCalendarType = "0";
        int attendessNum = 0;

        boolean isAllDay = false;

        try {
            isAllDay = c.getEvent().getAllDay().booleanValue();
        } catch(NullPointerException ex) {
            isAllDay = false;
        }


        if (isAllDay){
            // all day event
            dominoCalendarType = "2";
        } else {
            // @todo get attendees from SIF-E
            if (attendessNum == 0) {
                // appointment
                dominoCalendarType = "0";
            } else {
                // meeting
                dominoCalendarType = "3";
            }
        }

        return dominoCalendarType;
    }

    /**
     *
     * @param dominoType String
     * @return String
     */
    private static String clientCalTypeFromXML(String dominoType) {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("domino calendar type --> " + dominoType);
        }

        String clientCalendarType = "";

        if        (dominoType.equalsIgnoreCase("0")){
            clientCalendarType = "0";
        } else if (dominoType.equalsIgnoreCase("1")){
            clientCalendarType = "0";
        } else if (dominoType.equalsIgnoreCase("4")){
            clientCalendarType = "0";
        } else if (dominoType.equalsIgnoreCase("2")){
            clientCalendarType = "ALLDAY";
        }

        if (log.isLoggable(Level.FINEST)) {
            log.finest("client calendar type --> " + clientCalendarType);
        }

        return clientCalendarType;
    }


}