/**
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.domino.util;

import com.funambol.domino.items.model.ItemContact;

import java.util.Iterator;
import java.util.logging.Logger;
import java.util.List;
import java.util.ArrayList;

import com.funambol.foundation.pdi.contact.BusinessDetail;
import com.funambol.foundation.pdi.contact.PersonalDetail;
import com.funambol.foundation.pdi.contact.Title;
import com.funambol.foundation.pdi.contact.Phone;
import com.funambol.foundation.pdi.contact.Email;
import com.funambol.foundation.pdi.contact.WebPage;
import com.funambol.framework.logging.Sync4jLogger;


/**
 * <p>Utility class for Contact Manger</p>
 *
 * @version $Id: UtilityContact.java,v 1.1 2006/08/08 08:35:14 gibi Exp $
 */

public class UtilityContact {

    // ------------------------------------------------------------ Private data

    /**    */
    private static Logger log = Sync4jLogger.getLogger(Def.LOGGER_NAME);

    // ---------------------------------------------------------- Public Methods

    /**
     * returns the business email from
     * foundation contact object
     * The "email work"
     * is in the objcet:  BusinessDetail
     * it has the label:  Email3Address
     *
     * @param contact Contact
     * @return String
     */
    public static String[] getEmailsOfTypeFromFoundation(ItemContact contact, String type) {

        Email email;
        List emails, emailsOfType;
        Iterator interator;

        if (contact.getBusinessDetail() == null) {
            return new String[0];
        }

        if (Def.EMAIL_3_ADDRESS.equalsIgnoreCase(type) ||
                Def.BUSINESS_EMAIL_2_ADDRESS.equalsIgnoreCase(type) ||
                Def.BUSINESS_EMAIL_3_ADDRESS.equalsIgnoreCase(type)) {
            emails = contact.getBusinessDetail().getEmails();
        } else {
            if (Def.PERSONAL_EMAIL_1_ADDRESS.equalsIgnoreCase(type) ||
                    Def.PERSONAL_EMAIL_2_ADDRESS.equalsIgnoreCase(type)) {
                emails = contact.getBusinessDetail().getEmails();
            } else {
                return new String[0];
            }
        }

        emailsOfType = new ArrayList();

        if (emails == null) {
            return new String[0];
        }

        interator = emails.iterator();
        while (interator.hasNext()) {
            email = (Email) interator.next();
            if (email.getEmailType().equalsIgnoreCase(type)) {
                emailsOfType.add(email.getPropertyValueAsString());
            }
        }

        return (String[]) emailsOfType.toArray(new String[0]);
    }

    /**
     * returns the business phone from
     * foundation contact object
     * <ul>
     * <li>1 BusinessTelephoneNumber</li>
     * <li>2 Business2TelephoneNumber</li>
     * <li>3 BusinessFaxNumber</li>
     * </ul>
     *
     * @param contact Contact
     * @param type_e  int
     * @return String
     */
    public static String getBusinessPhoneFromFoundation(
            ItemContact contact,
            int type_e) {

        String phone = null;
        Phone bPhone = null;

        String type = Def.BUSINESS_PHONE_NUM;
        if (type_e == 1) {
            type = Def.BUSINESS_PHONE_NUM;
        } else if (type_e == 2) {
            type = Def.BUSINESS_2_PHONE_NUM;
        } else if (type_e == 3) {
            type = Def.BUSINESS_FAX_NUM;
        }

        BusinessDetail businessDetail = contact.getBusinessDetail();

        List businessPhones = businessDetail.getPhones();
        int size = 0;
        if (businessPhones != null) {
            size = businessPhones.size();
        }
        for (int i = 0; i < size; i++) {
            bPhone = (Phone) businessPhones.get(i);
            if (bPhone.getPhoneType().equalsIgnoreCase(type)) {
                phone = (String) bPhone.getPropertyValue();
                break;
            }
        }

        return phone;
    }

    public static String getBusinessPhoneFromFoundation(ItemContact contact,
                                                        String type) {

        List phones = contact.getBusinessDetail().getPhones();
        if (phones != null) {

            for (Iterator it = phones.iterator(); it.hasNext();) {
                Phone elem = (Phone) it.next();
                if (elem.getPhoneType().equalsIgnoreCase(type))
                    return (String) elem.getPropertyValue();
            }

        }

        return null;
    }

    public static String getPersonalPhoneFromFoundation(ItemContact contact,
                                                        String type) {

        List phones = contact.getPersonalDetail().getPhones();
        if (phones != null) {

            for (Iterator it = phones.iterator(); it.hasNext();) {
                Phone elem = (Phone) it.next();
                if (elem.getPhoneType().equalsIgnoreCase(type))
                    return (String) elem.getPropertyValue();
            }

        }
        return null;
    }

    /**
     * returns the personal phone from
     * foundation contact object
     * <ul>
     * <li>1 OtherTelephoneNumber</li>
     * <li>2 MobileTelephoneNumber</li>
     * <li>3 HomeTelephoneNumber</li>
     * <li>4 Home2TelephoneNumber</li>
     * <li>5 HomeFaxNumber</li>
     * </ul>
     *
     * @param contact Contact
     * @param type_e  int
     * @return String
     */
    public static String getPersonalPhoneFromFoundation(
            ItemContact contact,
            int type_e) {

        String phone = null;
        Phone pphone = null;

        String type = Def.PERSONAL_OTHER_PHONE_NUM;
        if (type_e == 1) {
            type = Def.PERSONAL_OTHER_PHONE_NUM;
        } else if (type_e == 2) {
            type = Def.PERSONAL_MOBILE_PHONE_NUM;
        } else if (type_e == 3) {
            type = Def.PERSONAL_HOME_PHONE_NUM;
        } else if (type_e == 4) {
            type = Def.PERSONAL_HOME_2_PHONE_NUM;
        } else if (type_e == 5) {
            type = Def.PERSONAL_HOME_FAX_NUMBER;
        }

        PersonalDetail personalDetail = contact.getPersonalDetail();

        List personalPhones = personalDetail.getPhones();
        int size = 0;
        if (personalPhones != null) {
            size = personalPhones.size();
        }

        for (int i = 0; i < size; i++) {
            pphone = (Phone) personalPhones.get(i);
            if (pphone.getPhoneType().equalsIgnoreCase(type)) {
                phone = (String) pphone.getPropertyValue();
                break;
            }
        }

        return phone;
    }


    /**
     * returns the personal email from
     * foundation contact object
     * <ul>
     * <li>1 Email1Address - Home</li>
     * <li>2 Email2Address - Other</li>
     * </ul>
     *
     * @param contact Contact
     * @param type_e  int
     * @return String
     */
    public static String getPersonalEmailFromFoundation(
            ItemContact contact,
            int type_e) {

        String email = null;
        Email pemail = null;

        String type = Def.PERSONAL_EMAIL_1_ADDRESS;
        if (type_e == 1) {
            type = Def.PERSONAL_EMAIL_1_ADDRESS;
        } else if (type_e == 2) {
            type = Def.PERSONAL_EMAIL_2_ADDRESS;
        }

        PersonalDetail pd = contact.getPersonalDetail();

        List personalEmails = pd.getEmails();
        int size = 0;
        if (personalEmails != null) {
            size = personalEmails.size();
        }

        for (int i = 0; i < size; i++) {
            pemail = (Email) personalEmails.get(i);
            if (pemail.getEmailType().equalsIgnoreCase(type)) {
                email = (String) pemail.getPropertyValue();
                break;
            }
        }

        return email;
    }


    /**
     * returns the title from
     * foundation contact object
     * <ul>
     * <li>1 JobTitle</li>
     * </ul>
     *
     * @param contact Contact
     * @param type_e  int
     * @return String
     */
    public static String getBusinessTitleFromFoundation(
            ItemContact contact,
            int type_e) {

        String title = "";
        Title btitle = null;

        String type = Def.BUSINESS_TITLE;
        if (type_e == 1) {
            type = Def.BUSINESS_TITLE;
        }

        BusinessDetail pd = contact.getBusinessDetail();

        List businessTitles = pd.getTitles();
        int size = 0;
        if (businessTitles != null) {
            size = businessTitles.size();
        }

        for (int i = 0; i < size; i++) {
            btitle = (Title) businessTitles.get(i);
            if (btitle.getTitleType().equalsIgnoreCase(type)) {
                title = (String) btitle.getPropertyValue();
                break;
            }
        }

        return title;
    }

    /**
     * set the title in the foundation contact
     * object.
     *
     * @param contact  Contact
     * @param newValue String
     * @param type_e   int
     */
    public static void setBusinessTitleInFoundation(
            ItemContact contact,
            String newValue,
            int type_e) {

        String type = Def.BUSINESS_TITLE;
        if (type_e == 1) {
            type = Def.BUSINESS_TITLE;
        }

        Title btitle = new Title();

        btitle.setTitleType(type);
        btitle.setPropertyValue(newValue);

        contact.getBusinessDetail().addTitle(btitle);
    }


    /**
     * set an email of a type in the foundation
     * contact object.
     *
     * @param contact  Contact
     * @param type     String
     * @param newValue String
     */
    public static void addEmailOfTypeInFoundation(
            ItemContact contact,
            String type,
            String newValue) {

        Email email = new Email();

        email.setEmailType(type);
        email.setPropertyValue(newValue);

        if (Def.EMAIL_3_ADDRESS.equalsIgnoreCase(type) ||
                Def.BUSINESS_EMAIL_2_ADDRESS.equalsIgnoreCase(type) ||
                Def.BUSINESS_EMAIL_3_ADDRESS.equalsIgnoreCase(type)) {
            BusinessDetail bd = contact.getBusinessDetail();
            if (bd != null) {
                if (bd.getEmails() != null) {
                    bd.getEmails().add(email);
                } else {
                    bd.addEmail(email);
                    ((Email) bd.getEmails().get(0)).setEmailType(type);
                }
            }

        } else {
            if (Def.PERSONAL_EMAIL_1_ADDRESS.equalsIgnoreCase(type) ||
                    Def.PERSONAL_EMAIL_2_ADDRESS.equalsIgnoreCase(type)) {
                PersonalDetail pd = contact.getPersonalDetail();
                if (pd != null) {
                    if (pd.getEmails() != null) {
                        pd.getEmails().add(email);
                    } else {
                        pd.addEmail(email);
                        ((Email) pd.getEmails().get(0)).setEmailType(type);
                    }
                }
            }
        }
    }

    /**
     * set the business email in the foundation
     * contact object.
     *
     * @param contact  Contact
     * @param newValue String
     */
    public static void setEmail1InFoundation(
            ItemContact contact,
            String newValue) {

        Email pemail = new Email();

        pemail.setEmailType(Def.PERSONAL_EMAIL_1_ADDRESS);
        pemail.setPropertyValue(newValue);

        contact.getPersonalDetail().addEmail(pemail);

    }

    /**
     * set the business email in the foundation
     * contact object.
     *
     * @param contact  Contact
     * @param newValue String
     */
    public static void setEmail2InFoundation(
            ItemContact contact,
            String newValue) {

        Email pemail = new Email();

        pemail.setEmailType(Def.PERSONAL_EMAIL_2_ADDRESS);
        pemail.setPropertyValue(newValue);

        contact.getPersonalDetail().addEmail(pemail);
    }

    /**
     * sets the business phone in the
     * foundation contact object
     * <p/>
     * <ul>
     * <li>1 BusinessTelephoneNumber</li>
     * <li>2 Business2TelephoneNumber</li>
     * <li>3 BusinessFaxNumber</li>
     * </ul>
     *
     * @param contact  Contact
     * @param newValue String
     * @param type_e   int
     */
    public static void setBusinessPhoneInFoundation(
            ItemContact contact,
            String newValue,
            int type_e) {

        String type = Def.BUSINESS_PHONE_NUM;
        if (type_e == 1) {
            type = Def.BUSINESS_PHONE_NUM;
        } else if (type_e == 2) {
            type = Def.BUSINESS_2_PHONE_NUM;
        } else if (type_e == 3) {
            type = Def.BUSINESS_FAX_NUM;
        }

        Phone bphone = new Phone();

        bphone.setPhoneType(type);
        bphone.setPropertyValue(newValue);

        contact.getBusinessDetail().addPhone(bphone);
    }

    public static void setBusinessPhoneInFoundation(ItemContact contact,
                                                    String value, String type) {


        Phone bphone = new Phone();

        bphone.setPhoneType(type);
        bphone.setPropertyValue(value);

        contact.getBusinessDetail().addPhone(bphone);
    }

    public static void setPersonalPhoneInFoundation(ItemContact contact,
                                                    String value, String type) {


        Phone bphone = new Phone();

        bphone.setPhoneType(type);
        bphone.setPropertyValue(value);

        contact.getPersonalDetail().addPhone(bphone);
    }

    /**
     * set the web page in the
     * foundation contact object
     *
     * @param contact  Contact
     * @param newValue String
     */
    public static void setPersonalWebPageInFoundation(
            ItemContact contact,
            String newValue) {


        WebPage bwebsite = new WebPage();

        bwebsite.setWebPageType(Def.PERSONAL_WEBPAGE);
        bwebsite.setPropertyValue(newValue);

        contact.getPersonalDetail().addWebPage(bwebsite);
    }

    /**
     * sets the personal phone in the
     * foundation contact object
     * <p/>
     * <ul>
     * <li>1 OtherTelephoneNumber</li>
     * <li>2 MobileTelephoneNumber</li>
     * <li>3 HomeTelephoneNumber</li>
     * <li>4 Home2TelephoneNumber</li>
     * <li>5 HomeFaxNumber</li>
     * </ul>
     *
     * @param contact  Contact
     * @param newValue String
     * @param type_e   int
     */
    public static void setPersonalPhoneInFoundation(
            ItemContact contact,
            String newValue,
            int type_e) {


        String type = Def.PERSONAL_OTHER_PHONE_NUM;
        if (type_e == 1) {
            type = Def.PERSONAL_OTHER_PHONE_NUM;
        } else if (type_e == 2) {
            type = Def.PERSONAL_MOBILE_PHONE_NUM;
        } else if (type_e == 3) {
            type = Def.PERSONAL_HOME_PHONE_NUM;
        } else if (type_e == 4) {
            type = Def.PERSONAL_HOME_2_PHONE_NUM;
        } else if (type_e == 5) {
            type = Def.PERSONAL_HOME_FAX_NUMBER;
        }

        Phone pphone = new Phone();

        pphone.setPhoneType(type);
        pphone.setPropertyValue(newValue);

        contact.getPersonalDetail().addPhone(pphone);
    }

    /**
     * <p>Get the first Personal WebPage from a foundation contact.</p>
     *
     * @param contact Contact
     * @return String
     */
    public static String getPersonalWebPageFromFoundation(ItemContact contact) {
        WebPage[] personalWebPages;
        List webPages;
        webPages = contact.getPersonalDetail().getWebPages();
        if (webPages != null) {
            personalWebPages = extractWebPagesOfType(webPages, Def.PERSONAL_WEBPAGE);
            if (personalWebPages.length > 0) {
                return personalWebPages[0].getPropertyValueAsString();
            }
        }
        return "";
    }

    /**
     * <p>Get the WebPages oa a type from a foundation contact.</p>
     *
     * @param contact Contact
     * @param types   String[]
     * @return String
     */
    public static String[] getWebPagesFromFoundation(ItemContact contact, String[] types) {
        WebPage webPage;
        List webPages, webPagesOfType;
        Iterator interator;

        if (contact.getBusinessDetail() == null) {
            return new String[0];
        }

        webPagesOfType = new ArrayList();

        for (int i = 0; types != null && i < types.length; i++) {
            if (Def.BUSINESS_WEBPAGE.equalsIgnoreCase(types[i]) ||
                    Def.BUSINESS_2_WEBPAGE.equalsIgnoreCase(types[i]) ||
                    Def.BUSINESS_3_WEBPAGE.equalsIgnoreCase(types[i])) {
                webPages = contact.getBusinessDetail().getWebPages();
            } else {
                if (Def.PERSONAL_WEBPAGE.equalsIgnoreCase(types[i]) ||
                        Def.OTHER_WEBPAGE.equalsIgnoreCase(types[i])) {
                    webPages = contact.getBusinessDetail().getWebPages();
                } else {
                    continue;
                }
            }
            if (webPages == null) {
                continue;
            }
            interator = webPages.iterator();
            while (interator.hasNext()) {
                webPage = (WebPage) interator.next();
                if (webPage.getWebPageType().equalsIgnoreCase(types[i])) {
                    webPagesOfType.add(webPage.getPropertyValueAsString());
                }
            }
        }
        return (String[]) webPagesOfType.toArray(new String[0]);
    }

    /**
     * <p>Get the first Other WebPage from a foundation contact.</p>
     *
     * @param contact Contact
     * @return String
     */
    public static String getOtherWebPageFromFoundation(ItemContact contact) {
        WebPage[] personalWebPages;
        List webPages;
        webPages = contact.getPersonalDetail().getWebPages();
        if (webPages != null) {
            personalWebPages = extractWebPagesOfType(webPages, Def.OTHER_WEBPAGE);
            if (personalWebPages.length > 0) {
                return personalWebPages[0].getPropertyValueAsString();
            }
        }
        return "";
    }

    /**
     * <p>Extract the webpages  od the input type from the input list.</p>
     *
     * @param webPages the input list
     * @param type     the input type
     * @return an array of WebPages
     */
    private static WebPage[] extractWebPagesOfType(List webPages, String type) {
        WebPage webPage;
        String webPageType;
        Iterator iter;
        List toReturn;

        iter = webPages.iterator();
        toReturn = new ArrayList();

        while (iter.hasNext()) {
            webPage = (WebPage) iter.next();
            webPageType = webPage.getWebPageType();
            if (type.equals(webPageType)) {
                toReturn.add(webPage);
            }
        }
        return (WebPage[]) toReturn.toArray(new WebPage[0]);
    }

    /**
     * Add web page in of type in foundation.
     *
     * @param contact  Contact
     * @param type     String
     * @param newValue String
     */
    public static void addWebPageInFoundation(
            ItemContact contact,
            String type,
            String newValue) {

        WebPage webPage = new WebPage();

        webPage.setWebPageType(type);
        webPage.setPropertyValue(newValue);

        if (Def.BUSINESS_WEBPAGE.equalsIgnoreCase(type) ||
                Def.BUSINESS_2_WEBPAGE.equalsIgnoreCase(type) ||
                Def.BUSINESS_3_WEBPAGE.equalsIgnoreCase(type)) {
            BusinessDetail bd = contact.getBusinessDetail();
            if (bd != null) {
                if (bd.getWebPages() != null) {
                    bd.getWebPages().add(webPage);
                } else {
                    bd.addWebPage(webPage);
                    ((WebPage) bd.getWebPages().get(0)).setWebPageType(type);
                }
            }

        } else {
            if (Def.PERSONAL_WEBPAGE.equalsIgnoreCase(type) ||
                    Def.OTHER_WEBPAGE.equalsIgnoreCase(type)) {
                PersonalDetail pd = contact.getPersonalDetail();
                if (pd != null) {
                    if (pd.getWebPages() != null) {
                        pd.getWebPages().add(webPage);
                    } else {
                        pd.addWebPage(webPage);
                        ((WebPage) pd.getWebPages().get(0)).setWebPageType(type);
                    }
                }
            }
        }
    }
}