/**
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.domino.xml;

import java.util.ArrayList;
import java.util.StringTokenizer;

/**
 * This class provides commons methods to parser XML msg
 *
 * @version $Id: XmlParser.java,v 1.1 2006/08/08 08:35:34 gibi Exp $
 **/
public class XmlParser {

    //---------------------------------------------------------------- Constants

    private static final String HEADER_REPLID_START    = "<rid:"    ;

    //----------------------------------------------------------- Public methods

    /**
     * Make a String[] by tags find with search.
     *
     * @param xmlInput tags about search
     * @param tagContent tag to find
     * @return find tags
     **/
    public static String[] getXMLTag(String[] xmlInput, String tagContent)
    throws XmlParseException {

        ArrayList rows     = null;

        String xmlInputTag = null ;
        String startTag    = null ;
        String endTag      = null ;

        int i = 0;

        startTag = "<"  + tagContent + ">" ;
        endTag   = "</" + tagContent + ">" ;

        rows = new ArrayList();

        try {

            for (int j=0, l = xmlInput.length; j < l; j++) {

                xmlInputTag = xmlInput[j];

                while (xmlInputTag.indexOf(startTag) != -1){
                    rows.add
                        (xmlInputTag.substring(xmlInputTag.indexOf(startTag)         +
                         startTag.length(), xmlInputTag.indexOf(endTag)))            ;
                    xmlInputTag = xmlInputTag.substring(xmlInputTag.indexOf(endTag)  +
                        endTag.length())                                             ;
                    i++;
                }

            }

        } catch (java.lang.StringIndexOutOfBoundsException e) {
            throw new XmlParseException(e.toString() + " : " + tagContent + " TAG");
        }

        return (String[])rows.toArray(new String[i]);

    }

    /**
     * Make a String by value of <i>tag</i>.
     *
     * @param xml xml msg
     * @param tag tag to find
     * @return tag value
     **/
    public static String getXMLTagValue(String xml, String tag)
    throws XmlParseException {

        String msg       = null ;
        String startTag  = null ;
        String endTag    = null ;

        String voidTag   = null ;

        voidTag  = "<"  + tag + "/>" ;

        startTag = "<"  + tag + ">" ;
        endTag   = "</" + tag + ">" ;


        if (xml.indexOf(voidTag) != -1 || xml.indexOf(startTag) == -1) {
            return null;
        }

        try {
            msg = xml.substring(xml.indexOf(startTag)  +
                                startTag.length()      ,
                                xml.indexOf(endTag))   ;
        } catch (java.lang.StringIndexOutOfBoundsException e) {
            throw new XmlParseException(e.toString() + " : " + tag + " TAG");
        }

        return msg;

    }

    /**
     * Make a String by value of <i>tag</i>.
     *
     * @param xml xml msg
     * @param tag tag to find
     * @return tag value
     **/
    public static String getXMLInitTagValue(String xml, String tag)
    throws XmlParseException {

        String msg       = null ;
        String startTag  = null ;
        String endTag    = null ;
        String voidTag   = null ;

        int begin        = 0;
        int init         = 0;

        voidTag  = "<"  + tag + "/>" ;
        startTag = "<"  + tag        ;
        endTag   = "</" + tag + ">"  ;

        if (xml.indexOf(voidTag) != -1 || xml.indexOf(startTag) == -1) {
            return null;
        }

        try {

            begin = xml.indexOf(startTag)                   ;

            init  = xml.indexOf(">", begin) + 1             ;

            //
            // tag void
            //
            if (xml.charAt(init) == '<' && xml.charAt(init + 1) == "/".charAt(0)) {
                return "";
            }

            msg = xml.substring(init, xml.indexOf(endTag))  ;

        } catch (java.lang.StringIndexOutOfBoundsException e) {
            throw new XmlParseException(e.toString() + " : " + tag + " TAG");
        }

        return msg;
    }


   /**
    * Make a href (reource name) from uri webdavtag.
    *
    * @param uri
    * @return id
    * @throws XmlParseException
    **/
   public static String getHrefFromUri(String uri)
    throws XmlParseException {

        String href = null;

        try {

            href = uri.substring(uri.lastIndexOf("/") + 1);

        } catch (java.lang.StringIndexOutOfBoundsException e) {
            throw new XmlParseException(e.toString() + " URI");
        }

        return href;

    }

    /**
     * Make a ruid from a response header.
     *
     * @param response
     * @return ruid
     * @throws XmlParseException
     **/
    public static String getRuidFromResponse(String response)
    throws XmlParseException {

        String ruid = null;

        try {

            int start = response.indexOf(HEADER_REPLID_START)          +
                                         HEADER_REPLID_START.length()  ;

            int end   = response.indexOf(">", start)                   ;

            ruid      = response.substring(start, end)                 ;

        } catch (java.lang.StringIndexOutOfBoundsException e) {
            throw new XmlParseException(e.toString() + " URI");
        }

        return ruid;

    }

    /**
     * @param value
     * @return <p>value<p> if value is not null, else ""
     **/
    public static String getFieldFromMultipleTag (String value)
    throws Exception {

        String []    fields = null;
        StringBuffer field  = new StringBuffer();

        fields = getXMLTag(new String[] {value}, "c:v");

        for (int i = 0, l = fields.length; i < l; i++) {

            if (i > 0) {
                field.append("; ");
            }

            field.append(fields[i]);

        }

         return field.toString();

    }

    /**
     * @param categories The categories value
     * @return <p>value<p> if value is not null, else ""
     **/
    public static String fieldToMultipleTag (String categories) {

        StringBuffer fields = new StringBuffer();

        StringTokenizer st = new StringTokenizer(categories, ";");

        while (st.hasMoreTokens()) {
            fields.append("<x:v>")
                  .append(st.nextToken().trim())
                  .append("</x:v>\n");
        }

        return fields.toString();
    }

    /**
     * @param value
     * @return <p>value<p> if value is not null, else ""
     **/
    public static String getString (String value) {

        if (value == null) {
            return "";
        } else  {
            return value;
        }

    }

}
